c This version has both decomposition parameters and rotation
c angle incorporated into it. The Ztrue is first 
c multiplied by the decomp matrix, then rotated
c if penalties are to be put on any parameters this should be
c done in the prejudice file.
c
c march 29, 1999 - alteration to allow for varying strike angles
c of input data, allows 1 strike value per datum(dp(:,3))
c July 25, 2001 - alteration to fordiv, PW's new adjoint method
c used, which returns derivatives with respect to log(resistivity)
c Nov 15, 2001 - alteration to compute (del)t(del) much faster
c Feb, 2002 - Occam inversion for ln(boundary depths) and log10(resistivity)
c		of large units (<=10)
C-----------------------------------------------------------------------
      program runocc
C-----------------------------------------------------------------------
c
C OCCAM 2.0: Steven Constable IGPP/SIO La Jolla CA 92093-0225
c Program Revision 2.01, 13 Jan 1993
c
c runocc is the calling program for OCCAM v2.0. 
C
C REFERENCES: CONSTABLE, PARKER & CONSTABLE, 1987: GEOPHYSICS 52, 289-300.
C             DEGROOT-HEDLIN & CONSTABLE, 1990: GEOPHYSICS 55, 1613-1624.
C             CONSTABLE, 1991: GEOPHYS. J. INT. 106, 387-388.
C             CONSTABLE, 1992: OCCAM DISTRIBUTION NOTES.
C
C REVISION HISTORY:
C     MARCH 1986,                                         (VERSION 1.2)
C     OCTOBER 1987,                                       (VERSION 1.3)
C     AUGUST 1988,                                        (VERSION 1.4)
C     JANUARY 1989 (WITH THANKS TO C.DEGROOT-HEDLIN),     (VERSION 1.5)
C     SEPTEMBER 1989,                                     (VERSION 1.5.2)
C     APRIL 1992 (MORE THANKS TO CATHERINE AND ALSO WSE)  (VERSION 2.0)
c     July 1994  includes decomposition (Catherine)
C
C IF YOU OBTAIN THIS CODE FROM A THIRD PERSON, PLEASE SEND YOUR NAME AND ADDRESS
C TO S. CONSTABLE.  YOU WILL THEN RECEIVE UPDATES, NEWS ON BUGS, ETC.
C
C VERSION 2.0 IS A RE-WRITE TO MAKE OCCAM INDEPENDENT OF THE DIMENSIONALITY
C OF THE FORWARD PROBLEM, AND TO MAKE MAXIMUM USE OF DYNAMIC MEMORY ALLOCATION, 
C AS THE ARRAY SIZES FOR THE 2D PROBLEM ARE NOW GETTING TOO LARGE FOR COMFORT.
C INCLUDE FILES ARE NOW USED TO STREAMLINE CHANGES OF DIMENSION.
C IMPLICIT DOUBLE PRECISION HAS BEEN REMOVED, AGAIN FOR STORAGE REASONS.
C CALLING PROGRAM IS NOW AN INTEGRAL PART OF THE PACKAGE AS ALL THE MODEL-
C DEPENDENT STUFF HAS BEEN SPUN OFF.
C
C IF YOU OBTAIN THIS CODE FROM A THIRD PERSON, PLEASE SEND YOUR NAME AND ADDRESS
C TO S. CONSTABLE.  YOU WILL THEN RECEIVE UPDATES, NEWS ON BUGS, ETC.
C
C SUBROUTINES WHICH MUST BE SUPPLIED BY THE USER:
C
C  FORMOD(NP,ND,PM,DP,DM), COMPUTES THE FORWARD FUNCTION FOR MODEL PM() AT
C    THE DATA PARAMETERS DP() AND RETURNS IT IN DM().
C  FORDIV(NP,ND,PM,DP,DM,WJ), COMPUTES THE FORWARD FUNCTION AS DOES FORMOD()
C    BUT ALSO RETURNS THE MATRIX OF PARTIALS, WJ(ND,NP).
C  OBJMAT(IRUF, DEL, NPEN, linkpr), ASSEMBLES THE PENALTY MATRIX (THIS WILL BE 
C  DEPENDENT ON MODEL TYPE AND DIMENSION).  
c  DATIME(DATETM), RETURNS AN 80 CHARACTER STRING WITH CURRENT DATE AND TIME.  CAN 
C    BE A DUMMY ROUTINE IF SYSTEM PROGRAMMING IS TOO MUCH.
C  INPUTD(FILENAME), TAKES THE NAMED FILE AND USES IT TO GET THE DATA IMAGE INTO 
C    THE COMMON BLOCK /DATA/.  SEE OCCAM HEADER.
C  INPUTM(FILENAME), TAKES THE NAMED FILE AND READS IN ANY FIXED ATTRIBUTES
C    OF THE MODEL, SUCH AS LAYER THICKNESSES, FINITE ELEMENT MESH, ETC.  COMMUNICATION
C    BETWEEN INPUTM() AND FORMOD() AND FORDIV() MUST BE VIA THE USER'S OWN COMMON
C    BLOCKS.  WITH STATIC COMMONS (CHECK YOU COMPILER!) THESE NEED NOT BE DECLARED
C    IN THE CALLING PROGRAMS.  
c
c Files to be supplied by the user:
c
c  STARTUP must be supplied, in the correct format, to provide inversion parameters.
c  data and model files pointed at by STARTUP
c
c Files generated by OCCAM:
c
c  LOGFIL provides a blow by blow account of the inversion
c  ITERxx provides the inversion parameters, including the model, at every iteration.
c     ITER00 is a copy of the startup file at iteration 0.  Any ITER file can be re-
c     named startup to continue an inversion.
c
c As the response of the model in dm() is not necessarily the response of the final
c model found, but rather the response of the last model tested in the univariate
c optimization routines, we leave it to the user to find the response of
c the final model, using low cunning or the program 'FindResponse.f'.
c
c includes:
      include 'imp.inc'
      include 'occamdim.inc'
c   uses npp
      include 'occam.inc'
c   uses np, pm(), idebug, iout
c
c local variables:
      integer lerr, maxitr, iruf, nit, itmax, konv, ifftol
c   lerr = error flag for file operations
c   maxitr = maximum number of subsequent iterations by this run
c   iruf = roughness measure, 0 (size penalty), 1 (1st deriv.), 2 (2nd deriv.)
c   nit = current iteration number (0=start)
c   i = loop index
c   itmax = maximum iteration number
c   konv = convergence flag from OCCAM
c   ifftol = convergence info from OCCAM (has the desired misfit been achieved?)
      real tolreq, tobt, pmu, stepsz, rlast
c   tolreq = desired misfit, in RMS standard errors
c   tobt = current misfit
c   pmu = current Lagrange multiplier
c   stepsz = current RMS step though model space
c   rlast = last RMS measure of roughness
      character*50 itform, descr, modfil, datfil
c   itform = format number of iteration files
c   descr = description line in iteration files
c   modfil = model file name
c   datfil = data file name
c
c parameters
      integer iof1, iof2
      parameter (iof1 = 13, iof2 = 15)
c   iof1 = unit number for the LOGFIL (passed to /result/iout)
c   iof2 = unit number for reading and writing ITER files
c
c calls:
c     occam, filerr, itrout, itrin, inputm, inputd
c      
C-----------------------------------------------------------------------
c Open startup file and read stuff.
c (Startup and iteration files should be machine written, and so we do
c not necessarily test the validity of the input data unless arrays are 
c involved (np and pm)).
c
      open (iof2, file='startup', status='old', iostat=lerr)
      if (lerr .ne. 0) then
        write(*,*) ' Error opening startup file'
        stop
      end if
      call itrin (itform,descr,modfil,datfil,maxitr,tolreq,iruf,
     *    nit,pmu,rlast,tobt,ifftol,iof2)      
      close(iof2)
c            
c if this is zeroth iteration copy the startup file into ITER00
      if (nit .eq. 0) then
c initialize the inversion parameters
        pmu = 4.0
        rlast = 1.0e+10
        tobt = 1000.0
        ifftol = 0
        call itrout (itform,descr,modfil,datfil,maxitr,tolreq,
     *    iruf,nit,pmu,rlast,tobt,ifftol,iof2)      
      end if
c
      itmax = maxitr + nit
c
c input the data
      if (abs(idebug) .eq. 1) write(*,*) 'reading data..'
      call inputd(datfil)
c input the fixed model characteristics
      if (abs(idebug) .eq. 1) write(*,*) 'reading model..'
      call inputm(modfil)
c
      iout = iof1
      open (iout, file='LOGFIL', iostat=lerr)
      if (lerr .ne. 0) then
        write(*,*) ' Error opening log file'
        stop
      end if
      write(*,*) 'Major I/O accomplished:'
      write(*,*) '  sit back and relax.......'
c
c do it
5     call occam(iruf,tolreq,tobt,ifftol,nit,pmu,rlast,stepsz,konv)
c
c write an iteration file
      call itrout (itform,descr,modfil,datfil,maxitr,tolreq,iruf,
     *    nit,pmu,rlast,tobt,ifftol,iof2)      
c
c let the console know roughly what is going on
      write(*,*) ' **ITERATION ', nit
      write(*,*) ' RMS MISFIT = ', tobt
      write(*,*) ' STEPSIZE = ', stepsz
      write(*,*) ' '
c
c test for convergence: continue inversion if 
c   1) iteration number does not exceed maximum, and
c   2) stepsize is still significant, and
c   3) rms misfit larger than required, and 
c   4) there are no irrecoverable convergence problems
c
      if (konv .ne. 0) then
        write (iof1,*) ' Convergence problems of type ', konv
        close(iof1)
        stop
      else if (nit .ge. itmax) then
        write (iof1,*) ' Stop on maximum iterations'
        close(iof1)
        stop
      else if ((tobt .ge. 1.01*tolreq) .or. (stepsz .ge. 0.0001)) then
        goto 5
      else
        write (iof1,*) ' Stop on normal convergence'
        close(iof1)
        stop
      end if
c      
c      
      end
c
C-----------------------------------------------------------------------
      SUBROUTINE OCCAM(IRUF,TOLREQ,TOBT,IFFTOL,NIT,PMU,RLAST,STEPSZ,
     * KONV)
C-----------------------------------------------------------------------
C
C OCCAM 2.0: Steven Constable IGPP/SIO La Jolla CA 92093-0225
c Subroutine Revision 2.01, 20 Jan 1993
c revisions for decomposition - CdG, 1994
C
C OCCAM EXECUTES ONE ITERATION OF A SMOOTH MODEL FINDER
C
C ON INPUT:
C
C  INCLUDE FILE 'OCCAM.INC':
C   /DATA/ CONTAINS
C     ND = NUMBER OF DATA
C     D(NDD) = VECTOR OF DATA VALUES
C     SD(NDD) = VECTOR OF ABSOLUTE STANDARD ERRORS IN THE DATA
C     DP(NDD,4) = MATRIX CONTAINING UP TO FOUR PARAMETERS ASSOCIATED WITH
C        EACH DATUM (RANGE, FREQUENCY, DATA TYPE ETC)
C     NPD = NUMBER OF PARAMETERS ASSOCIATED WITH EACH DATUM (E.G. FOR FREQ. ONLY
C              NPD = 1, FOR FREQ. AND RANGE NPD = 2)
C
C   /MODEL/ CONTAINS
C     NP = NUMBER OF MODEL PARAMETERS, USUALLY THE NUMBER OF LAYERS
C     PM(NPP) = VECTOR OF INITIAL MODEL PARAMETERS, USUALLY LOG10(LAYER RES'IES)
C
C   /RESULT/ CONTAINS
C     IDEBUG = 0 NORMAL, FASTEST AND QUIETEST EXECUTION
C            = 1 A LOW LEVEL OF VERBOSITY LETS YOU KNOW HOW THINGS ARE GOING
C            = 2 A TABLE OF RMS MISFIT VERSUS LAGRANGE MULTIPLIER IS PRODUCED
C        (USEFUL FOR TRACKING DOWN CONVERGENCE PROBLEMS).  THIS TABLE IS USED 
C        TO BRACKET THE MINIMUM
C            = 3 IF ONE ALSO WANTS A DUMP OF THE MODEL AT EACH LAGRANGE MULT IN
C        THE TABLE (NOT LIKELY TO BE REQUIRED BY END USERS)
c        if idebug is negative, the above absolute values are operational but
c        the step-reduction algorithm is disabled.
C     IOUT = FORTRAN UNIT NUMBER FOR OUTPUT, CAN EITHER BE TERMINAL OR A FILE
C
C  ARGUMENTS:
C    IRUF = 0 FOR MODEL MINIMIZATION
C         = 1 FOR FIRST DERIVATIVE MINIMIZATION
C         = 2 FOR SECOND DERIVATIVE MINIMIZATION
C     TOLREQ = REQUIRED RMS MISFIT
C     TOBT = RMS MISFIT FROM LAST ITERATION (LARGE INITIALLY)
C     IFFTOL = RECORD OF WHETHER FEASIBLE MODEL HAS BEEN ATTAINED (0=NO,
C         1=YES).  INITIALLY SET 0 AND THEN LEFT ALONE.
C     NIT = THE NUMBER OF PREVIOUS ITERATIONS IN THIS INVERSION. SET TO 0 ON FIRST 
C         CALL, IT WILL BE UPDATED BY OCCAM. 
C     PMU = LOG10(LAGRANGE) MULTIPLIER TO START SEARCHES WITH.  INITIALLY 
C         SET LARGE AND THEN LEFT FROM LAST CALL FOR EFFICIENCY
C     RLAST = ROUGHNESS OF LAST MODEL.  INITIALLY SET LARGE AND THEN LEFT.        
C
C ON OUTPUT:
C
C  /MODEL/ CONTAINS
C     PM(NPP) = VECTOR OF UPDATED MODEL PARAMETERS
C     DM(NDD) = VECTOR CONTAINING *APPROXIMATE* RESPONSE OF NEW MODEL
C  ARGUMENTS:
C     TOBT = RMS MISFIT OF NEW MODEL RESPONSE.
C     NIT = NUMBER OF PREVIOUS CALLS TO OCCAM1 DURING THIS INVERSION
C     STEPSZ = RMS MEASURE OF THE CHANGES IN THE MODEL PARAMETERS 
C     KONV = STATUS FLAG:
C        0 = NORMAL EXIT FOR A SUCCESSFUL ITERATION,
C        1 = A PERFECTLY SMOOTH MODEL HAS BEEN FOUND FOR THE REQUIRED TOLERANCE
C        2 = CONVERGENCE PROBLEMS.  UNABLE TO FIND A MODEL WITH AN R.M.S. MISFIT
C          LESS THAN OR EQUAL TO THAT OF THE LAST ITERATION'S
C        3 = CONVERGENCE PROBLEMS.  UNABLE TO FIND A MODEL WITH AN R.M.S. MISFIT
C          OF TOLREQ AND SMOOTHER THAN THE LAST ITERATION'S
C
C SUBROUTINES REQUIRED AND SUPPLIED:
C
C   MAKPTP, A SUBROUTINE WHICH CREATES THE ROUGHENING MATRIX TIMES ITS TRANS.
C   TOFMU(AMU), A FUNCTION WHICH RETURNS THE RMS MISFIT OF THE RESPONSE
C      OF THE MODEL CONSTRUCTED USING THE LAGRANGE MULTIPLIER AMU.
C      (CALLS CHOLIN,CHOLSL,FORMOD,ANORM)
C   INITM, TRMULT, MULT, ATAMULT AND ANORM, SUBROUTINES FOR MATRIX OPERATIONS
C   FMIN, A SUBROUTINE WHICH MINIMISES A UNIVARIATE FUNCTION
C   FROOT, A SUBROUTINE WHICH FINDS THE ROOT OF A UNIVARIATE FUNCTION
C   MINBRK, A SUBROUTINE WHICH BRACKETS A UNIVARIATE MINIMUM
C   SCANMU, A SUBROUTINE WHICH GENERATES THE MISFIT TABLE WHEN IBUG .GT. 0
C
C SUBROUTINES WHICH MUST BE SUPPLIED BY THE USER:
C
C  FORMOD(NP,ND,PM,DP,DM), COMPUTES THE FORWARD FUNCTION FOR MODEL PM() AT
C    THE DATA PARAMETERS DP() AND RETURNS IT IN DM().
C  FORDIV(NP,ND,PM,DP,DM,WJ), COMPUTES THE FORWARD FUNCTION AS DOES FORMOD()
C    BUT ALSO RETURNS THE MATRIX OF PARTIALS, WJ(ND,NP).
C  OBJMAT(IRUF, DEL, NPEN,LINKPR), ASSEMBLES THE PENALTY MATRIX (THIS WILL BE DEPENDENT ON 
C    MODEL TYPE AND DIMENSION)
C
C
C INCLUDES:
      INCLUDE 'imp.inc'
      INCLUDE 'occamdim.inc'
      INCLUDE 'occam.inc'
C
C ARGUMENTS:
      INTEGER IRUF,IFFTOL,NIT,KONV
      REAL TOLREQ,TOBT,PMU,RLAST,STEPSZ
C LOCAL VARIABLES:
      INTEGER I
      REAL TOL0, TMIN, TINT
      REAL AMU1, AMU2, AMU3, TAMU1, TAMU2, TAMU3
      REAL RUF, PMU2
C FUNCTIONS
      REAL TOFMU, FMIN, FROOT, ANORM, FNDRUF
      EXTERNAL TOFMU
C
C LOCAL PARAMETERS
      REAL TOLM, TOLI
C TOLM AND TOLI ARE THE TOLERANCES FOR FMIN AND FROOT RESPECTIVELY
C DECREASING THEM WILL IMPROVE ACCURACY AT THE COST OF MORE FORWARD 
C CALCULATIONS
      PARAMETER (TOLM= 0.05, TOLI = 0.002)
C
C-----------------------------------------------------------------------
      IF (abs(idebug) .EQ. 1) WRITE(*,*) ' ENTERING OCCAM..'
C SET THINGS UP FOR THIS ITERATION:
      KONV = 0
C FRAC CONTROLS THE STEP SIZE; NORMALLY WILL REMAIN AT 1.0 UNLESS WE HAVE
C CONVERGENCE PROBLEMS
      FRAC = 1.0
C CONSTRUCT THE PENALTY MATRIX MULTIPLIED INTO ITS TRANSPOSE
      IF (abs(idebug) .EQ. 1) WRITE(*,*)' CONSTRUCTING PENALTY MATRIX..'
      CALL MAKPTP(IRUF)
C CONSTRUCT WJTWJ, WJTWD, AND FIND INITIAL MISFIT
      IF (abs(idebug) .EQ. 1) WRITE(*,*) 
     *  ' CONSTRUCTING DERIVATIVE DEPENDENT MATRICES..'
      CALL MAKJTJ(TOL0)
C
      WRITE(IOUT,*) 'STARTING R.M.S. = ',TOL0
      IF (NIT .EQ. 0) THEN
	  if ((TOL0 .le. TOLREQ) .and. (IFFTOL .eq. 0)) then
	    write (*,*) ' This model is feasible'
	    IFFTOL = 1
	  end if
      END IF
      IF (abs(idebug) .EQ. 1) WRITE(*,*) 'STARTING R.M.S. = ',TOL0
      WRITE(IOUT,*) '** ITERATION ',NIT+1,' **'
C
C THE NEXT BLOCK OF CODE CONTROLS THE SELECTION OF THE LAGRANGE MULTIPLIER,
C PMU:
      IF (abs(idebug) .EQ. 1) 
     * WRITE(*,*) ' SEARCHING LAGRANGE MULTIPLIER:'
120   NFOR = 0
C PRODUCE THE MISFIT FUNCTION IF REQUIRED (THIS WILL BE USED TO BRACKET MIN)
      IF (abs(idebug) .EQ. 1) WRITE(*,*) ' ..BRACKETING MINIMUM..'
      IF (abs(idebug) .GE. 2) THEN
        CALL SCANMU(AMU1,AMU2,AMU3,TAMU2,TOLREQ)
      ELSE
C BRACKET THE MINIMUM USING MINBRK AND TWO GUESSES
        AMU1 = PMU - 1.0
        AMU2 = PMU
        CALL MINBRK(AMU1,AMU2,AMU3,TAMU1,TAMU2,TAMU3,TOFMU)
      END IF
C FIND THE MINIMUM
      IF (abs(idebug) .EQ. 1) WRITE(*,*) ' ..FINDING MINIMUM..'
      IF (TAMU2 .LT. TOLREQ) THEN
C WE'VE BEEN LUCKY AND FOUND AN ACCEPTABLE MINIMUM USING MINBRK
        TMIN = TAMU2
        PMU = AMU2
        WRITE(IOUT,*) 'MINIMUM TOL FROM MINBRK IS AT MU =',PMU
      ELSE
        TMIN = FMIN(AMU1,AMU2,AMU3,TAMU2,TOFMU,TOLM,PMU)
        WRITE(IOUT,*) 'MINIMUM TOL FROM FMIN IS AT MU =',PMU
      END IF
      WRITE(IOUT,*) 'AND IS =',TMIN
      WRITE(IOUT,*) 'USING ',NFOR,' EVALUATIONS OF FUNCTION'
C IF THE NEW MINIMUM TOLERANCE IS GREATER THAN THE TOLERANCE FROM THE
C PREVIOUS MODEL, WE ARE HAVING CONVERGENCE PROBLEMS. CUT THE STEP SIZE
      IF (((TMIN .GT. TOL0) .AND. (IFFTOL .EQ. 0) .OR.
     *    (TMIN .GT. TOLREQ) .AND. (IFFTOL .EQ. 1)) .and. 
     *    idebug .ge. 0) THEN
        WRITE(IOUT,*) 'DIVERGENCE PROBLEMS, CUTTING STEP SIZE'
        IF (abs(idebug) .EQ. 1) WRITE(*,*) ' ..DIVERGENCE PROBLEMS..'
        FRAC = FRAC*2.0
        IF (FRAC .GT. 1.0E+05) THEN
C WE HAVE CUT STEP SIZE A LOT TO NO EFFECT: GIVE UP
          KONV = 2
          RETURN
        ELSE
          GOTO 120
        END IF
      END IF
C
      IF (TMIN .LT. TOLREQ) THEN
        IF (abs(idebug) .EQ. 1) WRITE(*,*) ' ..FINDING INTERCEPT..'
C TOLERANCE IS BELOW THAT REQUIRED; FIND INTERCEPT.
C THE LOWER VALUE OF MU BRACKETING THE INTERCEPT IS EASILY FOUND: IT IS
C JUST THE MINIMUM
        AMU1 = PMU
        TAMU1 = TMIN
C THE UPPER BOUND IS FOUND BY TESTING SUCCESSIVELY GREATER MU
        AMU2 = PMU
        TAMU2 = TMIN
        NFOR = 0
140     IF (TAMU2 .LT. TOLREQ) THEN
C SUCCESSIVELY DOUBLE MU:
          AMU2 = AMU2 + 0.30103
          TAMU2 = TOFMU(AMU2)
          GOTO 140
        END IF
        PMU2 = FROOT(TOFMU,AMU1,AMU2,TAMU1,TAMU2,TOLREQ,TOLI)
        TINT = TAMU2 + TOLREQ
        WRITE(IOUT,*) 'INTERCEPT IS AT MU = ',PMU2
        WRITE(IOUT,*) 'AND IS = ',TINT
        WRITE(IOUT,*) 'USING ',NFOR,' FUNCTION EVALUATIONS'
      ELSE
C TOLERANCE IS NO YET SMALL ENOUGH.  WE WILL KEEP THE MINIMUM.
C SINCE FMIN RETURNS WITH PWK1() INSTEAD OF PWK2() WE NEED THIS COPY
        DO 150 I = 1,npinv
150       PWK2(I) = PWK1(I)
      END IF
C***END LAGRANGE MULTIPLIER SELECTION
C
C COMPUTE ROUGHNESS.  WE DO THIS BY A FUNCTION CALL TO AVOID HAVING THE PENALTY
C MATRIX HANG AROUND TAKING UP MEMORY.
      IF (abs(idebug) .EQ. 1) WRITE(*,*) 'COMPUTING ROUGHNESS..'
      RUF = FNDRUF(IRUF)
C
C IF WE ATTAINED THE INTERCEPT LAST ITERATION BUT THE MODEL IS GETTING 
C ROUGHER WE HAVE PROBLEMS WITH CONVERGENCE.
      IF ((RUF.GT.1.01*RLAST) .AND. (IFFTOL.EQ.1) .and. (idebug.ge.0)) 
     * THEN
        WRITE(IOUT,*) 'ROUGHNESS PROBLEMS, CUTTING STEP SIZE'
        FRAC = FRAC*2.0
C CHECK TO SEE IF ALL IS HOPELESS
        IF (FRAC .GT. 1.0E+05) THEN
          KONV = 3
          RETURN
        END IF
C OTHERWISE PLOUGH ON
        GOTO 120
      END IF
C
C SAVE NEW MODEL AND COMPUTE STEP SIZE
      IF (abs(idebug) .EQ. 1) WRITE(*,*) 'COMPUTING STEPSIZE..'
      DO 180 I = 1,npinv
        PWK3(I) = PWK2(I) - pminv(i)
180     pminv(i) = PWK2(I)
C THE STEPSIZE IS THE ACTUAL CHANGE IN THE MODEL, NORMALISED BY npinv:
      STEPSZ = SQRT(ANORM(npinv,PWK3)/npinv)
      WRITE(IOUT,*) 'STEPSIZE IS = ',STEPSZ
      WRITE(IOUT,*) 'ROUGHNESS IS = ',RUF
      WRITE(IOUT,*) ' '
C TIDY UP
      NIT = NIT + 1
      RLAST = RUF
      IF (TMIN .LT. TOLREQ) THEN
        IFFTOL = 1
        TOBT = TINT
      ELSE
        TOBT = TMIN
      END IF
C SEE IF WE HAVE A PERFECLY SMOOTH MODEL THAT FITS DATA:
      IF (RUF .LT. 1.0E-9 .AND. TOBT .LE. 1.01*TOLREQ) KONV = 1
      IF (abs(idebug) .EQ. 1) WRITE(*,*) 'LEAVING OCCAM'
      RETURN
      END
C
C-----------------------------------------------------------------------
       SUBROUTINE SCANMU(AM1,AM2,AM3,T2,T0)
C-----------------------------------------------------------------------
C 
C OCCAM 2.0: Steven Constable IGPP/SIO La Jolla CA 92093-0225
c Subroutine Revision 2.00, 13 May 1992
C
C SCANMU PRODUCES A REPORT OF MISFIT VERSUS LAGRANGE MULTIPLIER.
C IT CALLS THE FORWARD ROUTINE A LOT SO IS REALLY ONLY USED FOR DEBUGGING
C AND PARTICULARLY NASTY CASES
C IT LOOKS FOR THE GLOBAL MINIMUM AS FOLLOWS:
C   STARTING AT THE LARGEST MU USED FOR THE DEBUG REPORT, WE LOOK FOR THE 
C   FIRST MINIMUM.  WE ALSO LOOK FOR THE GLOLBAL MIN.  WE WANT THE GLOBAL 
C   MIN UNLESS THE LOCAL MIN AT LARGER MU IS LOWER THAN THE REQUIRED 
C   TOLERANCE. 
C 
C ON INPUT:
C   T0 IS THE REQUIRED TOLERANCE
C   /RESULT/ IDEBUG IS THE PRINT LEVEL FROM OCCAM1
C   /RESULT/ IOUT IS A UNIT NUMBER FOR OUTPUT
C ON OUTPUT:
C   AM1, AM2, AM3 ARE THREE LAGRANGE MULTIPLIERS WHICH BRACKET A MINIMUM
C   T2 IS THE TOLERANCE AT AM2.
C   /RESULT/ ..PMW1().. HAS THE MODEL ASSOCIATED WITH AM2,T2.
C SUBROUTINES CALLED: 
C   TOFMU
C
C INCLUDES:
      INCLUDE 'imp.inc'
      INCLUDE 'occamdim.inc'
      INCLUDE 'occam.inc'
C
C ARGUMENTS:
      REAL AM1,AM2,AM3,T2,T0
C LOCAL VARIABLES
      LOGICAL SEARCH
      INTEGER I,J,K
      REAL T1,TG2,T3,TL2,AML3,AML2,AML1,AMG3,AMG2,AMG1,AMU,T
C FUNCTIONS
      REAL TOFMU
C
C-----------------------------------------------------------------------
      WRITE(IOUT,*) 'MISFIT AS A FUNCTION OF MU:'
      T1 = 1.0E+09
      T2 = 1.1E+09
      TG2 = 0.9E+09
      AM2 = 28.
      AM1 = 28.
      SEARCH = .TRUE.
C
C START AT THE TOP (SMOOTHEST AND PRESUMABLY WORST FIT), SWEEPING FROM
C LOG10(MU) = 28 TO LOG10(MU) = 0 AT TWO VALUES PER DECADE
      DO 100 K = 28,0,-1
C KEEP THE LAST TWO VALUES FOR MINIMUM SEARCH
        T3 = T2
        T2 = T1
        AM3 = AM2
        AM2 = AM1
C KEEP THE MODEL ASSOCIATED WITH THE MIDDLE VALUE IF WE ARE SEARCHING
C FOR THE MINIMUM
        IF (SEARCH) THEN
          DO 10 I = 1,npinv
10          PWK1(I) = PWK2(I)
        END IF
C KEEP THE MODEL ASSOCIATED WITH THE MIDDLE VALUE ANYWAY IN CASE
C GLOBAL AND LOCAL MINIMA ARE FOUND
        DO 20 I = 1,npinv
20        PWK3(I) = PWK2(I)
C GET THE NEXT MU AND ASSOCIATED TOLERANCE
        AM1 = FLOAT(K)/2.
        T1 = TOFMU(AM1)
C
        IF ((T2 .LE. T1) .AND. (T2 .LE. T3) .AND. SEARCH) THEN
C WE HAVE FOUND THE FIRST MINIMUM. KEEP IT AND TURN OFF SEARCH
C (MODEL IS ALREADY STASHED IN PM1())
          SEARCH = .FALSE.
          TL2 = T2
          AML3 = AM3
          AML2 = AM2
          AML1 = AM1
        END IF
C KEEP AN EYE OUT FOR THE GLOBAL MIN IN CASE IT IS DIFFERENT
        IF (T2 .LE. TG2) THEN
          TG2 = T2
          AMG3 = AM3
          AMG2 = AM2
          AMG1 = AM1
C COPY ACCROSS MODEL ASSOCIATED WITH GLOBAL MIN.
          DO 30 I = 1,npinv
30          PWK1(I) = PWK3(I)
        END IF
C RECORD IN THE LOG FILE
        WRITE(IOUT,*) AM1,T1
C
C OUTPUT THE MODEL AS WELL IF REQUIRED
        IF (abs(idebug) .GE. 3) THEN
          DO 60 J = 1,npinv
60          WRITE(IOUT,*) '   ',PWK2(J)
        END IF
100   CONTINUE
C WRITE OUT THE GAUSS STEP AT HIGHEST DEBUG LEVEL
      IF (abs(idebug) .GE. 3) THEN
        AMU = -99.0
        T = TOFMU(AMU)
        WRITE(IOUT,*) AM1,T1
        DO 110 J = 1,npinv
110       WRITE(IOUT,*) '   ',PWK2(J)
      END IF
C
C NOW CHECK OUT THE MINIMA
C  IF THE FIRST MINIMUM ACHIEVES THE TOLERANCE REQUIRED WE WANT IT
      IF (TL2 .LE. T0) THEN
        T2 = TL2
        AM1 = AML1
        AM2 = AML2
        AM3 = AML3
      ELSE 
C OTHERWISE THE GLOBAL MIN IS JUST FINE
        T2 = TG2
        AM1 = AMG1
        AM2 = AMG2
        AM3 = AMG3
      END IF
      RETURN
      END
C
C-----------------------------------------------------------------------
      REAL FUNCTION TOFMU(ALOGMU)
C-----------------------------------------------------------------------
C
C OCCAM 2.0: Steven Constable IGPP/SIO La Jolla CA 92093-0225
c Subroutine Revision 2.01, 13 Jan 1993
C
C FUNCTION TOFMU RETURND THE RMS MISFIT OF THE RESPONSE OF A MODEL CONSTRUCTED
C USING THE GIVEN VALUE OF LAGRANGE MULTIPLIER
C
C ON INPUT:
C   ALOGMU = LOG10 OF THE LAGRANGE MULTIPLIER 
C            (-99. IF A ZERO LAGRANGE MULTIPLIER NEEDED)
C   /RESULT/ WJTWJ(), WJTWD(), PTP(), NFOR
C   /DATA/ DP(), D(), SD(), ND
C   /MODEL/ NP
C
C ON OUTPUT
C   TOFMU = R.M.S. MISFIT OR 1.0E+10 IF CHOLESKY DECOMPOSITION FAILED
C   /RESULT/ PWK2() CONTAINS THE MODEL WHICH PRODUCES THE MISFIT TOFMU
C   /MODEL/ DM() CONTIANS THE RESPONSE OF THE MODEL 
C
C SUBROUTINES CALLED:
C   CHOLIN, CHOLSL ARE R.L. PARKER'S SUBROUTINES TO PERFORM CHOLESKY DECOMP.
C       AND THEN SOLVE A LINEAR SYSTEM BY BACK SUBSTITUTION.
C   ANORM, FORMOD ARE EXPLAINED IN OCCAM1
C
C  AT VERSION 1.4 LOG10(MU) USED TO IMPROVE PERFORMANCE OF 1D OPTIMISATION
c
C INCLUDES:
      INCLUDE 'imp.inc'
      INCLUDE 'occamdim.inc'
      INCLUDE 'occam.inc'
C
C ARGUMENTS:
      REAL ALOGMU
C LOCAL VARIABLES
      REAL A(NPP,NPP+1), DWK1(NDD), AMU, PART, PWK4(NPP),pwk5(npp)
      INTEGER I, J, ISTAT
C FUNCTIONS:
      REAL ANORM
C
C-----------------------------------------------------------------------
      IF (ALOGMU. EQ. -99.) THEN
C THE GAUSS STEP IS REQUIRED
        AMU = 0.0
      ELSE
        AMU = 10.**(ALOGMU)
      END IF
      NFOR = NFOR + 1
C ADD AMU.TRANS(PARTIAL).PARTIAL TO TRANS(W.J).W.J
      DO 20 I = 1,npinv
        DO 20 J = 1,npinv
          PWK4(J) = WJTWD(J) 
20        A(I,J) = AMU*PTP(I,J) + WJTWJ(I,J)
C
C SOLVE THE LINEAR SYSTEM
      CALL CHOLIN(NPP,npinv,A,1,ISTAT)
      IF (ISTAT .NE. 0) THEN
C IF DECOMP FAILED WE GIVE WARNING BUT MIGHT STILL GET BY IF TOFMU
C IS SET TO A LARGE VALUE
        WRITE(*,*) 
     *    'Warning:- Cholesky decomp. failed at mu = ',AMU
        TOFMU = 1.0E+10
        RETURN
      END IF
      CALL CHOLSL(NPP,npinv,A,PWK2,PWK4,0)
C CUT STEP SIZE IF NECESSARY
      IF (FRAC .GT. 1.001) THEN
         PART = 1./FRAC
         DO 30 I = 1,npinv
30          PWK2(I) = (1.-PART)*pminv(I) + PART*PWK2(I)
      END IF
C first translate, then CALCULATE THE MODEL RESPONSE AND MISFIT
      do i=1,npinv
         pwk5(i)=pwk2(i)
      enddo
      call trnslt(pwk5,pwk4)
c pwk is often changed within trnslt, save the changes
      do i=1,npinv
         pwk2(i)=pwk5(i)
      enddo
c  
      CALL FORMOD(NP,ND,PWK4,DP,DM)
      DO 40 I = 1,ND
40      DWK1(I) = (D(I) - DM(I))/SD(I)
      TOFMU = SQRT(ANORM(ND,DWK1)/ND)
      write(*,*) tofmu,amu
      IF (abs(idebug) .EQ. 1) 
     *  WRITE(IOUT,*) 'TOFMU=', TOFMU,' AMU=', ALOGMU
      RETURN
      END
C
c-----------------------------------------------------------------------
      subroutine trnslt(pwk5,pwk4)
c
c translate from boundary type model to brick type model
c for input to formod
c DON'T ALLOW PINCHOUTS FOR NOW
c  on input:
c	pwk5 is the new boundary model
c  on output:
c	pwk4 is the new output model
      include 'imp.inc'
      include 'occamdim.inc'
      include 'occam.inc'
c local variables
      real pwk5(npp),pwk4(npp)
      integer i,j,k,ind
c
c MT is relatively insensitive to high resistivity, put
c ceiling of 10^6 on resitivities and floor of 10^-6
c (allowable range of resistivities found in the Earth)
      do k=1,nbig
         if(pwk5(k).gt. 6) pwk5(k)=6.
         if(pwk5(k).lt. -6) pwk5(k)=-6.
      enddo
c
c ensure parameter depths are all .ge. seadep 
      ind=nbig
      do k=1,nbig-1
         do j=1,lcol
            ind=ind+1
            if (seadep(j) .eq. 0) then
               if (pwk5(ind) .lt. alog(0.5*depth(2)))
     +            pwk5(ind) = alog(depth(2))-1
            else 
               if (pwk5(ind) .lt. alog(seadep(j))) 
     +            pwk5(ind) = alog(seadep(j))
            endif
         enddo
      enddo
c ensure that deeper boundaries are >= shallow boundaries
      do i=1,nbig-2
         do j=1,lcol
            ind=nbig+(i-1)*lcol+j
            if (pwk5(ind) .gt. pwk5(ind+lcol)) then
c set to depth of deeper boundary (assuming top layer better resolved)
               pwk5(ind+lcol) = pwk5(ind)
            endif
         enddo
      enddo
c for each column, use depths to assign idepth
      do j=1,lcol
         do i=1,nbig-1
            ind=(i-1)*lcol+nbig+j
            idepth(i,j)=1
            do k=1,llay
               if (pwk5(ind) .ge. alog((depth(k)+depth(k+1))/2.)) then
                  idepth(i,j) = k+1
               endif
            enddo
            pwk5(ind)=alog(depth(idepth(i,j)))
         enddo
      enddo
c now ensure that there are no pinchouts (reset idepth values)
c      do j=1,lcol
c         if (idepth(1,j) .lt. (iseadp(j)+1)) then   !first layer
c            idepth(1,j) = iseadp(j)+1
c            pwk5(nbig+j)=alog(depth(idepth(1,j)))
c         endif
c also ensure no pinchouts at bottom (for top layer)
c         if (idepth(1,j) .gt. (llay-nbig+2)) then
c            idepth(1,j) = llay-nbig+2
c            pwk5(nbig+j)=alog(depth(idepth(1,j)))
c         endif
c      enddo
c      do j=1,lcol
c        do i=2,nbig-1
c          if (idepth(i,j).le.idepth(i-1,j)) then
c             idepth(i,j)=idepth(i-1,j)+1
c             ind=(i-1)*lcol+nbig+j
c             pwk5(ind)=alog(depth(idepth(i,j)))
c          endif
c        enddo
c      enddo
c end of no pinchout code
c assign values to ijind
      do j=1,lcol
c for first unit
         if (idepth(1,j).gt.1) then
            do i=1,idepth(1,j)-1
               if (irindx(i,j) .ne. 0) ijind(i,j)=1
            enddo
         endif
c middle units
         do k=2,nbig-1
            if (idepth(k,j) .gt. 1) then
               do i=idepth(k-1,j),idepth(k,j)-1
                  if (irindx(i,j) .ne. 0) ijind(i,j)=k
               enddo
            endif
         enddo
c last unit
         do i=idepth(nbig-1,j),llay
            if (irindx(i,j) .ne. 0) ijind(i,j)=nbig
         enddo
      enddo
c
cc now assign values to the block model, and kblock and iblock
      do k=1,nbig
         kblock(k) = 0
      enddo
      ind = 0
      do i=1,llay
         do j=1,lcol
            if (irindx(i,j) .ne. 0) then
               ind=ind+1
               pwk4(ind)=pwk5(ijind(i,j))
               k=ijind(i,j)
               kblock(k)=kblock(k)+1
               iblock(k,kblock(k))=ind
            endif
         enddo
      enddo   
c make sure the total number of bricks is right
      j=0
      do i=1,nbig
         j=j+kblock(i)
         write(*,*) kblock(i), 'blocks in unit', i
         write(iout,*) kblock(i), 'blocks in unit', i
      enddo
      if (j.ne.np) then
         write(*,*) 'problem in trnslt, stopping'
         write(*,*) 'np, kblock'
         write(*,*) np, (kblock(i),i=1,nbig)
         stop
      endif
c
      return
      end
C-----------------------------------------------------------------------
      SUBROUTINE MAKJTJ(TOL0)
C-----------------------------------------------------------------------
C 
C OCCAM 2.0: Steven Constable IGPP/SIO La Jolla CA 92093-0225
c Subroutine Revision 2.00, 13 May 1992
C
C CONSTRUCTS MATRICES THAT DEPEND ON THE JACOBIAN (WJTWJ AND WJTWD).
C THE JACOBIAN ONLY NEEDS TO BE AROUND LONG ENOUGH TO DO
C THIS, SO BY STUFFING THIS INTO A SUBROUTINE DYNAMIC MEMORY ALLOCATION
C WILL MAKE WJ DISSAPEAR AFTER USE.
C 
C ON INPUT:
C   CURRENT MODEL AND DATA STORED IN COMMON /MODEL/ AND /DATA/ 
C ON OUTPUT:
C   /RESULT/ WJTWJ = W.J.TRANS(W.J)
C   /RESULT/ WJTWD = WEIGHTED, TRANSLATED DATA PREMULTIPLIED BY TRANS(W.J)
C   TOL0 = RMS MISFIT ASSOCIATED WITH CURRENT MODEL
C CALLS:
C   ATAMUL, MULT, TRMULT, FORDIV, ANORM
C
C INCLUDES:
      INCLUDE 'imp.inc'
      INCLUDE 'occamdim.inc'
      INCLUDE 'occam.inc'
C
C ARGUMENTS:
      REAL TOL0
C LOCAL VARIABLES
      REAL WJ(NDD,NPP),DWK1(NDD),DHAT(NDD),wjinv(ndd,npp)
      INTEGER I, J
C FUNCTIONS 
      REAL ANORM
C
C-----------------------------------------------------------------------
C CALCULATE THE MATRIX OF PARTIALS AND MODEL RESPONSE
c      write(*,*) 'calling fordiv'
      CALL FORDIV(NP,ND,PM,DP,DM,WJ)
c      open(5,file='jacob.out',form='unformatted')
c      write(5) ((wj(i,j),j=1,np),i=1,nd)
c      close(5)
c      write(*,*) 'back from fordiv'
c leave fordiv as it is and form modified Jacobeans in bndj
      call bndj(wj,wjinv)
c      open(5,file='wjinv.out',form='unformatted')
c      write(5) ((wjinv(i,j),j=1,npinv),i=1,nd)
c      close(5)
C CALC MISFIT VECTOR AND MISFIT
      DO 60 I = 1,ND
60      DWK1(I) = (D(I) - DM(I))/SD(I)
      TOL0 = SQRT(ANORM(ND,DWK1)/ND)
C WEIGHT THE JACOBIAN MATRIX
      DO 70 I = 1,ND
         DO 70 J = 1,npinv
70          wjinv(i,j) = wjinv(i,j)/sd(i)
C FORM W.J.TRANS(W.J)
      CALL ATAMUL(NDD,ND,NPP,npinv,wjinv,WJTWJ)
      open(5,file='wjtwj.out',form='unformatted')
      write(5) ((wjtwj(i,j),j=1,npinv),i=1,npinv)
      close(5)
C FORM THE WEIGHTED, TRANSLATED DATA AND PREMULTIPLY BY TRANS(W.J)
      CALL MULT(NDD,ND,NPP,npinv,1,wjinv,pminv,DHAT)
      DO 80 I = 1,ND
80       DHAT(I) = DWK1(I) + DHAT(I)
      CALL TRMULT(NDD,ND,NPP,npinv,1,wjinv,DHAT,WJTWD)
      return
      end
C
c-----------------------------------------------------------------------
      subroutine bndj(wj,wjinv)
c-----------------------------------------------------------------------
C includes:
      include 'imp.inc'
      include 'occamdim.inc'
      include 'occam.inc'
c local variables
      real wj(ndd,npp), wjinv(ndd,npp),p1,p0,ddep,ddepup
      integer nlyr,jup,jdown,i,j,k,ix
c
c double check
      write(*,*) 'idepths'
      do i=1,nbig-1
         write(*,*) (idepth(i,k),k=1,lcol)
      enddo
      write(*,*) 'kblock'
      write(*,*) (kblock(i),i=1,nbig)
c
c  first get modified derivatives for large resistivity blocks
c  sum up Jacobeans for small blocks within each large unit
      do i=1,nbig
         do j=1,nd
            wjinv(j,i)=0.0    
            do k=1,kblock(i)
               wjinv(j,i)=wjinv(j,i)+wj(j,iblock(i,k))
            enddo
         enddo
      enddo
c now form derivatives for log(boundary depths) 
      ix=nbig
      do i=1,nbig-1
         do j=1,lcol
            ix=ix+1
            nlyr=idepth(i,j)
            p1=pminv(i)
            jup = irindx(nlyr-1,j)
            jdown = irindx(nlyr,j)
            ddep=(alog(depth(nlyr+1))-alog(depth(nlyr)))
            ddepup=(alog(depth(nlyr))-alog(depth(nlyr-1)))
            if (jup .eq. 0) then	! forward diff at top of model
               p0=pminv(ijind(nlyr,j))	! allows for pinchouts,otherwise p0=pminv(i+1)
               do k=1,nd
                  wjinv(k,ix)=wj(k,jdown)*(p1-p0)/ddep
               enddo
            elseif (nlyr .eq. llay+1) then    ! backward diff at bottom
               p0=pminv(i+1)
               do k=1,nd
                  wjinv(k,ix)=wj(k,jup)*(p1-p0)/ddepup
               enddo
            else		! central differencing for all other layers
               p0=pminv(ijind(nlyr,j))
               do k=1,nd
                  wjinv(k,ix)=0.5*(p1-p0)*(wj(k,jdown)/ddep +
     +                wj(k,jup)/ddepup)
               enddo
            endif
         enddo
      enddo
c
      if (ix .ne. npinv) then
         write(*,*) 'problem in deriving modified Jacobean'
         write(*,*) 'report problem to Catherine'
         stop
      endif
      return
      end
c
C-----------------------------------------------------------------------
      SUBROUTINE MAKPTP(IRUF)      
C-----------------------------------------------------------------------
C 
C OCCAM 2.0: Steven Constable IGPP/SIO La Jolla CA 92093-0225
c Subroutine Revision 2.00, 13 May 1992
C
C CONSTRUCTS PENALTY MATRIX (DEL) MULTIPLIED BY ITS TRANSPOSE.
C THE PENALTY MATRIX ONLY NEEDS TO BE AROUND LONG ENOUGH TO DO
C THIS, SO BY STUFFING THIS INTO A SUBROUTINE DYNAMIC MEMORY ALLOCATION
C CAN GET RID OF IT AFTERWARDS
C 
C ON INPUT:
C   IRUF = 0 FOR NO ROUGHNENING (I.E. MIN. NORM MODELS) (ADDED IN V1.5)
C        = 1 FOR 1ST DERIVATIVE ROUGHNESS
C          2 FOR 2ND DERIVATIVE ROUGHNESS
C   /MODEL/ NP = NUMBER OF MODEL PARAMETERS
C ON OUTPUT:
C   /RESULT/ PTP() = DEL(TRANS)*DEL
C CALLS:
C   OBJMAT, TRMULT
C
C INCLUDES:
      INCLUDE 'imp.inc'
      INCLUDE 'occamdim.inc'
      INCLUDE 'occam.inc'
C
C ARGUMENTS:
      INTEGER IRUF
C LOCAL VARIABLES
      INTEGER NPEN, linkpr(nnp,2),i,j
      REAL DEL(NNP,NPP)
C
C-----------------------------------------------------------------------
C MAKE PARTIALS MATRIX:
      CALL OBJMAT(IRUF, DEL, NPEN,linkpr)
      IF (abs(idebug) .EQ. 1) 
     *   WRITE(*,*) ' NUMBER OF PENALTY TERMS =', NPEN
C FORM TRANS(DEL).DEL
      call deltdel(nnp,npen,npp,npinv,del,ptp,linkpr)
      RETURN
      END
c
C-----------------------------------------------------------------------
      subroutine deltdel(mad,ma,nad,na,a,c,lpair)
C-----------------------------------------------------------------------
c multiplies the transpose of del by del
c takes advantage of sparseness of the del matrix 
c (i.e. multiply out only connected bricks)
      include 'imp.inc'
c     calls INITM
C
      integer mad,ma,nad,na,i,j,kk,k,lpair(mad,2)
      real a(mad,na),c(nad,na), cij
c
      call initm(nad,na,na,c)
c connected pairs
      do kk=1,ma
         i=lpair(kk,1)
         j=lpair(kk,2)
         cij = 0.0
         do k=1,ma
            cij = a(k,i)*a(k,j) + cij
         enddo
         c(i,j) = cij
         c(j,i) = cij
      enddo
c diagonal component
      do i=1,na
         cij = 0.0
         do k=1,ma
            cij = a(k,i)*a(k,i) + cij
         enddo
         c(i,i) = cij
      enddo
      return
      end
C
C-----------------------------------------------------------------------
      REAL FUNCTION FNDRUF(IRUF)      
C-----------------------------------------------------------------------
C 
C OCCAM 2.0: Steven Constable IGPP/SIO La Jolla CA 92093-0225
c Subroutine Revision 2.00, 13 May 1992
C
C COMPUTES THE ROUGHNESS OF THE MODEL. FOR MEMORY ALLOCATION CONSIDERATIONS,
C THE PENALTY MATRIX HAS NOT BEEN KEPT AROUND, SO WE NEED TO REFORM IT.
C 
C ON INPUT:
C   IRUF = 0 FOR NO ROUGHNENING (I.E. MIN. NORM MODELS) (ADDED IN V1.5)
C        = 1 FOR 1ST DERIVATIVE ROUGHNESS
C          2 FOR 2ND DERIVATIVE ROUGHNESS
C   /RESULT/ PWK2() = LOCATION OF THE MODEL WE WANT TO MEASURE
C   /MODEL/ NP = NUMBER OF MODEL PARAMETERS
C ON OUTPUT:
C   FNDRUF = ROUGHNESS OF MODEL
C CALLS:
C   OBJMAT, TRMULT
C
C INCLUDES:
      INCLUDE 'imp.inc'
      INCLUDE 'occamdim.inc'
      INCLUDE 'occam.inc'
C
C ARGUMENTS:
      INTEGER IRUF, linkpr(nnp,2)
C LOCAL VARIABLES
      INTEGER I, K, NPEN
      REAL DEL(NNP,NPP), SUM, VECT(NPP)
C FUNCTIONS
      REAL ANORM
C
C-----------------------------------------------------------------------
C MAKE PARTIALS MATRIX:
      CALL OBJMAT(IRUF, DEL, NPEN,linkpr)
      DO 170 I = 1,NPEN
        SUM = 0.0
        do 165 k = 1,npinv
165       SUM = SUM + DEL(I,K)*PWK2(K)
170     vect(i) = sum
      FNDRUF = ANORM(NPEN,VECT)
      return
      end
C
C-----------------------------------------------------------------------
      subroutine itrout (itform,descr,modfil,datfil,maxitr,tolreq,
     *    iruf,nit,pmu,rlast,tobt,ifftol,iof2)      
C-----------------------------------------------------------------------
c
C OCCAM 2.0: Steven Constable IGPP/SIO La Jolla CA 92093-0225
c Subroutine Revision 2.00, 13 May 1992
C
c writes an iteration file for OCCAM
c
c includes:
      include 'imp.inc'
      include 'occamdim.inc'
      include 'occam.inc'
c   uses npinv, pminv(), idebug
c arguments:
      integer maxitr, iruf, nit, ifftol,iof2
      real tolreq, pmu, rlast, tobt, pwk5(npp)
      character*50 modfil, datfil, itform, descr, datetm
c local variables
      integer lerr, i
c   lerr = error flag on file operation
c   i = loop index
      character*2 num
c
C-----------------------------------------------------------------------
c create file name suffix
      write (num, '(i2)') nit
      if (nit .le. 9) num(1:1) = '0'
c open iteration file
      open (unit=iof2, file='ITER'//num, iostat=lerr)
      if (lerr .ne. 0) then
c we could be in the middle of an iteration, so don't die on error, but
c send a message and forge on regardless
        write(*,*) ' Error opening iteration file'
        return
      end if
      call datime(datetm)
c
c write stuff      
      write(unit=iof2,fmt='(a)') 'FORMAT:           '//itform
      write(unit=iof2,fmt='(a)') 'DESCRIPTION:      '//descr
      write(unit=iof2,fmt='(a)') 'MODEL FILE:       '//modfil
      write(unit=iof2,fmt='(a)') 'DATA FILE:        '//datfil
      write(unit=iof2,fmt='(a)') 'DATE/TIME:        '//datetm
      write(unit=iof2,fmt='(a,i3)')    'MAX ITER:         ',maxitr
      write(unit=iof2,fmt='(a,f6.3)')  'REQ TOL:          ',tolreq
      write(unit=iof2,fmt='(a,i3)')    'IRUF:             ',iruf
      write(unit=iof2,fmt='(a,i3)')    'DEBUG LEVEL:      ',idebug
      write(unit=iof2,fmt='(a,i3)')    'ITERATION:        ',nit
      write(unit=iof2,fmt='(a,g15.7)') 'PMU:              ',pmu
      write(unit=iof2,fmt='(a,g15.7)') 'RLAST:            ',rlast
      write(unit=iof2,fmt='(a,g15.7)') 'TLAST:            ',tobt
      write(unit=iof2,fmt='(a,i3)')    'IFFTOL:           ',ifftol
      write(unit=iof2,fmt='(a,i4)')    'NO. PARMS:        ',np
      if (nit .gt. 0) then
         do i=1,npinv
            pwk5(i)=pminv(i)
         enddo
c         write(*,*) 'pminv in itrout'
c         write(*,*) (pminv(i),i=1,npinv)
         call trnslt(pwk5,pm)
         write(unit=iof2,fmt='(5g14.6)') (pm(i), i=1,np)
      else
         write(unit=iof2,fmt='(5g14.6)') (pm(i), i=1,np)
      endif
      write(unit=iof2,fmt= *)   nbig
      write(unit=iof2,fmt='(5g14.6)') (pminv(i), i=1,nbig)
C
      close(unit=iof2)
C      
      return
      end
C
C-----------------------------------------------------------------------
	subroutine formod(np, nd, pm, dp, dm)
c-----------------------------------------------------------------------
c
c OCCAM2DMT 1.0: Steven Constable IGPP/SIO La Jolla CA 92093-0225
c Subroutine Revision 1.01, 13 Jan 1993
c revision to allow for impedance data - cdh -Feb 23, 1994
c impedances are broken up into real and imaginary
c parts so that all data can be treated as real
c allows for multiplication by decomposition matrix
c
c includes:
      include 'imp.inc'
      include 'mt2D.inc'
      include 'blkj.inc'
      include 'blkjc.inc'
c uses nfre, frek(), nrl, nrfix, y(), nbrick
c uses rhte(), phte(), kzte(), rhtm(), phtm()
c
c arguments:
      real pm(npp), dp(ndd,3), dm(ndd)
      integer np, nd
c local variables
      integer i, j, nexec, nres, itype, isite, ifreq,k
c   i,j = loop counters
c   nexec = instruction to PW2DI to perform only forward calculation
c   nres = number of resistivities in y() array
c   itype, isite, ifreq = integer versions of dp() entries
       real tesum, tmsum, xymult, yxmult
       real cose, sine, cos2, sin2, sico, arot
       real zxxr, zxxi, zxyr, zxyi, zyxr, zyxi, zyyr, zyyi
c   tesum = sum of logs of TE free static shifts 
c   tmsum = sum of logs of TM free static shifts 
c   xymult = product of decompostion parameters for Zxy component
c   yxmult = product of decompostion parameters for Zyx component
c   zetc   = place holders for data as they get rotated
c 
c-----------------------------------------------------------------------
c total number of resistivities = fixed resistivities + free res'ties 
c (but not air)
      nres = nrfix + nbrick
      nexec = 1
      tesum = 0.0
      tmsum = 0.0
      xymult= 0.0
      yxmult= 0.0
c      
c copy across resistivities from parameter array to y()
      do 10 i = 1, nbrick
10        y(nrfix+1+i) = 10.**pm(i)
c copy across any static shifts in the free parameters
      if (nfss .gt. 0) then
        do 12 i = 1,nsb
          if (isfx(i) .ne. 0) shift(i) = pm(isfx(i))
12        continue
      end if
c copy across any decomposition parameters in the free parameters
      if (nfdp .gt. 0) then
        do 22 i = 1,nstat
          do 22 j=1,4
          if (idfx(i,j) .ne. 0) dcomp(i,j) = pm(idfx(i,j))
22        continue
      end if
c 
      if(itfx .ne. 0) theta=pm(np)
c      write(*,*) 'theta in formod=',theta
c
c compute the MT response
      call PW2DI(nres,nexec)
c            
c strip off data:
c  the first column of dp() is the site number:
c  the second column of dp() is the frequency number:
c  the third column of dp() is the data type:
c  the fourth column of dp() is the input strike angle:
c   1 = TE resistivity
c   2 = TE phase
c   3 = real(tipper)
c   4 = imag(tipper)
c   5 = TM resistivity
c   6 = TM phase
c   7 = real(Zxx)                11 = real(Zyx)
c   8 = imag(Zxx)                12 = imag(Zyx)
c   9 = real(Zxy)                13 = real(Zyy)
c  10 = imag(Zxy)                14 = imag(Zyy)
c
c for every datum:
      do 150 i = 1, ndata
        isite = int(dp(i,1)+0.1)
        ifreq = int(dp(i,2)+0.1)
        itype = int(dp(i,3)+0.1)
        if(itype.eq.1) dm(i) = alog10(rhte(isite,ifreq))
        if(itype.eq.2) dm(i) = phte(isite,ifreq)
        if(itype.eq.3) dm(i) = real(kzte(isite,ifreq))
        if(itype.eq.4) dm(i) = aimag(kzte(isite,ifreq))
        if(itype.eq.5) dm(i) = alog10(rhtm(isite,ifreq))
        if(itype.eq.6) dm(i) = phtm(isite,ifreq)
        if(itype.eq.7) dm(i) = 0.
        if(itype.eq.8) dm(i) = 0.
        if(itype.eq.9) dm(i) = real(zimpe(isite,ifreq))
        if(itype.eq.10) dm(i) = aimag(zimpe(isite,ifreq))
        if(itype.eq.11) dm(i) = real(zimpm(isite,ifreq))
        if(itype.eq.12) dm(i) = aimag(zimpm(isite,ifreq))
        if(itype.eq.13) dm(i) = 0.
        if(itype.eq.14) dm(i) = 0.
c
c Search the static shifts to see if one applies to this datum
        do 140 j = 1, nsb
          if ((isx1(j).eq.isite) .and. (isx3(j).eq.itype)) then
c Apply the static shift
            dm(i) = dm(i) + shift(j)
          end if
140     continue
c Search the decomposition parameters to see if one applies to this datum
        do 145 j = 1, nstat
          do 145 k=1,4
          if (j.eq.isite) then
            if (itype .eq. 7) dm(i)=dcomp(j,1)*real(zimpm(isite,ifreq))
           if (itype .eq. 8) dm(i)=dcomp(j,1)*aimag(zimpm(isite,ifreq))
            if (itype .eq. 9) dm(i)=dcomp(j,2)*real(zimpe(isite,ifreq))
           if (itype .eq. 10) dm(i)=dcomp(j,2)*aimag(zimpe(isite,ifreq))
            if (itype .eq. 11) dm(i)=dcomp(j,3)*real(zimpm(isite,ifreq))
           if (itype .eq. 12) dm(i)=dcomp(j,3)*aimag(zimpm(isite,ifreq))
            if (itype .eq. 13) dm(i)=dcomp(j,4)*real(zimpe(isite,ifreq))
           if (itype .eq. 14) dm(i)=dcomp(j,4)*aimag(zimpe(isite,ifreq))
          end if
145       continue
150     continue
c
c Now compute the sums for the static shift constraints 
      do 160 i = 1, nsb
        if (isfx(i) .ne. 0) then
c This shift is a free parameter: 
          if (isx3(i) .eq. 1) then
            tesum = tesum + shift(i)
          else if (isx3(i) .eq. 5) then
            tmsum = tmsum + shift(i)
          end if
        end if
160   continue
c
c Use the sums to create the synthetic constraint data
      if (iscon.eq.0) then
c no constraint: do nothing
        continue
      else if (iscon .eq. 1) then
c constraint is to sum both TE and TM shifts together 
        dm(ndata+1) = tesum + tmsum
      else if (iscon .eq. 2) then
c constraint is to sum both TE and TM shifts separately 
        dm(ndata+1) = tesum
        dm(ndata+2) = tmsum
      end if
c
c Now compute the sums for the decomposition constraints (*0.01)
      do 260 i = 1, nstat
       do 260 j=1,4
        if (idfx(i,j) .ne. 0) then
c This parameter is free:
            if(j.eq.2) xymult = xymult + dcomp(i,j)
            if(j.eq.3) yxmult = yxmult + dcomp(i,j)
        end if
260   continue
c
c Use the sums to create the synthetic constraint data
      if (idcon.eq.0) then
c no constraint: do nothing
        continue
      else if (idcon .eq. 1) then
c constraint is to add TE and TM parameters together
        dm(ndata+1) = xymult+yxmult
       write(*,*) 'dm(last)in formod= ', dm(ndata+1)
      end if
c
c finally, handle the rotation, note theta is in radians
c data are handled in groups of 8 (common site,frequency, rotation angle)
      if (theta .ne. 0) then
        do 400 i=1,ndata,8
          arot = theta
          cose = cos(arot)
          sine = sin(arot)
          cos2 = cose*cose
          sin2 = sine*sine
          sico = sine*cose
          zxxr=dm(i)
          zxxi=dm(i+1)
          zxyr=dm(i+2)
          zxyi=dm(i+3)
          zyxr=dm(i+4)
          zyxi=dm(i+5)
          zyyr=dm(i+6)
          zyyi=dm(i+7)
          dm(i)  =cos2*zxxr-sico*zxyr-sico*zyxr+sin2*zyyr
          dm(i+1)=cos2*zxxi-sico*zxyi-sico*zyxi+sin2*zyyi
          dm(i+2)=sico*zxxr+cos2*zxyr-sin2*zyxr-sico*zyyr
          dm(i+3)=sico*zxxi+cos2*zxyi-sin2*zyxi-sico*zyyi
          dm(i+4)=sico*zxxr-sin2*zxyr+cos2*zyxr-sico*zyyr
          dm(i+5)=sico*zxxi-sin2*zxyi+cos2*zyxi-sico*zyyi
          dm(i+6)=sin2*zxxr+sico*zxyr+sico*zyxr+cos2*zyyr
          dm(i+7)=sin2*zxxi+sico*zxyi+sico*zyxi+cos2*zyyi
400     continue
      endif
c
      return
      end
c      
c-----------------------------------------------------------------------
      subroutine fordiv(np, nd, pm, dp, dm, wj)
c-----------------------------------------------------------------------
c
c OCCAM2DMT 1.0: Steven Constable IGPP/SIO La Jolla CA 92093-0225
c Subroutine Revision 1.01, 13 Jan 1993
c revision to allow for complex impedance - cdh - Feb23, 1994
c
c includes:
      include 'imp.inc'
      include 'mt2D.inc'
      include 'blkj.inc'
      include 'blkjc.inc'
c uses nfre, frek(), nrl, nrfix, y()
c uses rhte(), phte(), kzte(), rhtm(), phtm()

c arguments:
      real pm(npp), dp(ndd,3), dm(ndd), wj(ndd,npp)
      integer np, nd
c local variables
      real ppi, rad, tesum, tmsum, xymult, yxmult
c   pi and rad are useful constants
c   tesum = sum of logs of TE free static shifts 
c   tmsum = sum of logs of TM free static shifts 
      integer i, j, nexec, nres, itype, isite, ifreq, k,m
      real cose, sine, cos2, sin2, sico, c2ms2, arot
      real zxxr, zxxi, zxyr, zxyi, zyxr, zyxi, zyyr, zyyi
      real dzxxr,dzxxi,dzxyr,dzxyi,dzyxr,dzyxi,dzyyr,dzyyi
c   i,j = loop counters
c   nexec = instruction to PW2DI to perform only forward calculation
c   nres = number of resistivities in y() array
c   itype, isite, ifreq = integer versions of dp() entries
c   zetc   = place holders for data as they get rotated
c
c-----------------------------------------------------------------------
c total number of resistivities = fixed resistivities + free res'ties 
c (but not air)
      nres = nrfix + nbrick
      nexec = 2
      tesum = 0.0
      tmsum = 0.0
      xymult= 0.0
      yxmult= 0.0
c      
c copy across resistivities from parameter array to y()
      do 10 i = 1, nbrick
10        y(nrfix+1+i) = 10.**pm(i)
c copy across any static shifts in the free parameters
      if (nfss .gt. 0) then
        do 12 j = 1,nsb
          if (isfx(j) .ne. 0) shift(j) = pm(isfx(j))
12        continue
      end if
copy across any decomposition parameters in the free parameters
      if (nfdp .gt. 0) then
        do 22 i = 1,nstat
         do 22 j=1,4
          if (idfx(i,j) .ne. 0) dcomp(i,j) = pm(idfx(i,j))
22        continue
      end if
      write(*,'(4f10.4)') ((dcomp(i,j),j=1,4),i=1,nstat)
c 
      if(itfx .ne. 0) theta=pm(np)
c
c compute the MT response
      call PW2DI(nres,nexec)
c      
      ppi = 3.1415927
      rad = 180./ppi
c      
c strip off data:
c  the first column of dp() is the site number:
c  the second column of dp() is the frequency number:
c  the third column of dp() is the data type:
c  the fourth column of dp() is the input strike angle:
c   1 = TE resistivity
c   2 = TE phase
c   3 = real(tipper)
c   4 = imag(tipper)
c   5 = TM resistivity
c   6 = TM phase
c   7 = real(Zxx)                11 = real(Zyx)
c   8 = imag(Zxx)                12 = imag(Zyx)
c   9 = real(Zxy)                13 = real(Zyy)
c  10 = imag(Zxy)                14 = imag(Zyy)
c
c for every datum:
c derivatives from PW2DI are now with respect to log10(resistivity)
c no need to convert
      do 150 i = 1, ndata
        isite = int(dp(i,1)+0.1)
        ifreq = int(dp(i,2)+0.1)
        itype = int(dp(i,3)+0.1)
        if(itype.eq.1) then
          dm(i) = alog10(rhte(isite,ifreq)) 
          do 55 j = 1,nbrick
55          wj(i,j) = rted(j,isite,ifreq)
        else if(itype.eq.2) then
          dm(i) = phte(isite,ifreq)
          do 65 j = 1,nbrick
65          wj(i,j) = pted(j,isite,ifreq)
        else if(itype.eq.3) then
          dm(i) = real(kzte(isite,ifreq))
          do 75 j = 1,nbrick
75          wj(i,j) = real(kted(j,isite,ifreq))
        else if(itype.eq.4) then
          dm(i) = aimag(kzte(isite,ifreq))
          do 85 j = 1,nbrick
85          wj(i,j) = aimag(kted(j,isite,ifreq))
        else if(itype.eq.5) then
          dm(i) = alog10(rhtm(isite,ifreq)) 
          do 95 j = 1,nbrick
95          wj(i,j) = rtmd(j,isite,ifreq)
        else if(itype.eq.6) then
          dm(i) = phtm(isite,ifreq)
          do 105 j = 1,nbrick
105         wj(i,j) = ptmd(j,isite,ifreq)
        else if(itype.eq.7) then
          dm(i) = 0.
          do 205 j = 1,nbrick
205         wj(i,j) = 0.
        else if(itype.eq.8) then
          dm(i) = 0.
          do 215 j = 1,nbrick
215         wj(i,j) = 0.
        else if(itype.eq.9) then
          dm(i) = real(zimpe(isite,ifreq))
          do 225 j = 1,nbrick
225         wj(i,j) = real(zimped(j,isite,ifreq))
        else if(itype.eq.10) then
          dm(i) = aimag(zimpe(isite,ifreq))
          do 235 j = 1,nbrick
235         wj(i,j) = aimag(zimped(j,isite,ifreq))
        else if(itype.eq.11) then
          dm(i) = real(zimpm(isite,ifreq))
          do 245 j = 1,nbrick
245         wj(i,j) = real(zimpmd(j,isite,ifreq))
        else if(itype.eq.12) then
          dm(i) = aimag(zimpm(isite,ifreq))
          do 255 j = 1,nbrick
255         wj(i,j) = aimag(zimpmd(j,isite,ifreq))
        else if(itype.eq.13) then
          dm(i) = 0.
          do 265 j = 1,nbrick
265         wj(i,j) = 0.
        else if(itype.eq.14) then
          dm(i) = 0.
          do 275 j = 1,nbrick
275         wj(i,j) = 0.
        end if
c
c
c Handle the static shifts:
c Zero out any additional columns of the Jacobian matrix associated
c with free static shifts (there might not be a free shift for this
c datum)
        do 130 j = 1, nfss
130       wj(i,nbrick+j) = 0.0
c Search the static shifts to see if one applies to this datum:
        do 140 j = 1, nsb
          if ((isx1(j).eq.isite) .and. (isx3(j).eq.itype)) then
c apply the static shift
            dm(i) = dm(i) + shift(j)
c If this shift is a free parameter set the augmentation of WJ to 1.0
            if (isfx(j) .ne. 0) wj(i,isfx(j)) = 1.0
          end if
140     continue
c
150      continue
c
c Now compute the sums used in the ss constraint
      do 160 j = 1, nsb
        if (isfx(j) .ne. 0) then
c compute the sum of shifts for the shift constrains
          if (isx3(j) .eq. 1) then
            tesum = tesum + shift(j)
          else if (isx3(j) .eq. 5) then
            tmsum = tmsum + shift(j)
          end if
        end if
160      continue
c
c Now add rows to the Jacobian matrix for static shift 
c constraints and create the synthetic constraint data
      if (iscon.eq.0) then
c no constraint: do nothing
        continue
      else if (iscon .eq. 1) then
c constraint is to sum both TE and TM shifts together 
        dm(ndata+1) = tesum + tmsum
c zero out those cols of wj associated with model resistivities
        do 170 j = 1,nbrick
170       wj(ndata+1,j) = 0.0
c set those cols of wj associated with free static shifts (all ones
c for TE + TM summing)
        do 171 j = 1, nfss
171       wj(ndata+1,nbrick+j) = 1.0
      else if (iscon .eq. 2) then
c constraint is to sum both TE and TM shifts separately 
        dm(ndata+1) = tesum
        dm(ndata+2) = tmsum
c zero out those cols of wj associated with model resistivities
        do 180 j = 1,nbrick
          wj(ndata+1,j) = 0.0
180       wj(ndata+2,j) = 0.0
c set those cols of wj associated with free static shifts (need to
c test for either TE or TM sum)
        do 181 j = 1, nsb
          if (isfx(j) .ne. 0) then
            if (isx3(j) .eq. 1) then
c TE constraint
              wj(ndata+1,isfx(j)) = 1.0
              wj(ndata+2,isfx(j)) = 0.0
            else if (isx3(j) .eq. 5) then
c TM constraint
              wj(ndata+1,isfx(j)) = 0.0
              wj(ndata+2,isfx(j)) = 1.0
            end if
          end if
181     continue
      end if
c      
c Handle the decomposition matrix
c first multiply the derivatives for the blocks by the 
c decomposition parameters
c for every datum:
      if(ndcp .ne. 0) then
c these are derivatives wrt changing resistivity
      do 300 i = 1, ndata
        isite = int(dp(i,1)+0.1)
        ifreq = int(dp(i,2)+0.1)
        itype = int(dp(i,3)+0.1)
c Search the decomposition parameters to see if one applies to this datum
c and jacobean column
        do 311 j = 1, nstat
          if (j.eq.isite) then
             do 310 m=1,4
                if(itype.eq.7.and.m.eq.1) then 
                   dm(i) = dcomp(j,m)*real(zimpm(isite,ifreq))
                   do 320 k=1,nbrick
 320                  wj(i,k)=dcomp(j,m)*real(zimpmd(k,isite,ifreq))
                endif
                if(itype.eq.8.and.m.eq.1) then
                   dm(i)=dcomp(j,m)*aimag(zimpm(isite,ifreq))
                   do 325 k=1,nbrick
 325                  wj(i,k)=dcomp(j,m)*aimag(zimpmd(k,isite,ifreq))
                endif
                if(itype.eq.9.and.m.eq.2) then
                   dm(i)=dcomp(j,m)*real(zimpe(isite,ifreq))
                   do 330 k=1,nbrick
 330                  wj(i,k)=dcomp(j,m)*real(zimped(k,isite,ifreq))
                endif
                if(itype.eq.10.and.m.eq.2) then
                   dm(i)=dcomp(j,m)*aimag(zimpe(isite,ifreq))
                   do 335 k=1,nbrick
 335                  wj(i,k)=dcomp(j,m)*aimag(zimped(k,isite,ifreq))
                endif
                if(itype.eq.11.and.m.eq.3) then
                   dm(i)=dcomp(j,m)*real(zimpm(isite,ifreq))
                   do 340 k=1,nbrick
 340                  wj(i,k)=dcomp(j,m)*real(zimpmd(k,isite,ifreq))
                   endif
                if(itype.eq.12.and.m.eq.3) then
                   dm(i)=dcomp(j,m)*aimag(zimpm(isite,ifreq))
                   do 345 k=1,nbrick
 345                  wj(i,k)=dcomp(j,m)*aimag(zimpmd(k,isite,ifreq))
                endif
                if(itype.eq.13.and.m.eq.4) then
                   dm(i)=dcomp(j,m)*real(zimpe(isite,ifreq))
                   do 350 k=1,nbrick
 350                  wj(i,k)=dcomp(j,m)*real(zimped(k,isite,ifreq))
                endif
               if(itype.eq.14.and.m.eq.4) then
                   dm(i)=dcomp(j,m)*aimag(zimpe(isite,ifreq))
                   do 355 k=1,nbrick
 355                  wj(i,k)=dcomp(j,m)*aimag(zimped(k,isite,ifreq))
                endif
310          continue
          endif
311     continue
c these are derivatives wrt changing decomposition parameters
        do 360 j=1,nfdp
360       wj(i,nbrick+j) = 0.0
        do 365 j=1,nstat
         do 365 k=1,4
          if (idfx(j,k) .ne. 0 .and. j.eq.isite) then
            if(itype.eq.7.and.k.eq.1) 
     +             wj(i,idfx(j,k)) = real(zimpm(isite,ifreq))
            if(itype.eq.8.and.k.eq.1) 
     +             wj(i,idfx(j,k)) = aimag(zimpm(isite,ifreq))
            if(itype.eq.9.and.k.eq.2) 
     +             wj(i,idfx(j,k)) = real(zimpe(isite,ifreq))
            if(itype.eq.10.and.k.eq.2) 
     +             wj(i,idfx(j,k)) = aimag(zimpe(isite,ifreq))
            if(itype.eq.11.and.k.eq.3) 
     +             wj(i,idfx(j,k)) = real(zimpm(isite,ifreq))
            if(itype.eq.12.and.k.eq.3) 
     +             wj(i,idfx(j,k)) = aimag(zimpm(isite,ifreq))
            if(itype.eq.13.and.k.eq.4) 
     +             wj(i,idfx(j,k)) = real(zimpe(isite,ifreq))
            if(itype.eq.14.and.k.eq.4) 
     +             wj(i,idfx(j,k)) = aimag(zimpe(isite,ifreq))
          end if
365     continue
300   continue
      endif
c Now compute the sums used in the decomposition constraint
      do 370 j = 1, nstat
       do 370 k=1,4
        if (idfx(j,k) .ne. 0) then
            if(k.eq.2) xymult = xymult + dcomp(j,k)
            if(k.eq.3) yxmult = yxmult + dcomp(j,k)
        end if
370   continue
c
c Now add rows to the Jacobian matrix for decomposition
c constraints and create the synthetic constraint data
      if (idcon.eq.0) then
c no constraint: do nothing
        continue
      else if (idcon .eq. 1) then
c constraint is to add TE and TM sums together
        dm(ndata+1) = xymult + yxmult
       write(*,*) 'dm(last) in fordiv=',dm(ndata+1)
c zero out those cols of wj associated with model resistivities
        do 375 j = 1,nbrick
375       wj(ndata+1,j) = 0.0
c set those cols of wj associated with free decompostion
c parameters (=1 for summing)
        j=0
        do 380 k = 1, nstat
         do 380 m=1,4
         if(idfx(k,m).ne.0) then
           j=j+1
           if(m.eq.2.or.m.eq.3) wj(ndata+1,nbrick+j) = 1.0
         end if
380      continue
      end if
c
c finally, handle the rotation 
c zero out the column of jacobean dealing with derivatives wrt theta
      if (itfx .ne. 0) then
        do 399 i=1,nd
399       wj(i,np) = 0.0
      end if
c data are handled in groups of 8 (common site,frequency, rotation angle )
      if (theta .ne. 0) then
       do 400 i=1,ndata,8
         arot = theta
         cose=cos(arot)
         sine=sin(arot)
         cos2=cose*cose
         sin2=sine*sine
         sico=sine*cose
         c2ms2=cose*cose-sine*sine
         zxxr=dm(i)
         zxxi=dm(i+1)
         zxyr=dm(i+2)
         zxyi=dm(i+3)
         zyxr=dm(i+4)
         zyxi=dm(i+5)
         zyyr=dm(i+6)
         zyyi=dm(i+7)
         dm(i)=cos2*zxxr-sico*zxyr-sico*zyxr+sin2*zyyr
         dm(i+1)=cos2*zxxi-sico*zxyi-sico*zyxi+sin2*zyyi
         dm(i+2)=sico*zxxr+cos2*zxyr-sin2*zyxr-sico*zyyr
         dm(i+3)=sico*zxxi+cos2*zxyi-sin2*zyxi-sico*zyyi
         dm(i+4)=sico*zxxr-sin2*zxyr+cos2*zyxr-sico*zyyr
         dm(i+5)=sico*zxxi-sin2*zxyi+cos2*zyxi-sico*zyyi
         dm(i+6)=sin2*zxxr+sico*zxyr+sico*zyxr+cos2*zyyr
         dm(i+7)=sin2*zxxi+sico*zxyi+sico*zyxi+cos2*zyyi
c now do the rotations for all the derivatives wrt 
c resistivities and decomposition parameters as well
         do 410 k=1,nbrick+ndcp
            dzxxr = wj(i,k)
            dzxxi = wj(i+1,k)
            dzxyr = wj(i+2,k)
            dzxyi = wj(i+3,k)
            dzyxr = wj(i+4,k)
            dzyxi = wj(i+5,k)
            dzyyr = wj(i+6,k)
            dzyyi = wj(i+7,k)
            wj(i,k)=cos2*dzxxr-sico*dzxyr-sico*dzyxr+sin2*dzyyr
            wj(i+1,k)=cos2*dzxxi-sico*dzxyi-sico*dzyxi+sin2*dzyyi
            wj(i+2,k)=sico*dzxxr+cos2*dzxyr-sin2*dzyxr-sico*dzyyr
            wj(i+3,k)=sico*dzxxi+cos2*dzxyi-sin2*dzyxi-sico*dzyyi
            wj(i+4,k)=sico*dzxxr-sin2*dzxyr+cos2*dzyxr-sico*dzyyr
            wj(i+5,k)=sico*dzxxi-sin2*dzxyi+cos2*dzyxi-sico*dzyyi
            wj(i+6,k)=sin2*dzxxr+sico*dzxyr+sico*dzyxr+cos2*dzyyr
            wj(i+7,k)=sin2*dzxxi+sico*dzxyi+sico*dzyxi+cos2*dzyyi
410      continue
c now handle the derivatives wrt theta (final column of jacobean)
c if theta is sought
         if(itfx .ne. 0) then
           wj(i,np)=-2.*sico*zxxr-c2ms2*(zxyr+zyxr)+2.*sico*zyyr
           wj(i+1,np)=-2.*sico*zxxi-c2ms2*(zxyi+zyxi)+2.*sico*zyyi
           wj(i+2,np)=c2ms2*zxxr-2*sico*(zxyr+zyxr)-c2ms2*zyyr
           wj(i+3,np)=c2ms2*zxxi-2*sico*(zxyi+zyxi)-c2ms2*zyyi
           wj(i+4,np)=c2ms2*zxxr-2*sico*(zxyr+zyxr)-c2ms2*zyyr
           wj(i+5,np)=c2ms2*zxxi-2*sico*(zxyi+zyxi)-c2ms2*zyyi
           wj(i+6,np)=2.*sico*zxxr+c2ms2*(zxyr+zyxr)-2.*sico*zyyr
           wj(i+7,np)=2.*sico*zxxi+c2ms2*(zxyi+zyxi)-2.*sico*zyyi
         endif 
400    continue
      endif
c 
      write(*,*) 'np,nd=',np,nd
c
      return
      end
c     
c-----------------------------------------------------------------------
      SUBROUTINE OBJMAT(IRUF, DEL, NPEN, linkpr)
C-----------------------------------------------------------------------
C 
c OCCAM2DMT 1.0: Steven Constable IGPP/SIO La Jolla CA 92093-0225
c Subroutine Revision 1.01, 13 Jan 1993
C
C CONSTRUCTS PENALTY MATRIX (DEL) FOR 2DMT CASE.
C penalty is first derivative roughness of adjacent block depths
C 
C ON INPUT:
C   IRUF = (INTEGER) 1 FOR 1ST DERIVATIVE ROUGHNESS (ONLY TYPE SUPPORTED
C           BUT REQUIRED FOR COMPATIBILITY WITH OCCAM)
C ON OUTPUT:
C   DEL() = (REAL) PENALTY MATRIX
C   NPEN = (INTEGER) NUMBER OF NON-ZERO ROWS IN PENALTY MATRIX
C CALLS:
C   INITM
C
C-----------------------------------------------------------------------
      INCLUDE 'imp.inc'
      INCLUDE 'mt2D.inc'
C USES WIDTH()
      INCLUDE 'occam.inc'
C USES NP
C
C ARGUMENTS:
      REAL DEL(NNP,NPP)
      INTEGER IRUF, NPEN, linkpr(nnp,2)
C LOCAL VARIABLES
      INTEGER I, J, ind
      REAL PENALT
C   I,J = LOOP COUNTERS
C   PENALT = WEIGHTING OF PENALTY
C
C-----------------------------------------------------------------------
C ONLY FIRST DERIVATIVE SMOOTHING IS SUPPORTED FOR 2D
      IF (IRUF .NE. 1) THEN
        WRITE(*,*) 'ONLY FIRST DERIVATIVE SMOOTHING IS SUPPORTED FOR 2D'
        STOP
      END IF
C del has size [(nbig-1)*(lcol-1) by npinv]
      npen=(nbig-1)*(lcol-1)
c use next line if penalizing adjacent resistivities
c     npen=(nbig-1)*lcol
      CALL INITM(NNP,npen,npinv,DEL)
C EVERY NEW ROW OF DEL IS A NEW PENALTY; SET THE INITIAL COUNT TO ZERO
      NPEN = 0
c use next loop to smooth adjacent resistivities somewhat
c     write(*,*) 'penalizing adjacent resistivities'
c     do 30 i=1,nbig-1
c        npen=npen+1
c        del(npen,i) = -0.4
c        del(npen,i+1) = 0.4
c        linkpr(npen,1)=i
c        linkpr(npen,2)=i+1
c30   continue
C
C FOR EVERY unit boundary
      do 100 i = 1,nbig-1
C
C HORIZONTAL SMOOTHING:
C penalize adjacent depths
        do 20 j = 1,lcol-1
           npen = npen+1
c           penalt = 2./(width(j)+width(j+1))
           penalt=1.
           ind = nbig + (i-1)*lcol + j
           del(npen, ind) = -1.*penalt
           del(npen, ind+1) = 1.*penalt
           linkpr(npen,1)=ind
           linkpr(npen,2)=ind+1
20      continue
100   continue
c
      return
      end
C **********************************************************************
