C These subroutines handle matrix math, internal reads, and error messages
C-----------------------------------------------------------------------
      REAL FUNCTION ANORM(N,D)
C-----------------------------------------------------------------------
C RETURNS THE SQUARE OF THE EUCLIDEAN NORM OF A VECTOR
      INCLUDE 'imp.inc'
C
      INTEGER I, N
      REAL D(N)
C
      ANORM = 0.0
      DO 10 I = 1,N
        ANORM = ANORM + D(I)*D(I)
10    CONTINUE
      RETURN
      END
C
C
C-----------------------------------------------------------------------
      SUBROUTINE INITM(MD,M,N,A)
C-----------------------------------------------------------------------
C ZEROS A 2D MATRIX
      INCLUDE 'imp.inc'
C
      INTEGER MD, M, N, I, J
      REAL A(MD,N)
C
      DO 10 I = 1,M
        DO 10 J = 1,N
10        A(I,J) = 0.0
      RETURN
      END
C
C
C-----------------------------------------------------------------------
      SUBROUTINE MULT(MAD,MA,NAD,NA,NB,A,B,C)
C-----------------------------------------------------------------------
C MULTIPLIES TWO 2D MATRICES
      INCLUDE 'imp.inc'
C
      INTEGER MAD, MA, NAD, NA, NB, I, J, K
      REAL A(MAD,NA),B(NAD,NB),C(MAD,NB), CIJ
C
      DO 12 I = 1,MA
        DO 12 J = 1,NB
          CIJ = 0.0
          DO 10 K = 1,NA
10          CIJ = A(I,K)*B(K,J) + CIJ
12        C(I,J) = CIJ
      RETURN
      END
C
C
C-----------------------------------------------------------------------
      SUBROUTINE TRMULT(MAD,MA,NAD,NA,NB,A,B,C)
C-----------------------------------------------------------------------
C MULTIPLIES THE TRANSPOSE OF A 2D MATRIX BY ANOTHER MATRIX
      INCLUDE 'imp.inc'
C
      INTEGER MAD, MA, NAD, NA, NB, I, J, K
      REAL A(MAD,NA),B(MAD,NB),C(NAD,NB), CIJ
      DO 12 I = 1,NA
        DO 12 J = 1,NB
          CIJ = 0.0
          DO 10 K = 1,MA
10          CIJ = A(K,I)*B(K,J) + CIJ
12        C(I,J) = CIJ
      RETURN
      END
C
C
C-----------------------------------------------------------------------
      SUBROUTINE MINBRK(AX,BX,CX,FA,FB,FC,FUNC)
C-----------------------------------------------------------------------
C
C OCCAM 2.0: Steven Constable IGPP/SIO La Jolla CA 92093-0225
c Subroutine Revision 2.00, 13 May 1992
C
C MINBRK BRACKETS A UNIVARIATE MINIMUM OF A FUNCTION. TO BE USED PRIOR TO
C A UNIVARIATE MINIMISATION ROUTINE.
C BASED ON A ROUTINE IN NUMERICAL RECIPES BY PRESS ET AL
C MODIFIED SO THAT THE MODEL ASSOCIATED WITH THE MISFITS IS CARRIED AROUND
C FOR USE IN THE MINIMISATION ROUTINES AND POSSIBLY ULTIMATELY KEPT AS THE
C RESULT OF THIS ITERATION;
C
C ON INPUT:
C   AX, BX = TWO DISTINCT ESTIMATES OF THE MINIMUM'S WHEREABOUTS
C   FUNC = THE FUNCTION IN QUESTION
C   /MODEL/ NP = NUMBER OF MODEL PARAMETERS
C ON OUTPUT:
C   AX,BX,CX = THREE NEW POINTS WHICH BRACKET THE MINIMUM
C   FA,FB,FC = FUNC(AX), FUNC(BX) ETC.
C   /RESULT/ PWK1() = THE MODEL ASSOCIATED WITH BX AND FB
C
C INCLUDES:
      INCLUDE 'imp.inc'
      INCLUDE 'occamdim.inc'
      INCLUDE 'occam.inc'
      EXTERNAL FUNC
C
C ARGUMENTS:
      REAL AX,BX,CX,FA,FB,FC
C LOCAL VARIABLES:
      INTEGER I
      REAL DUM,R,Q,ULIM,U,FU
C LOCAL PARAMETERS
      REAL GOLD,GLIMIT,TINY
      PARAMETER (GOLD=1.618034, GLIMIT=100., TINY=1.E-32)
C FUNCTIONS:
      REAL FUNC
C
C-----------------------------------------------------------------------
      FB = FUNC(BX)
C MAKE PWK1 ASSOCIATED WITH B
      DO 4 I = 1,npinv
4       PWK1(I) = PWK2(I)
C PM2 WILL BE ASSOCIATED WITH A
      FA = FUNC(AX)
      IF (FB.GT.FA) THEN
C SWITCH A AND B
        DUM = AX
        AX = BX
        BX = DUM
        DUM = FB
        FB = FA
        FA = DUM
C KEEP PWK1 ASSOCIATED WITH B (A WILL BE LOST SO DOESN'T MATTER)
        DO 5 I = 1,npinv
5         PWK1(I) = PWK2(I)
      ENDIF
      CX = BX + GOLD*(BX - AX)
      FC = FUNC(CX)
C MAIN TEST. IF IT FAILS WE EXIT WITH B
10    IF (FB.GE.FC) THEN
C KEEP PWK1 ASSOCIATED WITH C 
        DO 11 I = 1,npinv
11        PWK1(I) = PWK2(I)
        R = (BX - AX)*(FB - FC)
        Q = (BX - CX)*(FB - FA)
        U = BX-((BX-CX)*Q-(BX-AX)*R)/(2.*SIGN(MAX(ABS(Q-R),TINY),Q-R))
        ULIM = BX + GLIMIT*(CX - BX)
        IF ((BX - U)*(U - CX).GT.0.) THEN
          FU = FUNC(U)
          IF (FU.LT.FC) THEN
            AX = BX
            FA = FB
            BX = U
            FB = FU
C KEEP PWK1 ASSOCIATED WITH B 
            DO 12 I = 1,npinv
12            PWK1(I) = PWK2(I)
            GO TO 10
          ELSE IF (FU.GT.FB) THEN
            CX = U
            FC = FU
            GO TO 10
          ENDIF
          U = CX + GOLD*(CX - BX)
          FU = FUNC(U)
        ELSE IF ((CX - U)*(U - ULIM).GT.0.) THEN
          FU = FUNC(U)
          IF (FU.LT.FC) THEN
            BX = CX
            CX = U
            U = CX + GOLD*(CX - BX)
            FB = FC
            FC = FU
C KEEP PWK1 ASSOCIATED WITH B 
            DO 13 I = 1,npinv
13            PWK1(I) = PWK2(I)
            FU = FUNC(U)
          ENDIF
        ELSE IF ((U - ULIM)*(ULIM - CX).GE.0.) THEN
          U = ULIM
          FU = FUNC(U)
        ELSE
          U = CX + GOLD*(CX - BX)
          FU = FUNC(U)
        ENDIF
        AX = BX
        BX = CX
        CX = U
        FA = FB
        FB = FC
        FC = FU
        GO TO 10
      ENDIF
      RETURN
      END
C
C-----------------------------------------------------------------------
      REAL FUNCTION FMIN(AX,BX,CX,FBX,F,TOL,XMIN)
C-----------------------------------------------------------------------
C
C OCCAM 2.0: Steven Constable IGPP/SIO La Jolla CA 92093-0225
c Subroutine Revision 2.00, 13 May 1992
C
C FMIN RETURNS THE MINIMUM VALUE OF A FUNCTION
C BASED ON A ROUTINE IN NUMERICAL RECIPES BY PRESS ET AL 
C MODIFIED SO THAT THE MODEL ASSOCIATED WITH THE MISFITS IS CARRIED AROUND
C FOR USE POSSIBLE ULTIMATE USE AS THE RESULT OF THIS ITERATION;
C
C ON INPUT:
C  AX,BX,CX = INDEPENDENT VARIABLE WHICH BRACKET THE MINIMUM
C  FBX = F(BX) (USUALLY AVAILABLE FROM THE BRACKETING PROCEDURE)
C  F = FUNCTION IN QUESTION
C  TOL = FRACTIONAL TOLERANCE REQUIRED IN THE INDEPENDENT VARIABLE
C  /RESULT/ PWK1 = MODEL ASSOCIATED WITH BX AND FBX
C  /bound/ npinv = NUMBER OF MODEL PARAMETERS
C ON OUTPUT:
C  XMIN = ABSCISSA OF MINIMUM
C  FMIN = F(XMIN)
C  /RESULT/ PWK1 = MODEL ASSOCIATED WITH XMIN AND FMIN
C
C INCLUDES:
      INCLUDE 'imp.inc'
      INCLUDE 'occamdim.inc'
      INCLUDE 'occam.inc'
      EXTERNAL F
C
C ARGUMENTS:
      REAL AX,BX,CX,FBX,TOL,XMIN
C LOCAL VARIABLES
      REAL AA,B,V,W,X,E,FX,FV,FW,TOL1,Q,R,P,ETEMP,DD,FU,U,XM,TOL2
      INTEGER I,ITER
C LOCAL PARAMETERS
      INTEGER ITMAX
      REAL CGOLD,ZEPS
c      PARAMETER (ITMAX=100,CGOLD=.3819660,ZEPS=1.0E-10)
      PARAMETER (ITMAX=100,CGOLD=.3819660,ZEPS=0.01)
C FUNCTION:
      REAL F
C
C-----------------------------------------------------------------------
      AA = MIN(AX,CX)
      B = MAX(AX,CX)
      V = BX
      W = V
      X = V
      E = 0.
      FX = FBX
      FV = FX
      FW = FX
      DO 11 ITER = 1,ITMAX
        XM = 0.5*(AA + B)
        TOL1 = TOL*ABS(X) + ZEPS
        TOL2 = 2.*TOL1
        IF (ABS(X - XM).LE.(TOL2 - .5*(B - AA))) GOTO 30
        IF (ABS(E).GT.TOL1) THEN
          R = (X - W)*(FX - FV)
          Q = (X - V)*(FX - FW)
          P = (X - V)*Q - (X - W)*R
          Q = 2.*(Q - R)
          IF (Q.GT.0.) P =  - P
          Q = ABS(Q)
          ETEMP = E
          E = DD
          IF (ABS(P).GE.ABS(.5*Q*ETEMP).OR.P.LE.Q*(AA - X) .OR.
     *        P.GE.Q*(B - X)) GOTO 1
          DD = P/Q
          U = X + DD
          IF (U - AA.LT.TOL2 .OR. B - U.LT.TOL2) DD = SIGN(TOL1,XM - X)
          GOTO 2
        ENDIF
1       IF (X.GE.XM) THEN
          E = AA - X
        ELSE
          E = B - X
        ENDIF
        DD = CGOLD*E
2       IF (ABS(DD).GE.TOL1) THEN
          U = X + DD
        ELSE
          U = X + SIGN(TOL1,DD)
        ENDIF
        FU = F(U)
        IF (FU.LE.FX) THEN
          IF (U.GE.X) THEN
            AA = X
          ELSE
            B = X
          ENDIF
          V = W
          FV = FW
          W = X
          FW = FX
          X = U
          FX = FU
          DO 5 I = 1,npinv
5           PWK1(I) = PWK2(I)
        ELSE
          IF(U.LT.X) THEN
            AA = U
          ELSE
            B = U
          ENDIF
          IF (FU.LE.FW .OR. W.EQ.X) THEN
            V = W
            FV = FW
            W = U
            FW = FU
          ELSE IF (FU.LE.FV .OR. V.EQ.X .OR. V.EQ.W) THEN
            V = U
            FV = FU
          ENDIF
        ENDIF
11    CONTINUE
      WRITE(*,*) 'MAXIMUM ITERATIONS EXCEEDED IN FMIN'
30    XMIN = X
      FMIN = FX
      RETURN
      END
C
C-----------------------------------------------------------------------
      REAL FUNCTION FROOT(FUNC,X1,X2,FA,FB,OFF,TOL)
C-----------------------------------------------------------------------
C
C OCCAM 2.0: Steven Constable IGPP/SIO La Jolla CA 92093-0225
c Subroutine Revision 2.00, 13 May 1992
C
C FROOT FINDS THE POINT AT WHICH A UNIVARIATE FUNCTION ATTAINS A GIVEN
C  VALUE
C BASED ON A ROUTINE IN NUMERICAL RECIPES BY PRESS ET AL 
C MODIFIED SO THAT THE MODEL ASSOCIATED WITH THE MISFITS IS CARRIED AROUND
C FOR USE POSSIBLE ULTIMATE USE AS THE
C RESULT OF THIS ITERATION;
C
C ON INPUT:
C  FUNC = THE FUNCTION IN QUESTION
C  X1,X2 = INDEPENDENT VARIABLES BRACKETING THE ROOT
C  FA,FB = FUNC(X1),FUNC(X2) (USUALLY AVAILABLE FROM THE BRACKETING)
C  OFF = VALUE OF THE FUNCTIONAL AT THE REQUIRED ABSCISSA
C  TOL = FRACTIONAL TOLERANCE REQUIRED FOR ABSCISSA
C  /RESULT/ PWK2 = MODEL ASSOCIATED WITH X2 AND FB
C  /RESULT/ PWK1 = MODEL ASSOCIATED WITH X1 AND FA
C  /MODEL/ NP = NUMBER OF MODEL PARAMTERS
C
C ON OUTPUT:
C  FB = FUNC(FROOT) - OFF
C  FROOT = ABSCISSA REQUIRED
C  /RESULT/ PWK2 = MODEL ASSOCIATED WITH FB, FROOT
C
C INCLUDES:
      INCLUDE 'imp.inc'
      INCLUDE 'occamdim.inc'
      INCLUDE 'occam.inc'
      EXTERNAL FUNC
C
C ARGUMENTS:
      REAL X1,X2,FA,FB,OFF,TOL
C LOCAL VARIABLES
      REAL AA,B,FC,C,E,DD,TOL1,P,Q,R,S,XM
      INTEGER I,ITER
C LOCAL PARAMETERS
      INTEGER ITMAX
      REAL EPS
      PARAMETER (ITMAX = 100,EPS = 3.E-4)
C FUNCTIONS:
      REAL FUNC
C
C-----------------------------------------------------------------------
      AA = X1
      B = X2
      FA = FA - OFF
      FB = FB - OFF
      IF (FB*FA.GT.0.) WRITE(*,*) 'ROOT NOT BRACKETED IN FROOT'
      FC = FB
C MODELS WILL BE CARRIED AROUND AS:
C   AA: PWK1()
C   B: PWK2()
C   C: PWK3()
C SO COPY PM2 (WITH B) INTO WK (WITH C)
      DO 1 I = 1,npinv
1       PWK3(I) = PWK2(I)
      DO 11 ITER = 1,ITMAX
C IF B AND C ARE NOT ON DIFFERENT SIDES OF ROOT THEN MAKE THEM  SO
        IF (FB*FC.GT.0.) THEN
          C = AA
          FC = FA
          DD = B - AA
          E = DD
C COPY PMF (WITH AA) INTO WK (WITH C)
          DO 2 I = 1,npinv
2           PWK3(I) = PWK1(I)
        ENDIF
        IF (ABS(FC).LT.ABS(FB)) THEN
C IF B IS NOT CLOSER THAN C MAKE IT SO
          AA = B
          B = C
          C = AA
          FA = FB
          FB = FC
          FC = FA
C SWITCH PM2 (WITH B) AND WK (WITH C) (AA WILL BE LOST IF WE DON'T EXIT)
          DO 3 I = 1,npinv
            PWK1(I) = PWK2(I)
            PWK2(I) = PWK3(I)
3           PWK3(I) = PWK1(I)
        ENDIF
        TOL1 = 2.*EPS*ABS(B) + 0.5*TOL
        XM = .5*(C - B)
        IF (ABS(XM).LE.TOL1 .OR. FB.EQ.0.)  THEN
          FROOT = B
          RETURN
        ENDIF
        IF (ABS(E).GE.TOL1 .AND. ABS(FA).GT.ABS(FB)) THEN
          S = FB/FA
          IF (AA.EQ.C) THEN
            P = 2.*XM*S
            Q = 1. - S
          ELSE
            Q = FA/FC
            R = FB/FC
            P = S*(2.*XM*Q*(Q - R) - (B - AA)*(R - 1.))
            Q = (Q - 1.)*(R - 1.)*(S - 1.)
          ENDIF
          IF (P.GT.0.) Q =  - Q
          P = ABS(P)
          IF (2.*P .LT. MIN(3.*XM*Q - ABS(TOL1*Q),ABS(E*Q))) THEN
            E = DD
            DD = P/Q
          ELSE
            DD = XM
            E = DD
          ENDIF
        ELSE
          DD = XM
          E = DD
        ENDIF
        AA = B
        FA = FB
C COPY PM2 (WITH B) INTO PMF (WITH AA)
          DO 4 I = 1,npinv
4           PWK1(I) = PWK2(I)
        IF (ABS(DD) .GT. TOL1) THEN
          B = B + DD
        ELSE
          B = B + SIGN(TOL1,XM)
        ENDIF
C THE NEW FUNCION EVALUATION AUTOMATICALLY USES PM2 FOR B
        FB = FUNC(B) - OFF
11    CONTINUE
      WRITE(*,*) 'MAXIMUM ITERATIONS EXCEEDED IN FROOT'
      FROOT = B
      RETURN
      END
C
C-----------------------------------------------------------------------
      SUBROUTINE CHOLSL(LA,N, A, X, B, ISTAGE)
C-----------------------------------------------------------------------
C
C OCCAM 2.0: Steven Constable IGPP/SIO La Jolla CA 92093-0225
C WRITTEN BY:   R.L. PARKER
C
C$$$$$ CALLS NO OTHER ROUTINES
C  ROUTINE FOR BACK-SUBSTITUTION SOLUTION OF A LINEAR SYSTEM THAT HAS
C  ALREADY BEEN FACTORED INTO CHOLESKY FACTORS BY 'CHOLIN' (Q.V.)
C  LA      LEADING DIMENSION OF A
C  N       THE ORDER OF MATRIX
C  A       ARRAY CONTAINING ON AND ABOVE ITS DIAGONAL THE TRANSPOSE OF
C          THE CHOLESKY FACTOR.  THE NORMAL WAY OF FINDING THE FACTORS
C          IS TO CALL  CHOLIN  AS FOLLOWS
C     CALL CHOLIN(N, A, 1, IFBAD)
C     CALL CHOLSL(N, A, X, B, 0)
C  X       THE UNKNOWN VECTOR.  SOLUTION TO EQUATIONS IS RETURNED HERE
C  B       THE RIGHT SIDE OF SYSTEM
C  ISTAGE  THE MODE OF THE SOLUTION.  IF=1 DO NOT COMPLETE BACK-
C     SUBSTITUTION.  INSTEAD RETURN INVERSE(L-TRANS)*B .  OTHER VALUES
C     FORM THE COMPLETE SOLUTION.
C R.L. PARKER
C
C INCLUDES:
      INCLUDE 'imp.inc'
C ARGUMENTS
      INTEGER LA,N,ISTAGE
      REAL A(LA,N+1),X(N),B(N)
C LOCAL VARIABLES
      INTEGER I,K,II,KK,I1
      REAL Z
C
C-----------------------------------------------------------------------
      DO 2500 I=1,N
      Z=B(I)
      IF (I.EQ.1) GO TO 2500
      DO 2200 KK=2,I
      K=I-KK+1
 2200 Z=Z - A(K,I+1)*X(K)
 2500 X(I)=Z/A(I,I+1)
      IF (ISTAGE.EQ.1) RETURN
C  COMPLETE BACK-SUBSTITUTION
      DO 2700 II=1,N
      I=N-II+1
      Z=X(I)
      I1=I+1
      IF (I.EQ.N) GO TO 2700
      DO 2600 K=I1,N
 2600 Z= Z - A(I,K+1)*X(K)
 2700 X(I)=Z/A(I,I+1)
      RETURN
      END                                                              
C
C-----------------------------------------------------------------------
      SUBROUTINE CHOLIN(LA,N, A, JSTAGE, IFBAD)
C-----------------------------------------------------------------------
C
C OCCAM 2.0: Steven Constable IGPP/SIO La Jolla CA 92093-0225
C WRITTEN BY:   R.L. PARKER
C
C$$$$$ CALLS NO OTHER ROUTINES
C  INVERSION AND FACTORIZATION OF POSITIVE DEFINITE SYMMETRIC MATRICES
C  FROM ALGOL ROUTINE OF WILKINSON & REINSCH 'LINEAR ALGEBRA' PP16-17.
C  EXPLANATION
C  CHOLIN  CALCULATES THE FACTOR  L  IN THE CHOLESKY FACTORIZATION
C  A = L L-TRANS, OR THE INVERSE OF L , OR THE INVERSE OF  A.
C  TRANSPOSED STORAGE IS FOR THE CONVENIENCE OF FORTRAN USERS.  HOWEVER,
C  IT MEANS THAT L-TRANS AND INVERSE OF L-TRANS ARE COMPUTED.
C  ARGUMENTS
C    A     AN ARRAY DIMENSION N,N+1.  ON ENTRY THE INPUT MATRIX SHOULD
C          SHOULD OCCUPY THE 1ST  N COLS, OR IF DESIRED, JUST THE LOWER
C          LOWER LEFT TRIANGLE.  ON EXIT THE UPPER TRIANGLE OF THE ARRAY
C          DEFINED BY COLS 2,3...N+1 CONTAINS THE DESIRED RESULT.
C          THIS CAN BE CONVENIENTLY ACCESSED AS A(1,2), OR A(N+1) IF
C          SINGLY DIMENSIONED IN CALLING PROGRAM.
C    N     THE ORDER OF THE MATRIX
C    LA    LEADING DIMENSION OF A
C  JSTAGE  INDICATES DESIRED CALCULATION - IF=1 UPPER TRIANGLE CONTAINS
C          L-TRANS.  IF=2 UPPER TRIANGLE CONTAINS INVERSE OF L-TRANS.
C          IF=0 OR 3 CONTAINS THE UPPER HALF OF THE INVERSE OF A.  NOTE
C          LOWER TRIANGLE IS NOT DESTROYED. IF=-2,-3, THE CHOLESKY
C          FACTORS ARE ASSUMED TO BE IN PLACE ON ENTRY.
C  IFBAD   ERROR FLAG, SET TO 1 IF MATRIX NOT POSITIVE DEFINITE. SET TO
C          0 IF OK.
C R.L. PARKER
C
C INCLUDES:
      INCLUDE 'imp.inc'
C PASS PARAMETERS
      INTEGER LA,N,JSTAGE,IFBAD
      REAL A(LA,N+1)
C LOCAL VARIABLES
      INTEGER I,J,K,KBACK,ISTAGE,I1,J1,J2,N1
      REAL X,Y,Z
C
C-----------------------------------------------------------------------
C  STAGE 1 - FORM FACTORS (HERE L-TRANS IS STOTED ABOVE DIAGONAL)
      ISTAGE=IABS(JSTAGE)
      IF (JSTAGE.LT.0) GO TO 1550
      IFBAD=0
      DO 1500 I=1,N
      I1=I + 1
      DO 1400 J=I,N
      J1=J + 1
      X=A(J,I)
      IF (I.EQ.1) GO TO 1200
      DO 1100 KBACK=2,I
      K=I-KBACK+1
 1100 X=X - A(K,J1)*A(K,I1)
 1200 IF (I.NE.J) GO TO 1300
      IF (X.LE.0.0) GO TO 4000
      Y=1.0/SQRT(X)
      A(I,I1)=Y
      GO TO 1400
 1300 A(I,J1)=X*Y
 1400 CONTINUE
 1500 CONTINUE
      IF (ISTAGE.NE.1) GO TO 2000
 1550 DO 1600 I=1,N
 1600 A(I,I+1)=1.0/A(I,I+1)
      IF (ISTAGE.EQ.1) RETURN
C  STAGE 2 - INVERSION OF L-TRANS
 2000 DO 2600 I=1,N
      I1=I+1
      IF (I1.GT.N) GO TO 2600
      DO 2500 J=I1,N
      Z=0.0
      J1=J + 1
      J2=J - 1
      DO 2200 KBACK=I,J2
      K=J2-KBACK+I
 2200 Z=Z - A(K,J1)*A(I,K+1)
      A(I,J1)=Z*A(J,J1)
 2500 CONTINUE
 2600 CONTINUE
      IF (ISTAGE.EQ.2) RETURN
C  STAGE 3  - CONSTRUCTION OF IVERSE OF  A  ABOVE DIAGONAL.
      DO 3500 I=1,N
      DO 3500 J=I,N
      Z=0.0
      N1=N + 1
      J1=J + 1
      DO 3200 K=J1,N1
 3200 Z=Z + A(J,K)*A(I,K)
      A(I,J+1)=Z
 3500 CONTINUE
      RETURN
C  ERROR EXIT
 4000 IFBAD=1
C ERROR MESSAGE SUPRESSED - S.CONSTABLE
C      WRITE(*,400)
C 400  FORMAT(44H0CHOLIN FAILS - MATRIX NOT POSITIVE DEFINITE           )
      RETURN
      END  
C
C-----------------------------------------------------------------------
      SUBROUTINE ATAMUL(MAD,MA,NAD,NA,A,C)
C-----------------------------------------------------------------------
      INCLUDE 'imp.inc'
C OCCAM 2.0: Steven Constable IGPP/SIO La Jolla CA 92093-0225
C WRITTEN BY:  C. DEGROOT-HEDLIN   
C
C DESCRIPTION: 
C
C MULTIPLIES THE TRANSPOSE OF A 2D MATRIX BY ITSELF
C TAKES ADVANTAGE OF SYMMETRY ABOUT DIAGONAL
C
C
C ARGUMENTS
      INTEGER MAD,MA,NAD,NA
      REAL A(MAD,NA),C(NAD,NA)
C
C LOCAL VARIABLES
      INTEGER I,J,K
      REAL CIJ
C
C-----------------------------------------------------------------------
      DO 12 I = 1,NA
        DO 12 J = I,NA
          CIJ = 0.0
          DO 10 K = 1,MA
10          CIJ = A(K,I)*A(K,J) + CIJ
12        C(I,J) = CIJ
      DO 15 I=2,NA
        DO 15 J=1,I-1
15        C(I,J) = C(J,I)
      RETURN
      END
C
C-----------------------------------------------------------------------
      real function rdcode(string, iof2)
C-----------------------------------------------------------------------
C
C OCCAM 2.0: Steven Constable IGPP/SIO La Jolla CA 92093-0225
c Subroutine Revision 2.00, 13 Jan 1993
C
c rdcode provides a patch to get around the lack of an internal
c free-format (list-directed) read on some compilers, notably the
c cray.  This version for reals.  Butchered from decode by Bob Parker.
c 
c on input:
c    string = character string containing number to be read
c    iof2 = unit number of open file, to be closed on error 
c on output:
c    rdcode contains a real number from string
c calls:
c    filerr
c
c includes:
      include 'imp.inc'
c input arguments:
      character*(*) string
	integer iof2
c local variables
      character*40 local
	integer k, k1, kn, l
c
c  Terminate line at % sign
      kn=index(string//'%', '%') - 1
      k1=1
      do 1100 k=k1, kn
        if (string(k:k) .ne. ' ') goto 1200
 1100 continue
      call filerr (' No number found by rdcode', iof2)
 1200 do 1300 l=k, kn
        if (string(l:l).eq. ',' .or. string(l:l) .eq. ' ') goto 1500
 1300 continue
 1500 local=string(k:l-1)
      read (local, '(f40.0)', err=1900) rdcode
	return
 1900 call filerr (' Error reading string in rdcode', iof2)
      end
C-----------------------------------------------------------------------
      integer function idcode(string, iof2)
C-----------------------------------------------------------------------
C
C OCCAM 2.0: Steven Constable IGPP/SIO La Jolla CA 92093-0225
c Subroutine Revision 2.00, 13 Jan 1993
C
c idcode provides a patch to get around the lack of an internal
c free-format (list-directed) read on some compilers, notably the
c cray.  This version for integers.  Butchered from decode by Bob Parker.
c 
c on input:
c    string = character string containing number to be read
c    iof2 = unit number of open file, to be closed on error 
c on output:
c    idcode contains an integer number from string
c calls:
c    filerr
c
c includes:
      include 'imp.inc'
c input arguments:
      character*(*) string
	integer iof2
c local variables
      character*40 local
	integer k, k1, kn, l
	real realno
c
c  Terminate line at % sign
      kn=index(string//'%', '%') - 1
      k1=1
      do 1100 k=k1, kn
        if (string(k:k) .ne. ' ') goto 1200
 1100 continue
      call filerr (' No number found by idcode', iof2)
 1200 do 1300 l=k, kn
        if (string(l:l).eq. ',' .or. string(l:l) .eq. ' ') goto 1500
 1300 continue
 1500 local=string(k:l-1)
      read (local, '(f40.0)', err=1900) realno
	idcode = int(realno)
	return
 1900 call filerr (' Error reading string in idcode', iof2)
      end
C-----------------------------------------------------------------------
      subroutine filerr(mssg, io1)
C-----------------------------------------------------------------------
C
C OCCAM 2.0: Steven Constable IGPP/SIO La Jolla CA 92093-0225
c Subroutine Revision 2.00, 13 May 1992
C
c filerr prints an error message, closes file io1, and stops
c
c on input:
c    mssg = character string containing error message
c    i01 = unit number of file to close (0 = no file open)
c on output:
c    outputs nothing
c calls:
c    no other routines
c
c
c includes:
      include 'imp.inc'
c input arguments:
      character*(*) mssg
      integer io1
c
C-----------------------------------------------------------------------
      write(*,*) mssg
      if (io1 .gt. 0) close (io1)
      stop
      end
C-----------------------------------------------------------------------
      subroutine datime(datetm)
c-----------------------------------------------------------------------
c OCCAM 2.0: Steven Constable IGPP/SIO La Jolla CA 92093-0225
c Subroutine Revision 1.00, 18 May 1992
c
c Dummy date and time utility for OCCAM 2.0.  
c
c on input:
c   nothing
c on output:
c   datetm = 80 character string containing message saying that date and time
c           are not available
c      
      character*80 datetm
c      
c-----------------------------------------------------------------------
      datetm = 'No date and time available on this system'      
      return
      end
c-----------------------------------------------------------------------
