c code written by Phil Wannamaker
C **********************************************************************
      SUBROUTINE PW2DI(NRES,NEXEC)
C **********************************************************************
      INCLUDE 'imp.inc'
C
C========================== INCLUDE FILES ==============================
C
      INCLUDE 'mt2D.inc'
      include 'blke.inc'
      include 'blkf.inc'
      include 'blki.inc'
      include 'blkjc.inc'
      include 'blkl.inc'
      include 'blkd.inc'
C
C============================ INTERFACE ================================
C
C WRITTEN BY:  P. WANNAMAKER, UURI.
C
C DESCRIPTION: 
C
C     FOR THE FORWARD PROBLEM AND JACOBIANS,
C     FINITE ELEMENTS ARE USED WITH LINEAR BASIS FUNCTIONS.  TOPO-
C     GRAPHIC OR BATHYMETRIC VARIATIONS ACROSS THE PROFILE CAN BE
C     INCLUDED AS WELL (WANNAMAKER ET AL, 1986, GEOPHYSICS).  MKS
C     UNITS AND EXP(+JWT) TIME DEPENDENCE ARE ASSUMED.  THE + COORD-
C     INATE DIRECTIONS FOR THE PROBLEM ARE X- NORTH, Y-EAST, Z-DOWN.
C     STRIKE DIRECTION IS ASSUMED N-S.  ORIGIN IS AT A USER-SPECIFIED
C     NODAL COLUMN AT THE HIGHEST TOPOGRAPHIC POINT.  A SECONDARY
C     FIELD FORMULATION OF THE FINITE ELEMENT EQUATIONS HAS BEEN
C     UTILIZED HERE TO IMPROVE ACCURACY (WANNAMAKER ET AL, 1987, GEOP.
C     J. ROY. ASTR. SOC.).  THE LAYERED HOST FOR THE INHOMOGENEITY BY
C     CONVENTION IS TAKEN TO BE THAT WITH THE HIGHER RESISTIVITY IN
C     THE UPPERMOST LAYER.  IN THIS VERSION, THE PARAMETER JACOBIANS
C     ARE COMPUTED BY TAKING ADVANTAGE OF RECIPROCITY.  THERE IS A
C     REPORT DOCUMENTATION BY WANNAMAKER (1990) DESCRIBING THE METHOD.
C
C     ROUTINES NEEDED:  PW2DI, PARFLD, COAMP, ZLFLD, GAUSS, AUXFLD,
C                       BNORM, DERIV, APPARMT, addair, RECIJAC, PARJAC, 
C                       GAUSSD, AUXADJ, APPRMTJ, DRVADJ
C
C ON INPUTS:
C         NRES - NO. OF UNIQUE RESISTIVITY MEDIA IN THE MESH
C                (INCLUDES ANY SEAWATER BUT NOT THE AIR)
C         NFRE - NO. OF FREQUENCIES TO BE MODELED
C         NEXEC - IF .EQ. 0, JUST PRINT INPUT PARAMETERS AND MESH
C                 TO CHECK CORRECTNESS
C               - IF .EQ. 1, COMPUTE RESPONSE
C               - IF .EQ. 2, INVERT DATA
C         Y - ARRAY OF RESISTIVITIES OF MEDIA MAKING UP THE MESH;
C             LAST VALUE IS THAT OF ANY SEAWATER.
C         FREK - ARRAY OF FREQUENCIES
C         NRL - SITE LOCATIONS BY FE MESH NODE NUMBER
C
C         /BLKI/ IDX - PARAMETER DEFINING MODE OF PROBLEM TO BE SOLVED
C             - IF .EQ. 1, SOLVE BOTH TE AND TM
C             - IF .EQ. 2, SOLVE TE ONLY
C             - IF .EQ. 3, SOLVE TM ONLY
C         /BLKI/ NODEY - NO. OF NODES IN THE Y-DIRECTION OF THE FINITE
C                 ELEMENT MESH
C         /BLKI/ NODEZ - NO. OF NODES IN THE Z-DIRECTION OF THE FINITE
C                 ELEMENT MESH
C         /BLKI/ NPRM - NO. OF POLYGONAL MEDIA OF INVERSION MODEL
C         /BLKI/ LAPRM - INTEGER LABELS OF EACH INVERSION MEDIUM
C         /BLKD/ DLY - ARRAY OF ELEMENT COLUMN WIDTHS
C         /BLKD/ DLZ - ARRAY OF ELEMENT ROW HEIGHTS
C         /BLKE/ NPT - INTEGER LABELS FOR EACH TRIANGULAR ELEMENT OF EACH 
C               RECTANGULAR ELEMENT OF THE MESH.  THESE LABELS ARE 
C               ASSIGNED THE VALUES IN THE RESISTIVITY ARRAY Y.  
C               LABEL 0 IS RESERVED FOR AIR AND LABEL -1 IS RESERVED
C               FOR SEAWATER.  TRIANGULAR ELEMENT LABELING ORDER IS 
C               TOP, LEFT, BOTTOM AND RIGHT.
C
C         STORAGE PARAMETERS:
C         IPY - MAX. NO. OF NODES IN THE Y-DIRECTION.  
C         IPZ - MAX. NO. OF NODES IN THE Z-DIRECTION. 
C         IPI - MAX. NO. OF POLYGONAL MEDIA IN AN INVERSION MODEL. 
C
C============== PARAMETER AND LOCAL VARIABLE DECLARATIONS ==============
C
C     ---- PASS PARAMETERS
      INTEGER NRES,NEXEC
C
C     ---- LOCAL VARIABLES
c rj converted to local variable for dynamic memory allocation
      COMPLEX RJ(IPI,IP2)
c      COMPLEX FT,PF,VF,HF,TP,ZIMP,FTD,ZIMPD,KZYD
      INTEGER lidx,NODEY1,MAIR,NODEZ1
      INTEGER I,J,K,L,M,III,NYRB
c      REAL RAD,XEO,XMUO,AIOR
      real conds(0:ipi36)
      COMPLEX S(IP2,IP3),R(IP2)
C
C
C=======================================================================
C
C     ----ZERO OUT LOCAL S(), R() AND IF INVERTING, PARAMETER RJ()
      DO 1000 J=1,IP3
        DO 1000 I=1,IP2
          S(I,J)=CMPLX(0.,0.)
 1000 CONTINUE
      DO 1001 J=1,IP2
        R(J)=CMPLX(0.,0.)
 1001 CONTINUE
      IF (NEXEC.EQ.2) then
      DO 1002 J=1,IP2
        DO 1002 I=1,IPI
          RJ(I,J)=CMPLX(0.,0.)
 1002   CONTINUE
      END IF
C        
      NODEY1 = NODEY - 1
C
C     ---- DEFINE LOCATION OF Y=0 AT CENTER OF MESH
      YY(1) = 0.0
      DO 20 I = 2,NODEY
         YY(I) = YY(I-1) + DLY(I-1)
20    CONTINUE
C
      DO 30 I = 1,NODEY
        YY(I) = YY(I) - YY(NODEY)/2.
30    CONTINUE
c
c add the air layer if necessary
      lidx = idx
      call addair(lidx, mair)
      NODEZ1 = NODEZ - 1
      NTR = MAIR + 1
C
C     ---- CHANGE FROM RESISTIVITIES INPUT TO CONDUCTIVITIES AND
C          ASSOCIATE CONDUCTIVITIES WITH THE MESH INPUT CODE
c      RA = 1.0E+18
      conds(1) = 1.0E-18  ! air conductivity
      conds(0) = 3.25     ! seawater conductivity
      DO 90 I = 2,NRES+1
        conds(i) = 1./Y(I)
90    CONTINUE
C
c      ISEA=NRES+1
      ISEA = -1   !Changed by Kerry, Aug 17, 2001
c      conds(ISEA+1) = 1./Y(NRES+1)
      write(*,*) 'isea,conds(isea+1)=',isea,conds(isea+1)
      DO 100 J = 1,NODEZ1
        DO 101 L = 1,NODEY1
          DO 102 M = 1,4
            K = NPT(J,M,L)
            SIG(J,M,L) = conds(K+1)
102       CONTINUE
101     CONTINUE
c        write(*,*) 'layer',j
c        write(*,'(12f6.2)') (sig(j,1,l),l=1,nodey1)
100   CONTINUE
C
C     ---- POSITION THE DATUM (HIGHEST TOPOGRAPHIC POINT)
      L = NTR
      ZZ(L) = 0.
      DO 110 I = NTR,NODEZ1
        ZZ(L+1) = ZZ(L) + DLZ(I)
        L = L + 1
110   CONTINUE
C
      IF(MAIR .NE. 0)THEN
        L = NTR
        DO 130 III = 1,MAIR
          I = MAIR + 1 - III
          ZZ(L-1) = ZZ(L) - DLZ(I)
          L = L - 1
130     CONTINUE
      END IF
C
C     ---- SPECIFY NODAL ROW DEPRESSIONS OF TOPO/BATH SURFACE.
      DO 140 I = 2,NODEY1
        NTAO(I) = 0
        DO 141 J = NTR,NODEZ1
          IF(NPT(J,4,I-1) .EQ. 0  .AND. NPT(J,2,I) .EQ. 0 )GO TO 142
          IF(NPT(J,4,I-1) .EQ. ISEA .AND. NPT(J,2,I) .EQ. ISEA)GO TO 142
          GO TO 140
142       CONTINUE
          NTAO(I) = J - MAIR
141     CONTINUE
140   CONTINUE
      NTAO(1) = NTAO(2)
      NTAO(NODEY) = NTAO(NODEY1)
C
C     ---- DISCERN BOUNDARY LAYERING PARAMETERS FROM INPUT MESH
      NYRB = 1
      DO 150 I = 1,2
        IF(I .EQ. 2)NYRB = NODEY1
        NOLA(I) = 1
        NZZB(I) = NTR
        DO 160 J = NTR,NODEZ1-1
C
C         ---- PATCH TO PREVENT ARRAY OVERRUN
          IF (NOLA(I) .GE. NL)GOTO 160
          IF(NPT(J,1,NYRB) .EQ. 0)THEN
            NZZB(I) = J + 1
          ELSE
            RLR(NOLA(I),I) = 1./SIG(J,2*I,NYRB)
            IF(NPT(J,2*I,NYRB) .NE. NPT(J+1,2*I,NYRB))THEN
              DLR(NOLA(I),I) = ZZ(J+1) - ZZ(NZZB(I))
              NOLA(I) = NOLA(I) + 1
            END IF
          END IF
160     CONTINUE
        RLR(NOLA(I),I) = 1./SIG(NODEZ1,2*I,NYRB)
150   CONTINUE
C calling new version of PARFLD, etc
c
      NDX = IDX
      NMODE = 1
      IF(NDX .EQ. 1)NMODE = 2
      DO 180 LFRQ = 1,NFRE
        write(*,*) 'freq # ', lfrq
C
        DO 200 IMD = 1,NMODE
C
C         ---- FORM AND SOLVE GLOBAL SYSTEM OF EQUATIONS
          CALL PARFLD
C
C         ---- CALCULATE AUXILIARY FIELDS, REAL AND IMAGINARY, 
C              AND APPARENT RESISTIVITY AND PHASE AND TIPPER
C       
          CALL APPARMT
C
C  CALCULATE DATA-PARAMETER JACOBIAN IF INVERSION IS SELECTED.
C
          IF(NEXEC .EQ. 2)THEN
             CALL RECIJAC(NRES)
          END IF
C
200     CONTINUE
180   CONTINUE
      IDX=NDX
C
c remove the air layer if necessary
      call addair(0, mair)

      RETURN
      END
C
c-----------------------------------------------------------------------
      subroutine addair(lidx, mair)
c-----------------------------------------------------------------------
c
c OCCAM2DMT 1.0: Steven Constable IGPP/SIO La Jolla CA 92093-0225
c Subroutine Revision 1.01, 13 Jan 1993
c 
c addair adds or removes an air layer of 10 nodes 
c from the FE mesh (nodez, dlz(), npt() in common blocks)
c if a TE mode calculation is being done.  Much of this code has been
c abstracted from Phil Wannamaker's PW2DI
c
c on input:
c   lidx = flag for direction and action (local version of one in PW2DI): 
c           = 1 or 2 causes addition of air layer
c           = 3 sets mair = 0 and does nothing else
c           = otherwise removes air layer  
c   mair = unchanged from any previous call to addair
c
c on ouput:
c   mair = number of air layers, 0 for TM, 10 for TE
c   /blki/nodez, /blkd/dlz() and /blke/npt() are modified for TE calculation
c
c calls:
c   no other routines
c
c-----------------------------------------------------------------------
c includes:
      include 'imp.inc'
      include 'mt2Ddim.inc'
      include 'blkd.inc'
      include 'blke.inc'
      include 'blki.inc'
c  uses dlz(), yy()
c  uses npt()
c  uses nodez, nodey
c
c arguments 
      integer lidx, mair
c local variables
      REAL DZF,XA,XB,FXA,FXM,XM
      integer i,j,l,m

c-----------------------------------------------------------------------
      if (lidx .eq. 3) then
        MAIR = 0
        return
      else if (lidx.eq.1  .or. lidx.eq.2) then
C
        MAIR = 10
        NODEZ = NODEZ + MAIR
c
c make room in dlz() and npt() for air layers (push things down mair elements)
        do 10 j = nodez-mair-1, 1, -1
          dlz(j+mair) = dlz(j)
          do 10 l = 1, nodey-1
            do 10 m = 1,4
10            npt(j+mair,m,l) = npt(j,m,l)
C
c compute the thickness of the elements in the air layer
C TOTAL AIR THICKNESS IS ONE-HALF MESH WIDTH, I.E., 
C YY(NODEY) ABOVE.  ELEMENT ROW JUST ABOVE SURFACE 
C EQUALS THAT JUST BELOW AND ROW THICKNESSES INCREASE
C EXPONENTIALLY UPWARD.  IN ALL, THERE ARE MAIR ELEMENT 
C ROWS OF AIR.
        DZF = YY(NODEY)/DLZ(MAIR + 1)
        XA = 1.
        XB = DZF**0.25
        DO 40 I = 1,50
          XM = (XA + XB)/2.
          IF((XM-XA)/XM .LT. 1.0E-04)GO TO 41
          FXA = 1.
          FXM = 1.
          DO 42 J = 1,MAIR-1
            FXA = 1. + XA*FXA
            FXM = 1. + XM*FXM
 42       CONTINUE
          FXA = FXA - DZF
          FXM = FXM - DZF
C
          IF(SIGN(1.,FXA)*SIGN(1.,FXM) .LE. 0)THEN
            XB = XM
          ELSE
            XA = XM
          END IF
 41       CONTINUE
 40     CONTINUE
C
        DLZ(MAIR) = DLZ(MAIR + 1)
        DO 43 I = 1,MAIR-1
          J = MAIR - I
          DLZ(J) = XM*DLZ(J+1)
43      CONTINUE
C
c fill the air layer lookup table with 0 (air)
        DO 50 j = 1,MAIR
          DO 50 l = 1,NODEY-1
            DO 50 m = 1,4
50            NPT(j,m,l) = 0
c
      else
c remove air layers in dlz() and npt() (push things up mair elements)
        if (mair .ne. 0) then
          do 60 j = 1, nodez-mair-1
            dlz(j) = dlz(j+mair)
            do 60 l = 1, nodey-1
              do 60 m = 1,4
60              npt(j,m,l) = npt(j+mair,m,l)
          nodez = nodez - mair
        end if
      end if
      return
      end
C
C
      SUBROUTINE PARFLD
C
C     THIS ROUTINE COMPUTES THE GLOBAL FINITE ELEMENT SYSTEM TO GIVE
C     THE FIELD PARALLEL TO STRIKE.  TASKS PERFORMED INCLUDE THE
C     PRIMARY FIELDS FOR THE SOURCE VECTOR AND AT EACH NODAL COLUMN,
C     LOADING THE MATRIX ELEMENTS INTO THE GLOBAL SYSTEM, INCORPORA-
C     TING ANY DIRICHLET BOUNDARY CONDITIONS, AND SOLVING THE SYSTEM.
C
      include 'imp.inc'
      include 'mt2D.inc'
      include 'blkd.inc'
      include 'blke.inc'
      include 'blkf.inc'
      include 'blki.inc'
      include 'blkl.inc'
      COMPLEX CP(4),CP12,CP23,CP34,CP14,CP1234,A11,A12,A13,A15,A22,
     &A24,A25,A33,A34,A35,A44,A45,A55,E11,G11,C11,H11,E22,H22,C22,
     &E33,G33,E44,A55I,AA55,AAD55,ZLFLD,FSE,FSH 
      COMPLEX CK(4),CK12,CK13,CK14,CK23,CK24,CK34
      COMPLEX FP1(IPZ),FP2(IPZ),FP3(IPZ),FP4(IPZ),DJ1,DJ2,DJ3,DJ4,
     &  DSS1,DSS2,DSS3,DSS4,FP5(IPZ),FP6(IPZ),FP7(IPZ),FP8(IPZ),SM1,
     &  SM2,SM3,SM4,SM5,RU1,RU2,RU3,RU4,RU5,RC1,RC2,RC3,RC4,S,RM,RJ
      COMMON /BLKC/ AA55(4,IPZ,IPY),AAD55(4)
      COMMON /FORSOL/ S(IP3,IP2),RM(IP2),RJ(IPY,IP2)
c miscellaneous stuff that needs to be defined
      integer i,nodey1,nodez1,nband,nban1,nlyr,nlyr1,lzp,j,jm1,k,l1,l,
     &   m,l2,l3,l4,nji,iyn,it,ib
      real pi,rad,w,wu,we,ra,zap,rhal,phal,rhar,phar,zt,z1,z2,z3,z5,z6,
     &   z7,z8,zl,dlzy,dlyz,a,b,c,c4,dy4
C
      IF(NDX .EQ. 1)IDX = 2
      IF(IMD .EQ. 2)IDX = 3
      IF(IDX .EQ. 2)THEN
       NTQ = 1
       NDZ = NODEZ
       NDZ1 = NDZ - 1
       DO 10 I = 1,NODEY
        NTA(I) = NTAO(I) + NTR + NDZ*(I-1)
10     CONTINUE
      ELSE IF(IDX .EQ. 3)THEN
       NTQ = NTR
       NDZ = NODEZ - NTR + 1
       NDZ1 = NDZ - 1
       DO 15 I = 1,NODEY
        NTA(I) = NTAO(I) + 1 + NDZ*(I-1)
15     CONTINUE
      END IF
      NODEY1 = NODEY - 1
      NODEZ1 = NODEZ - 1
      NBAND = NDZ + 2
      NNODE = NODEY*NDZ
      NBAN1 = NBAND - 1
C
C     COMPUTE APPARENT RESISTIVITIES AND IMPEDANCE PHASES FOR LEFT
C     AND RIGHT SIDE LAYER SEQUENCES.
C
      PI = 3.1415927
      RAD = 180./PI
      W = 2.*PI*FREK(LFRQ)
      WU = W*PI*4.E-07
      WE = W*8.85433E-12
      RA = 1.0E+18
      NLYR = NOLA(1)
      NLYR1 = NLYR - 1
      IF (NLYR .EQ. 1) GO TO 20
       DO 30 I = 1,NLYR1
        DDP(I) = DLR(I,1)
        RL(I) = RLR(I,1)
30     CONTINUE
20    CONTINUE
      RL(NLYR) = RLR(NLYR,1)
      CALL COAMP(WU,WE,NLYR)
      IF(IMD .EQ. 1) THEN
       ZAP = 0.
       IF(NPT(NZZB(1),1,1) .EQ. ISEA)ZAP = DDP(1)
       FSE = ZLFLD(WU,ZAP,1,NLYR)
       FSH = ZLFLD(WU,ZAP,2,NLYR)
       RHAL = CABS(FSE/FSH)**2/WU
       PHAL = RAD*ATAN2(AIMAG(FSE/FSH),REAL(FSE/FSH))
      END IF
C
      NLYR = NOLA(2)
      NLYR1 = NLYR - 1
      IF(NLYR .EQ. 1)GO TO 40
       DO 50 I = 1,NLYR1
        DDP(I) = DLR(I,2)
        RL(I) = RLR(I,2)
50     CONTINUE
40    CONTINUE
      RL(NLYR) = RLR(NLYR,2)
      CALL COAMP(WU,WE,NLYR)
      IF (IMD .EQ. 1) THEN
       ZAP = 0.
       IF(NPT(NZZB(2),1,NODEY1) .EQ. ISEA)ZAP = DDP(1)
       FSE = ZLFLD(WU,ZAP,1,NLYR)
       FSH = ZLFLD(WU,ZAP,2,NLYR)
       RHAR = CABS(FSE/FSH)**2/WU
       PHAR = RAD*ATAN2(AIMAG(FSE/FSH),REAL(FSE/FSH))
      END IF
C
C     THIS SECTION CALCULATES THE PRIMARY E- OR H-FIELDS FOR THE TE
C     AND TM PROBLEMS.  FIELDS IN THE LAYERED HOST ARE CALCULATED FOR
C     AN INCIDENT E-FIELD AMPLITUDE OF (1.,0.) AT THE EARTH'S SURFACE
C     (NOT AT THE TOP OF THE MESH) POLARIZED IN THE +X DIRECTION FOR
C     THE TE MODE AND IN THE +Y DIRECTION FOR THE TM MODE.
C     ZLFLD(Z,1) IS FOR E AND ZLFLD(Z,2) IS FOR H.  
C
      LSID = 1
      IF(RLR(1,1) .LT. RLR(1,2))LSID = 2         !WHICH SIDE IS HOST
C
      NLYR = NOLA(LSID)
      NLYR1 = NLYR - 1
      IF(NLYR .EQ. 1)GO TO 60
       DO 70 I = 1,NLYR1
        DDP(I) = DLR(I,LSID)
        RL(I) = RLR(I,LSID)
70     CONTINUE
60    CONTINUE
      RL(NLYR) = RLR(NLYR,LSID)
      CALL COAMP(WU,WE,NLYR)
      DO 80 I = NTQ,NODEZ1
       ZT = ZZ(I) - ZZ(NZZB(LSID))
       Z1 = ZT + DLZ(I)/6.
       Z2 = ZT + DLZ(I)/2.
       Z3 = ZT + DLZ(I)*(5./6.)
       LZP = 0
       IF(Z2 .GE. 0.)LZP = 1
       IF(NLYR .EQ. 1)GO TO 100
        DO 110 J = 2,NLYR
         JM1 = J - 1
         IF(Z2 .GT. DDP(JM1))LZP = J
110     CONTINUE
100    CONTINUE
       IF(LZP .EQ. 0)GO TO 120
        SIGH(I) = CMPLX(1./RL(LZP),WE)
        GO TO 130
120    CONTINUE
        SIGH(I) = CMPLX(1./RA,WE)
130    CONTINUE
       IF(IDX .EQ. 3)GO TO 140
        FP1(I) = ZLFLD(WU,Z1,1,NLYR)
        FP2(I) = ZLFLD(WU,Z2,1,NLYR)
        FP3(I) = ZLFLD(WU,Z3,1,NLYR)
        FP4(I) = FP2(I)
        GO TO 90
140    CONTINUE
        FP1(I) = -ZLFLD(WU,Z1,2,NLYR)
        FP2(I) = -ZLFLD(WU,Z2,2,NLYR)
        FP3(I) = -ZLFLD(WU,Z3,2,NLYR)
        FP4(I) = FP2(I)
        Z5 = ZT
        Z6 = ZT + DLZ(I)/4.
        Z7 = ZT + DLZ(I)*(3./4.)
        Z8 = ZT + DLZ(I)
        FP5(I) = ZLFLD(WU,Z5,1,NLYR)
        FP6(I) = ZLFLD(WU,Z6,1,NLYR)
        FP7(I) = ZLFLD(WU,Z7,1,NLYR)
        FP8(I) = ZLFLD(WU,Z8,1,NLYR)
90     CONTINUE
80    CONTINUE
      DO 150 I = NTQ,NODEZ
       ZL = ZZ(I) - ZZ(NZZB(LSID))
       IF(IDX .EQ. 2)THEN
        FPA(I) = ZLFLD(WU,ZL,1,NLYR)
        FPB(I) = ZLFLD(WU,ZL,2,NLYR)
       ELSE
        FPA(I) = -ZLFLD(WU,ZL,2,NLYR)
        FPB(I) = ZLFLD(WU,ZL,1,NLYR)
       END IF
150   CONTINUE
C
C     THIS SECTION CALCULATES A 4 X 4 ELEMENT MATRIX FOR RECTANGULAR
C     ELEMENT MADE UP OF 4 TRIANGULAR ELEMENTS. THE 5 X 5 MATRIX IS
C     OBTAINED FROM PROPER COMBINATION OF THE FOUR 3 X 3 TRIANGULAR
C     ELEMENT MATRICIES AND REDUCED TO A 4 X 4 MATRIX THROUGH STATIC
C     CONDENSATION. THIS ELIMINATES THE DEGREE OF FREEDOM ASSOCIATED
C     WITH THE INTERNAL NODE OF THE RECTANGLE, THUS REDUCING THE
C     SIZE OF THE GLOBAL SYSTEM MATRIX.
C
      DO 160 I = 1,NNODE
       RM(I) = (0.,0.)
160   CONTINUE
      DO 170 K = 1,NBAND
       DO 180 I = 1,NNODE
        S(K,I) = (0.,0.)
180    CONTINUE
170   CONTINUE
C
      L1 = 0
      DO 190 L = 1,NODEY1
       L1 = L1 + 1
       DO 200 M = NTQ,NODEZ1
        DLZY = DLZ(M)/(2.*DLY(L))
        DLYZ = DLY(L)/(2.*DLZ(M))
        A = -(DLZY + DLYZ)/2.
        B = (DLZY - DLYZ)/2.
        C = DLY(L)*DLZ(M)/48.
        C4 = C*4.
        DY4 = DLY(L)/4.
C
C     CALCULATE THE APPROPRIATE COEFFICIENTS OF THE HELMHOLTZ
C     EQUATION FOR INSERTION INTO THE ELEMENT MATRICIES
C     ACCORDING TO WHETHER THE TE OR TM MODE IS BEING SOLVED.
C
        IF(IDX .EQ. 2)THEN
         DO 210 I = 1,4
          CK(I) = 1./CMPLX(0.,WU)
          CP(I) = -CMPLX(SIG(M,I,L),WE)
210      CONTINUE
        ELSE
         DO 220 I = 1,4
          CK(I) = 1./CMPLX(SIG(M,I,L),WE)
          CP(I) = -CMPLX(0.,WU)
220      CONTINUE
        END IF
        CK12 = CK(1) + CK(2)
        CK13 = CK(1) + CK(3)
        CK14 = CK(1) + CK(4)
        CK23 = CK(2) + CK(3)
        CK24 = CK(2) + CK(4)
        CK34 = CK(3) + CK(4)
        CP12 = CP(1) + CP(2)
        CP23 = CP(2) + CP(3)
        CP34 = CP(3) + CP(4)
        CP14 = CP(1) + CP(4)
        CP1234 = CP12 + CP34
C
C       THESE ARE THE ELEMENTS OF THE 5 X 5 MATRIX
C
        A11 = A*CK12 + 2.*C*CP12
        A12 = -B*CK(2) + C*CP(2)
        A13 = B*CK(1) + C*CP(1)
        A15 = DLYZ*CK(1) + DLZY*CK(2) + C*CP12
        A22 = A*CK23 + 2.*C*CP23
        A24 = B*CK(3) + C*CP(3)
        A25 = DLYZ*CK(3) + DLZY*CK(2) + C*CP23
        A33 = A*CK14 + 2.*C*CP14
        A34 = -B*CK(4) + C*CP(4)
        A35 = DLYZ*CK(1) + DLZY*CK(4) + C*CP14
        A44 = A*CK34 + 2.*C*CP34
        A45 = DLYZ*CK(3) + DLZY*CK(4) + C*CP34
        A55 = 2.*(C*CP1234 - DLYZ*CK13 - DLZY*CK24)
C
C       THESE ARE THE ELEMENTS OF THE 5 X 1 SOURCE VECTOR
C
        DJ1 = (CMPLX(SIG(M,1,L),WE) - SIGH(M))*FP1(M)
        DJ2 = (CMPLX(SIG(M,2,L),WE) - SIGH(M))*FP2(M)
        DJ3 = (CMPLX(SIG(M,3,L),WE) - SIGH(M))*FP3(M)
        DJ4 = (CMPLX(SIG(M,4,L),WE) - SIGH(M))*FP4(M)
        SM1 = (0.,0.)
        SM2 = (0.,0.)
        SM3 = (0.,0.)
        SM4 = (0.,0.)
        SM5 = (0.,0.)
        IF(IDX .EQ. 2)GO TO 230
         DSS1 = 1. - SIGH(M)/CMPLX(SIG(M,1,L),WE)
         DSS2 = 1. - SIGH(M)/CMPLX(SIG(M,2,L),WE)
         DSS3 = 1. - SIGH(M)/CMPLX(SIG(M,3,L),WE)
         DSS4 = 1. - SIGH(M)/CMPLX(SIG(M,4,L),WE)
         DJ1 = CMPLX(0.,WU)*DSS1*FP1(M)
         DJ2 = CMPLX(0.,WU)*DSS2*FP2(M)
         DJ3 = CMPLX(0.,WU)*DSS3*FP3(M)
         DJ4 = CMPLX(0.,WU)*DSS4*FP4(M)
         SM1 = DSS1*(2.*FP5(M) - FP6(M)) + DSS2*FP6(M)
         SM2 = -DSS2*FP7(M) - DSS3*(2.*FP8(M) - FP7(M))
         SM3 = DSS1*(2.*FP5(M) - FP6(M)) + DSS4*FP6(M)
         SM4 = -DSS3*(2.*FP8(M) - FP7(M)) - DSS4*FP7(M)
         SM5 = -FP6(M)*(2.*DSS1 - DSS2 - DSS4)
     &    - FP7(M)*(DSS2 - 2.*DSS3 + DSS4)
230     CONTINUE
        RU1 = C4*(DJ1 + DJ2) + DY4*SM1
        RU2 = C4*(DJ2 + DJ3) + DY4*SM2
        RU3 = C4*(DJ1 + DJ4) + DY4*SM3
        RU4 = C4*(DJ3 + DJ4) + DY4*SM4
        RU5 = C4*(DJ1 + DJ2 + DJ3 + DJ4) + DY4*SM5
C
C       ELEMENTS OF THE 4 X 4 CONDENSED MATRIX AND SOURCE
C
        A55I = 1./A55
        AA55(1,M,L) = A15*A55I
        AA55(2,M,L) = A25*A55I
        AA55(3,M,L) = A35*A55I
        AA55(4,M,L) = A45*A55I
        E11 = A11 - A15*AA55(1,M,L)
        G11 = A12 - A15*AA55(2,M,L)
        C11 = A13 - A15*AA55(3,M,L)
        H11 = -A15*AA55(4,M,L)
        E22 = A22 - A25*AA55(2,M,L)
        H22 = -A25*AA55(3,M,L)
        C22 = A24 - A25*AA55(4,M,L)
        E33 = A33 - A35*AA55(3,M,L)
        G33 = A34 - A35*AA55(4,M,L)
        E44 = A44 - A45*AA55(4,M,L)
        RC1 = RU1 - A15*(RU5/A55)
        RC2 = RU2 - A25*(RU5/A55)
        RC3 = RU3 - A35*(RU5/A55)
        RC4 = RU4 - A45*(RU5/A55)
C
C       THIS SECTION LOADS ELEMENT MATRICIES INTO GLOBAL MATRIX
C
        S(1,L1) = S(1,L1) + E11
        S(2,L1) = S(2,L1) + G11
        S(NBAN1,L1) = S(NBAN1,L1) + C11
        S(NBAND,L1) = S(NBAND,L1) + H11
        L2 = L1 + 1
        S(1,L2) = S(1,L2) + E22
        S(NDZ,L2) = S(NDZ,L2) + H22
        S(NBAN1,L2) = S(NBAN1,L2) + C22
        L3 = L1 + NDZ
        S(1,L3) = S(1,L3) + E33
        S(2,L3) = S(2,L3) + G33
        L4 = L3 + 1
        S(1,L4) = S(1,L4) + E44
        RM(L1) = RM(L1) + RC1
        RM(L2) = RM(L2) + RC2
        RM(L3) = RM(L3) + RC3
        RM(L4) = RM(L4) + RC4
        L1 = L1 + 1
200    CONTINUE
190   CONTINUE
C
C     INCORPORATE BOUNDARY CONDITIONS INTO GLOBAL EQUATION SYSTEM.
C     METHOD 1 OF HUEBNER (1983, P. 52) IS USED, REDUCING UNDERFLOWS.
C     ZERO CONDITIONS ARE USED AT THE MESH TOP FOR THE TM MODE AND AT
C     THE MESH BOTTOM FOR THE TE MODE.  NEUMANN CONDITIONS ELSEWHERE.
C
      NJI = NNODE - NDZ + 1
      IYN = 1
      DO 240 IT = 1,NJI,NDZ
       IF(IDX .EQ. 3)THEN
        S(1,IT) = (1.,0.)
        IF(IYN .EQ. 1)GO TO 250
         S(2,IT-1) = (0.,0.)
         S(NDZ,IT-NDZ+1) = (0.,0.)
         S(NBAN1,IT-NDZ) = (0.,0.)
         IF(IYN .EQ. 2)GO TO 250
         S(NBAND,IT-NBAN1) = (0.,0.)
250     CONTINUE
        S(2,IT) = (0.,0.)
        S(NDZ,IT) = (0.,0.)
        S(NBAN1,IT) = (0.,0.)
        S(NBAND,IT) = (0.,0.)
        RM(IT) = (0.,0.)
       END IF
       IF(IDX .EQ. 2)THEN
        IB = IT + NDZ1
        S(1,IB) = (1.,0.)
        IF(IYN .EQ. NODEY)GO TO 260
         S(2,IB) = (0.,0.)
         S(NDZ,IB) = (0.,0.)
         S(NBAN1,IB) = (0.,0.)
         IF(IYN .EQ. NODEY1)GO TO 260
         S(NBAND,IB) = (0.,0.)
260     CONTINUE
        S(2,IB-1) = (0.,0.)
        S(NDZ,IB-NDZ+1) = (0.,0.)
        IF(IYN .EQ. 1)GO TO 270
         S(NBAN1,IB-NDZ) = (0.,0.)
         S(NBAND,IB-NBAN1) = (0.,0.)
270     CONTINUE
        RM(IB) = (0.,0.)
       END IF
       IYN = IYN + 1
240   CONTINUE
C
C     SOLVE THE GLOBAL SYSTEM USING GAUSSIAN ELIMINATION
C
      CALL GAUSS(NNODE,NBAND)
C
      RETURN
      END
C
      SUBROUTINE COAMP(WM,WP,NLYR)
C
C     SUBROUTINE TO COMPUTE AMPLITUDE AND REFLECTION COEFFICIENTS FOR
C     ELECTRIC FIELDS PROPAGATING IN LAYERED EARTH.  COEFFICIENTS ARE
C     PASSED TO FUNCTION ZLFLD IN COMMON BLOCK BLKP.
C
      include 'imp.inc'
      include 'mt2Ddim.inc'
      include 'blkl.inc'
      COMPLEX KL10,KL1,ARG,AEX,AEXI,HTN(NL1),A2EX(NL1),ZLA,ZL1,RFC,
     &COA,CNUM,DEN,TL1
      COMMON /BLKP/ ARG(NL1),AEX(NL1),KL10,KL1(NL),RFC(NL),COA(NL),
     &HL(NL1),ZL1,ZLA
      integer nlyr,nlyr1,i,j,k,kk1
      real wm,wp,hl,ra
c
      RA = 1.0E+30
      KL10 = CSQRT(-CMPLX(0.,WM)*CMPLX(1./RA,WP))
      DO 10 I = 1,NLYR
       KL1(I) = CSQRT(-CMPLX(0.,WM)*CMPLX(1./RL(I),WP))
10    CONTINUE
      IF(NLYR .GT. 1)THEN
       NLYR1 = NLYR - 1
       HL(1) = DDP(1)
       IF(NLYR1 .EQ. 1)GO TO 30
        DO 40 I = 2,NLYR1
         HL(I) = DDP(I) - DDP(I-1)
40      CONTINUE
30     CONTINUE
       DO 50 J = 1,NLYR1
        ARG(J) = (0.,1.)*KL1(J)*HL(J)
        IF(REAL(ARG(J)) .GT. 100.)GO TO 60
         AEX(J) = CEXP(-ARG(J))
         GO TO 70
60      CONTINUE
         AEX(J) = (0.,0.)
70      CONTINUE
        IF(REAL(ARG(J)) .GT. 40.)GO TO 80
         AEXI = 1./AEX(J)
         HTN(J) = (AEXI - AEX(J))/(AEXI + AEX(J))
         GO TO 81
80      CONTINUE
         HTN(J) = (1.,0.)
81      CONTINUE
        A2EX(J) = AEX(J)*AEX(J)
50     CONTINUE
       ZLA = WM/KL1(NLYR)
       DO 90 I = 1,NLYR1
        J = NLYR - I + 1
        ZL1 = WM/KL1(J-1)
        RFC(J) = (ZLA - ZL1)/(ZLA + ZL1)
        ZLA = ZL1*(ZLA + ZL1*HTN(J-1))/(ZL1 + ZLA*HTN(J-1))
90     CONTINUE
       ZL1 = WM/KL10
       COA(1) = (2.*ZLA/(ZLA + ZL1))/(1. + RFC(2)*A2EX(1))
       TL1 = COA(1)*(1. + RFC(2)*A2EX(1))
       RFC(1) = TL1 - 1.
       IF(NLYR1 .EQ. 1)GO TO 100
        DO 110 K = 2,NLYR1
         KK1 = K - 1
         DEN = 1. + RFC(K+1)*A2EX(K)
         CNUM = (1. + RFC(K))*AEX(KK1)
         COA(K) = COA(KK1)*CNUM/DEN
110     CONTINUE
100    CONTINUE
       COA(NLYR) = COA(NLYR1)*AEX(NLYR1)*(1. + RFC(NLYR))
       GO TO 120
      ELSE
       COA(1) = 2.*KL10/(KL1(1) + KL10)
       TL1 = COA(1)
       RFC(1) = (KL10 - KL1(1))/(KL10 + KL1(1))
      END IF
120   CONTINUE
      RETURN
      END
C
      COMPLEX FUNCTION ZLFLD(WM,Z,IFL,NLYR)
C
C     FUNCTION COMPUTES ELECTRIC (IFL .EQ. 1)  OR MAGNETIC (IFL .EQ.
C     2) FIELD AT A DEPTH Z DUE TO UNIT-AMPLITUDE, DOWNGOING ELECTRIC
C     VECTOR AT SURFACE.  REFLECTION AND TRANSMISSION COEFFICIENTS
C     FOR THE LAYERING ARE PASSED FROM COAMP IN COMMON BLOCK BLKP.
C
      include 'imp.inc'
      include 'mt2Ddim.inc'
      include 'blkl.inc'
      COMPLEX KL10,KL1,K1WC,RFC,COA,ARG,AEX,AR,AE,AEI,AR1,AR2,ZL1,ZLA
      COMMON /BLKP/ ARG(NL1),AEX(NL1),KL10,KL1(NL),RFC(NL),COA(NL),
     &HL(NL1),ZL1,ZLA
      integer ifl,nlyr,lzp,i,im1
      real wm,z,hl,lzp1,zd1,zd2
      IF(Z .LT. 0.)THEN
       AR = (0.,1.)*KL10*Z
       IF(IFL .EQ. 1)ZLFLD = CEXP(-AR)*(1.+RFC(1)*CEXP(2.*AR))
C      IF(IFL .EQ. 1)ZLFLD = CMPLX(CDEXP(DCMPLX(-AR))*
C    &  (1.+RFC(1)*CDEXP(DCMPLX(2.*AR))))
       IF(IFL .EQ. 2)ZLFLD = (KL10/WM)*CEXP(-AR)*
     &  (1.-RFC(1)*CEXP(2.*AR))
C      IF(IFL .EQ. 2)ZLFLD = CMPLX((KL10/WM)*CDEXP(DCMPLX(-AR))*
C    &  (1.-RFC(1)*CDEXP(DCMPLX(2.*AR))))
       GO TO 10
      ELSE
       IF(NLYR .GT. 1)THEN
        LZP = 1
        DO 30 I = 2,NLYR
         IM1 = I - 1
         IF(Z .GT. DDP(IM1))LZP = I
30      CONTINUE
        K1WC = (KL1(LZP)/WM)*COA(LZP)
        IF(LZP .EQ. NLYR)GO TO 40
         IF(REAL(ARG(LZP)) .GT. 40.)GO TO 50
          AR = (0.,1.)*KL1(LZP)*(Z-DDP(LZP))
          AE = CEXP(-AR)
          IF(IFL .EQ. 1)ZLFLD = COA(LZP)*(AE + RFC(LZP+1)/AE)*AEX(LZP)
          IF(IFL .EQ. 2)ZLFLD = K1WC*(AE - RFC(LZP+1)/AE)*AEX(LZP)
         GO TO 10
50       CONTINUE
         IF(LZP .EQ. 1)GO TO 60
          LZP1 = LZP - 1
          ZD1 = Z - DDP(LZP1)
          ZD2 = Z - 2.*DDP(LZP) + DDP(LZP1)
         GO TO 70
60       CONTINUE
          ZD1 = Z
          ZD2 = Z - 2.*DDP(LZP)
70       CONTINUE
         AR1 = (0.,1.)*KL1(LZP)*ZD1
         AE = (0.,0.)
         AEI = (0.,0.)
         IF(REAL(AR1) .GT. 100.)GO TO 80
          AE = CEXP(-AR1)
          AR2 = (0.,1.)*KL1(LZP)*ZD2
         IF(REAL(AR2) .LT. -100.)GO TO 80
          AEI = CEXP(AR2)
80       CONTINUE
          IF(IFL .EQ. 1)ZLFLD = COA(LZP)*(AE + RFC(LZP+1)*AEI)
          IF(IFL .EQ. 2)ZLFLD = K1WC*(AE - RFC(LZP+1)*AEI)
         GO TO 10
40      CONTINUE
        AR = (0.,1.)*KL1(LZP)*(Z-DDP(LZP-1))
        IF(REAL(AR) .GT. 100.)GO TO 90
         IF(IFL .EQ. 1)ZLFLD = COA(LZP)*CEXP(-AR)
         IF(IFL .EQ. 2)ZLFLD = K1WC*CEXP(-AR)
        GO TO 10
90      CONTINUE
         ZLFLD = (0.,0.)
        GO TO 10
       ELSE
        AR = (0.,1.)*KL1(1)*Z
        IF(REAL(AR) .GT. 100.)GO TO 100
         IF(IFL .EQ. 1)ZLFLD = COA(1)*CEXP(-AR)
         IF(IFL .EQ. 2)ZLFLD = (KL1(1)/WM)*COA(1)*CEXP(-AR)
        GO TO 10
100     CONTINUE
         ZLFLD = (0.,0.)
        GO TO 10
       END IF
      END IF
10    CONTINUE
      RETURN
      END
C
      SUBROUTINE GAUSS(NNODE,NBAND)
C
C     ROUTINE TO REDUCE FINITE ELEMENT MATRIX.  GAUSSIAN ELIMINATION
C     WITHOUT PIVOTING BUT SPECIALIZED TO BANDED SYMMETRIC MATRICES
C     IS USED.  WRITTEN BY L. RIJO, 1977.  SUBSCRIPTING REVERSED IN
C     MATRIX S TO IMPROVE RUN-TIME, BY P. LUGAO, 1994.  ARITHMETIC IF
C     STATEMENTS REPLACED BY P. WANNAMAKER, 1994.
C
      include 'imp.inc'
      include 'mt2Ddim.inc'
      COMPLEX S,RM,RJ,D
      COMPLEX*16 C
      COMMON /FORSOL/ S(IP3,IP2),RM(IP2),RJ(IPY,IP2)
      INTEGER NNODE,NBAND,N,NBLIM,L,I,J,K,M
C
      DO 10 N = 1,NNODE
       NBLIM = NBAND
       IF(N .GT. NNODE-NBAND+1)NBLIM = NNODE - N + 1
       IF(NBLIM .LT. 2)GO TO 10
       DO 20  L = 2,NBLIM
        I = N + L - 1
         C = S(L,N)/S(1,N)
         J = 0
         DO 40 K = L,NBAND
          J = J + 1
          S(J,I) = S(J,I) - C*S(K,N)
40       CONTINUE
         S(L,N) = C
20     CONTINUE
10    CONTINUE
C
C     FORWARD REDUCTION OF SOURCES
C
      DO 50 N = 1,NNODE
       NBLIM = NBAND
       IF(N .GT. NNODE-NBAND+1)NBLIM = NNODE - N + 1
       IF(NBLIM .LT. 2)GO TO 70
       DO 60 L = 2,NBLIM
        I = N + L - 1
        C = S(L,N)
        RM(I) = RM(I) - RM(N)*C
60     CONTINUE
70     CONTINUE
       D = 1./S(1,N)
       RM(N) = RM(N)*D
50    CONTINUE
C
C     BACK SUBSTITUTION OF SOURCES
C
      DO 80 M = 2,NNODE
       N = NNODE + 1 - M
       NBLIM = NBAND
       IF(N .GT. NNODE-NBAND+1)NBLIM = NNODE - N + 1
       IF(NBLIM .LT. 2)GO TO 80
       DO 90 L = 2,NBLIM
        K = N + L - 1
        C = S(L,N)
        RM(N) = RM(N) - RM(K)*C
90     CONTINUE
80    CONTINUE
      RETURN
      END
C
      SUBROUTINE APPARMT
C 
C     CALCULATE AUXILIARY FIELDS, REAL AND IMAGINARY, AND APPARENT
C     RESISTIVITY, IMPEDANCE PHASE AND TIPPER.  AUXILIARY FIELDS
C     RETURNED FROM CALL TO AUXFLD.
C
      include 'imp.inc'
      include 'mt2D.inc'
      include 'blkd.inc'
      include 'blkf.inc'
      include 'blki.inc'
      include 'blkj.inc'
      include 'blkjc.inc'
      COMPLEX S,RM,FT,PF,HF,VF,TP,ZIMP,RJ
      COMMON /FORSOL/ S(IP3,IP2),RM(IP2),RJ(IPY,IP2)
c miscellaneous stuff
      integer nzzn,in,i,nto
      real pi,rad,w,wu
C
      PI = 3.1415927
      RAD = 180./PI
      W = 2.*PI*FREK(LFRQ)
      WU = W*PI*4.E-07
      NZZN = NTAO(1) + NTR
      DO 10 IN = 1,NRC
       AFN(IN) = (0.,0.)
       AFT(IN) = (0.,0.)
       I = NRL(IN)
       CALL AUXFLD(I,IN)
       NTO = NTAO(I) + NTR
       FT = RM(NTA(I)) + FPA(NTO)
       PF = FT/FPA(NZZN)
       HF = AFT(IN)/FPB(NZZN)
       VF = AFN(IN)/FPB(NZZN)
       TP = VF/HF
       IF(IDX .EQ. 2)THEN
        ZIMP = FT/AFT(IN)
        zimpe(in,lfrq)=zimp
        RHTE(IN,LFRQ) = CABS(ZIMP)**2/WU
        PHTE(IN,LFRQ) = RAD*ATAN2(AIMAG(ZIMP),REAL(ZIMP))
        KZTE(IN,LFRQ) = TP
       ELSE
        ZIMP = AFT(IN)/FT
        zimpm(in,lfrq)=zimp
        RHTM(IN,LFRQ) = CABS(ZIMP)**2/WU
        PHTM(IN,LFRQ) = RAD*ATAN2(-AIMAG(ZIMP),-REAL(ZIMP))
        TZTM(IN,LFRQ) = TP
       END IF
10    CONTINUE
      RETURN
      END
C
      SUBROUTINE AUXFLD(I,IN)
C
C     CALCULATE THE TOTAL FIELD NORMAL TO THE SLOPE AND THE TOTAL
C     FIELD PARALLEL TO THE SLOPE AND TRANSVERSE TO STRIKE.  THIS IS
C     DONE THROUGH THREE-POINT DIFFERENCE APPROXIMATIONS TO MAXWELL'S
C     EQUATIONS IN SUBROUTINE DERIV.  AUXILIARY FIELDS ARE RETURNED
C     TO MAIN IN ARRAYS AFN (NORMAL FIELD) AND AFT (TRANSVERSE 
C     FIELD).  FOR THE TE MODE , THE TRANSVERSE H-FIELD IS COMPUTED
C     IN A HORIZONTAL DIRECTION.  SIMILAR PROCEDURES ARE CARRIED OUT
C     FOR THE AUXILIARY JACOBIAN QUANTITIES IN AFND AND AFTD.
C
      include 'imp.inc'
      include 'mt2D.inc'
      include 'blkd.inc'
      include 'blke.inc'
      include 'blkf.inc'
      include 'blki.inc'
      COMPLEX S,RM,DERIV,DV,DH,UC,RJ 
      COMMON /FORSOL/ S(IP3,IP2),RM(IP2),RJ(IPY,IP2)
      DIMENSION IR(8)
      INTEGER I,IN,IR,NT,NCL,NCL1,NRW,NRW1,IH,IV,NPL,NPR,NPL1,NPR1,IH1,
     &   IH2,IH3,IV1,IV2,IV3,IY1,IY2,IZ1,IZ2,ISB,NCSH,NHS,MSH,NTZ
      REAL PI,W,WU,WE,AN,SIGU
C
      PI = 3.1415927
      W = 2.*PI*FREK(LFRQ)
      WU = W*PI*4.E-07
      WE = W*8.85433E-12
C
C     DEFINE NODAL ROW AND COLUMN LOCATIONS OF FIELDPOINT
C
      NT = NTA(I)
      NCL = I
      NCL1 = NCL - 1
      NRW = NTAO(I) + NTR
      NRW1 = NRW - 1
C
C     COMPUTE SHIFT IN INDICES OF NODAL FIELDS ALONG STRIKE USED FOR
C     DETERMINING AUXILIARY FIELDS.  NODES USED ARE KEPT WITHIN THE
C     EARTH.  FOR BATHYMETRY, NODES ARE SHIFTED BY TWO INTO SEAWATER.
C
      IH = 0
      IV = -1
      AN = 0.
      NPL = 0
      NPR = 0
      IF(I .NE. 1)NPL = NPT(NRW,1,NCL1)
      IF(I .NE. NODEY)NPR = NPT(NRW,1,NCL)
      NPARF(IN) = NPT(NRW,2,NCL)      !RESIS. LABEL UNDER RECEIVER
      IF(NPR .EQ. 0 .OR. NPR .EQ. ISEA)NPARF(IN) = NPT(NRW,4,NCL1)
      NPL1 = 0
      NPR1 = 0
      IF(NRW .NE. 1)THEN
       NPL1 = NPT(NRW1,3,NCL1)
       NPR1 = NPT(NRW1,3,NCL)
      END IF
      IF(I .EQ. 1 .OR. I .EQ. NODEY)GO TO 30
       CALL BNORM(NCL,NRW,NCL1,NRW1,AN,IR)       !SLOPE AT RECEIVER
       IF(NPL .EQ. 0)IH = 1                      !HORIZONTAL SHIFT TO
       IF(NPR .EQ. 0)IH = -1                     !AVOID AIR OR SEA
       IF(NPL .EQ. 0 .AND. NPR .EQ. 0)IH = 0
       IF(NRW .NE. 1)THEN                        !SEAFLOOR SHIFTS
        IF(NPL1 .EQ. ISEA)IH = -1
        IF(NPR1 .EQ. ISEA)IH = 1
        IF(NPL1 .EQ. ISEA .AND. NPR1 .EQ. ISEA)IH = 0
        IF(NPL1 .EQ. ISEA .OR. NPR1 .EQ. ISEA)IV = 1
       END IF
       GO TO 40
30    CONTINUE
      IF(I .EQ. 1 .OR. I .EQ. 2)THEN
       IH = 1
       IF(NRW .NE. 1 .AND. NPR1 .EQ. ISEA)IV = 1
      END IF
      IF(I .EQ. NODEY .OR. I .EQ. NODEY-1)THEN
       IH = -1
       IF(NRW .NE. 1 .AND. NPL1 .EQ. ISEA)IV = 1
      END IF
      AN = 0.
40    CONTINUE
C      IF(IV .EQ. 1)AN = 0.            !ASSUME SEAFLOOR LOCALLY FLAT
      IF(IDX .EQ. 2)AN = 0.                      !TE HORIZ. & VERT.
      IF(IV .EQ. 1)NPARF(IN) = -1           !NO E-PRIM. TERM IN SEA
C
C     INDICES FOR FIELDPOINT LOCATIONS AND ELEMENT DIMENSIONS
C
      IH1 = NT + NDZ*(IH - 1)
      IH2 = NT + NDZ*IH
      IH3 = NT + NDZ*(IH + 1)
      IV1 = NT - IV - 1
      IV2 = NT - IV
      IV3 = NT - IV + 1
      IY1 = NCL1 + IH
      IY2 = NCL + IH
      IZ1 = NRW1 - IV
      IZ2 = NRW - IV
      ISB = IZ1 + (1 + IV)/2
C
C     COMPUTE VERTICAL AND HORIZONTAL FIELD DERIVATIVES (DV AND DH).
C     AVOID JUMPS IN SIG OR SIGH ONE NODE AWAY WITH 1ST-ORDER DIFF'S.
C
      NCSH = 1
      IF(LSID .EQ. 2)NCSH = NODEY-1
      DV = -IV*(RM(IV2) - RM(IV1))/DLZ(ISB)       !DEFAULT 1ST-ORDER
      IF(IZ1 .LT. 1)GO TO 50
       IF(I .LT. NODEY) THEN
        IF(NPT(ISB,2,NCL) .NE. NPT(ISB-IV,2,NCL))GO TO 50
       END IF
       IF(I .GT. 1) THEN
        IF(NPT(ISB,4,NCL-1) .NE. NPT(ISB-IV,4,NCL-1))GO TO 50
       END IF
       IF(NPT(ISB,3,NCSH) .NE. NPT(ISB-IV,1,NCSH))GO TO 50 !AVOID SIGH
       DV = DERIV(RM(IV1),RM(IV2),RM(IV3),DLZ(IZ1),DLZ(IZ2),IV)
50    CONTINUE
      DH = (0.,0.)
      IF(I .EQ. 1 .OR. I .EQ. NODEY)GO TO 52
       IF(IH .NE. 0)THEN
        NHS = 1
        MSH = 1
        IF(IH .EQ. 1)NHS = 0
        IF(IH .EQ. 1)MSH = -1
        IF(NPT(ISB,2+IV,NCL-NHS) .NE. NPT(ISB,2+IV,NCL+1-3*NHS))THEN
         NTZ = NT + NDZ*(1-2*NHS)
         DH = MSH*(RM(NT) - RM(NTZ))/DLY(NCL-NHS) !DEFAULT 1ST-ORDER
         GO TO 52
        END IF
       END IF
       DH = DERIV(RM(IH1),RM(IH2),RM(IH3),DLY(IY1),DLY(IY2),IH)
52    CONTINUE
C
C     TE MODE AUXILIARY FIELDS
C
      IF(IDX .EQ. 2)THEN
       AN = 0.
       UC = -CMPLX(0.,WU)
       AFN(IN) = -(FPB(NRW) + DV/UC)*SIN(AN) - (DH/UC)*COS(AN)
       AFT(IN) = (FPB(NRW) + DV/UC)*COS(AN) - (DH/UC)*SIN(AN)
       CSIGU(IN) = UC                  !SAVED FOR PARJAC AND AUXADJ
C
C     TM MODE AUXILIARY FIELDS
C
      ELSE
       SIGU = SIG(ISB,2,NCL)
       IF(I .EQ. NODEY) SIGU = SIG(ISB,4,NCL-1)
       UC = CMPLX(SIGU,WE)
C       AN = 0.
       AFN(IN) = (-(SIGH(ISB)*FPB(NRW) +DV)*SIN(AN) - DH*COS(AN))/UC
       AFT(IN) = ((SIGH(ISB)*FPB(NRW) + DV)*COS(AN) - DH*SIN(AN))/UC
       CSIGU(IN) = UC                  !SAVED FOR PARJAC AND AUXADJ
      END IF
      RETURN
      END
C
      SUBROUTINE BNORM(NCL,NRW,NCL1,NRW1,AN,IR)
C
C     CALCULATE DOWNWARD BISECTOR FROM FIELD NODE AT BREAK IN TOPO-
C     GRAPHY.  AR, AL CLOCKWISE FROM UP, AN CLOCKWISE FROM DOWN.
C
      include 'imp.inc'
      include 'mt2Ddim.inc'
      include 'blkd.inc'
      include 'blke.inc'
      include 'blki.inc'
      COMPLEX TAL,TAR
      INTEGER NCL,NRW,NCL1,NRW1,IR,I
      REAL AN,SM,AL,AR
      DIMENSION IR(8)
C
C     IDENTIFY RESISTIVITY CODES OF TRIANGULAR ELEMENTS SURROUNDING
C     FIELD NODE.  UPPERMOST LEFT OCTANT IS IR(1), RIGHT IS IR(8).
C
      IR(3) = NPT(NRW,1,NCL1)
      IR(4) = NPT(NRW,4,NCL1)
      IR(5) = NPT(NRW,2,NCL)
      IR(6) = NPT(NRW,1,NCL)
      IF(NRW .EQ. 1)GO TO 10
       IR(1) = NPT(NRW1,4,NCL1)
       IR(2) = NPT(NRW1,3,NCL1)
       IR(7) = NPT(NRW1,3,NCL)
       IR(8) = NPT(NRW1,2,NCL)
      GO TO 20
10    CONTINUE
       IR(1) = 0
       IR(2) = 0
       IR(7) = 0
       IR(8) = 0
20    CONTINUE
      DO 40 I = 1,8
       IF (IR(I) .EQ. ISEA) IR(I) = 0
40    CONTINUE
C
C     DETERMINE SLOPE TO LEFT OF FIELD NODE
C
      SM = 1.0E-05
      TAL = 100.*CMPLX(1.,-SM)
      IF (NRW .EQ. 1) GO TO 50
      TAL = DLZ(NRW1)*CMPLX(1.,-SM)
      IF (IR(1).EQ.0 .AND. IR(2).NE.0)TAL = CMPLX(DLZ(NRW1),-DLY(NCL1))
50    CONTINUE
      IF (IR(2).EQ.0 .AND. IR(3).NE.0)TAL = DLY(NCL1)*CMPLX(SM,-1.)
      IF (IR(3).EQ.0 .AND. IR(4).NE.0)TAL = CMPLX(-DLZ(NRW),-DLY(NCL1))
      IF (IR(4).EQ.0 .AND. IR(5).NE.0)TAL = DLZ(NRW)*CMPLX(-1.,-SM)
C
C     DETERMINE SLOPE TO RIGHT OF FIELD NODE
C
      TAR = 100.*CMPLX(1.,SM)
      IF (NRW .EQ. 1) GO TO 60
      TAR = DLZ(NRW1)*CMPLX(1.,SM)
      IF (IR(8).EQ.0 .AND. IR(7).NE.0)TAR = CMPLX(DLZ(NRW1),DLY(NCL))
60    CONTINUE
      IF (IR(7).EQ.0 .AND. IR(6).NE.0)TAR = DLY(NCL)*CMPLX(SM,1.)
      IF (IR(6).EQ.0 .AND. IR(5).NE.0)TAR = CMPLX(-DLZ(NRW),DLY(NCL))
      IF (IR(5).EQ.0 .AND. IR(4).NE.0)TAR = DLZ(NRW)*CMPLX(-1.,SM)
      AL = ATAN2(AIMAG(TAL),REAL(TAL))
      AR = ATAN2(AIMAG(TAR),REAL(TAR))
      AN = (AL + AR)/2.
      RETURN
      END
C
      COMPLEX FUNCTION DERIV(R1,R2,R3,D1,D2,IF)
C
C     FUNCTION ESTIMATES DERIVATIVE AT LEFT END, MIDDLE OR RIGHT END
C     OF A COMPLEX THREE-POINT SEQUENCE BY FITTING THE PARABOLA
C     R = A + B*D + C*D**2.  PARABOLA COEFFICIENTS ARE CALCULATED
C     FROM AMPLITUDES AND SEPARATIONS OF THE THREE POINTS.
C
      include 'imp.inc'
      COMPLEX R1,R2,R3,B,C
      INTEGER IF
      REAL D1,D2,DEN
C
      DEN = D1*D2*D2 + D1*D1*D2
C     A = R2
      B = ((R3-R2)*D1*D1 - (R1-R2)*D2*D2)/DEN
      C = ((R3-R2)*D1 + (R1-R2)*D2)/DEN
      IF(IF)10,20,30
C     DERIVATIVE AT LEFT POINT
10    CONTINUE
       DERIV = B - 2.*C*D1
      GO TO 40
C     DERIVATIVE AT MIDDLE POINT
20    CONTINUE
       DERIV = B
      GO TO 40
C     DERIVATIVE AT RIGHT POINT
30    CONTINUE
       DERIV = B + 2.*C*D2
40    CONTINUE
      RETURN
      END
C
C
C
      SUBROUTINE RECIJAC(NRES)
C
C     SUBROUTINE TO CALCULATE DATA-PARAMETER JACOBIANS USING
C     RECIPROCITY.  PARALLEL FIELD QUANTITIES COMPUTED FOR TE MODE
C     ONLY BY LOADING UNIT SOURCES AT MT RECEIVER POINTS.  TM MODE
C     JACOBIAN ALONG STRIKE IS ZERO.  TRANSVERSE AND VERTICAL FIELD
C     QUANTITIES COMPUTED BY DEFINING A DISTRIBUTED SOURCE AT MT
C     FIELD POINT AND NEIGHBOURING POINTS FOR FIRST OR SECOND-ORDER
C     DIFFERENCING.
C
      include 'mt2D.inc'
      include 'blki.inc'
      COMPLEX S,RM,RJ,FLDJ,FLDJH,FLDJV
      COMMON /RECIP/ FLDJ(IPY,ipi36),FLDJH(IPY,ipi36),FLDJV(IPY,ipi36)
      COMMON /FORSOL/ S(IP3,IP2),RM(IP2),RJ(IPY,IP2)
C
      DO 10 KP = 1,NRES
       DO 11 I = 1,NRC
        FLDJ(I,KP) = (0.,0.)
        FLDJH(I,KP) = (0.,0.)
        FLDJV(I,KP) = (0.,0.)
11     CONTINUE 
10    CONTINUE 
C      
C     ALONG-STRIKE COMPONENT COMPUTED BY LOADING ARRAY OF RECIPROCAL
C     SOURCES AT MT RECEIVERS AND SOLVING IN GAUSSD.
C
      DO 15 JY = 1,NNODE
       DO 16 IY = 1,NODEY
        RJ(IY,JY) = (0.,0.)
16     CONTINUE  
15    CONTINUE  
      IX = 0
      DO 17 ILD = 1,NRC
       I = NTA(NRL(ILD))
       IF(IDX .EQ. 2 .OR. NPARF(ILD) .EQ. -1)RJ(ILD,I) = (1.,0.)
17    CONTINUE
      NBAND=NDZ+2
      CALL GAUSSD(NNODE,NBAND,NRC)
      CALL PARJAC(IX)
C
C     CALCULATE AUXILIARY FIELD JACOBIANS, USING DISTRIBUTED SOURCES.
C     FOR TE MODE, CALCULATE BOTH HORIZONTAL AND VERTICAL AUXILIARY
C     JACOBIANS.  FOR TM MODE, CALCULATE ONLY JACOBIAN OF AUXILIARY
C     ELECTRIC FIELD ALONG SLOPE.
C
      DO 20 IX = 1,2
       DO 21 JY = 1,NNODE
        DO 22 IY = 1,NODEY
         RJ(IY,JY) = (0.,0.)
22      CONTINUE  
21     CONTINUE  
       IF(IX .EQ. 2 .AND. IDX .EQ. 3)GO TO 20              !NO EZ
       DO 23 ILD = 1,NRC
        I = NRL(ILD)
        CALL AUXADJ(I,ILD,IX)     !DEFINE DISTRIBUTED SOURCE VECTORS
23     CONTINUE
       CALL GAUSSD(NNODE,NBAND,NRC)
       CALL PARJAC(IX)
20    CONTINUE
C
C     CONVERT E AND H-FIELD JACOBIANS TO THOSE OF LOG APP RHO, PHASE
C     AND TIPPER.  JACOBIANS ARE WITH RESPECT TO LOG RHO.
C
      CALL APPRMTJ
C
      RETURN
      END
C
      SUBROUTINE PARJAC(IX)
C
C     THIS ROUTINE COMPUTES THE DERIVATIVE OF THE FIELD PARALLEL TO
C     STRIKE WITH RESPECT TO THE CONDUCTIVITY OF A POLYGONAL REGION
C     IN THE MESH USING RECIPROCITY.  THIS IS ACCOMPLISHED FOLLOWING
C     SOLUTION OF UNIT OR DISTRIBUTED RECIPROCAL SOURCES BY WEIGHTING
C     WITH ELEMENT PROPERTIES AND TOTAL FIELDS.
C
      include 'mt2D.inc'
      include 'blkd.inc'
      include 'blke.inc'
      include 'blkf.inc'
      include 'blki.inc'
      include 'blkl.inc'
      COMPLEX AA55,AAD55,FLDJ,FLDJH,FLDJV
      COMPLEX CPD(4),CPD12,CPD23,CPD34,CPD14,CPD1T4,AD11,AD12,AD13,
     &AD15,AD22,AD24,AD25,AD33,AD34,AD35,AD44,AD45,AD55,ED11,GD11,
     &CD11,HD11,ED22,HD22,CD22,ED33,GD33,ED44,
     &FT1,FT2,FT3,FT4,RJC1,RJC2,RJC3,RJC4,RJ,S,RM
      COMPLEX DSIG,CKD(4),CKD12,CKD13,CKD14,CKD23,CKD24,CKD34
      COMMON /BLKC/ AA55(4,IPZ,IPY),AAD55(4)
      COMMON /RECIP/ FLDJ(IPY,ipi36),FLDJH(IPY,ipi36),FLDJV(IPY,ipi36)
      COMMON /FORSOL/ S(IP3,IP2),RM(IP2),RJ(IPY,IP2)
C
      PI = 3.1415927
      W = 2.*PI*FREK(LFRQ)
      WE = W*8.85433E-12
C
C     LOOP OVER ELEMENTS, CONSTRUCTING DERIVATIVE SOURCE TERMS
C
      NODEY1 = NODEY - 1
      NODEZ1 = NODEZ - 1
      L1 = 0
      DO 140 L = 1,NODEY1
       L1 = L1 + 1
       DO 150 M = NTQ,NODEZ1
        L2 = L1 + 1
        L3 = L1 + NDZ 
        L4 = L3 + 1
C
C       CALCULATE THE APPROPRIATE COEFFICIENTS FOR INSERTION INTO
C       THE SOURCE. IF TRIANGLES WITHIN RECTANGULAR ELEMENT ARE NOT
C       ALL THE SAME, SUM THEIR CONTRIBUTIONS SEPARATELY (NPE = 4). 
C
        NPT1 = NPT(M,1,L)
        NPE = 1
        DO 120 IC = 2,4
         IF(NPT(M,IC,L) .NE. NPT1)NPE = 4
120     CONTINUE
        IF(NPE .EQ. 1 .AND. NPT1 .EQ. 0)GO TO 131
        DLZY = DLZ(M)/(2.*DLY(L))
        DLYZ = DLY(L)/(2.*DLZ(M))
        A = -(DLZY + DLYZ)/2.
        B = (DLZY - DLYZ)/2.
        C = DLY(L)*DLZ(M)/48.
C
C       CALCULATE THE APPROPRIATE COEFFICIENTS FOR INSERTION INTO
C       SOURCE VECTOR ACCORDING TO WHETHER TE OR TM MODE SOLVED.
C
        DO 130 IE = 1,NPE
         NPTE = NPT(M,IE,L)
         IF(NPE .EQ. 1)THEN                 !HOMOGENEOUS RECTANGLE
          IF(IDX .EQ. 2)THEN                !TE MODE
           DO 132 I = 1,4
            CKD(I) = CMPLX(0.,0.)
            CPD(I) = CMPLX(1.,0.)
132        CONTINUE
          ELSE                              !TM MODE
           DO 133 I = 1,4
            DSIG = 1./CMPLX(SIG(M,I,L),WE)
            CKD(I)= DSIG*DSIG
            CPD(I) = CMPLX(0.,0.)
133        CONTINUE
          END IF
         ELSE                               !INHOMOGENEOUS RECTANGLE
          DO 135 I = 1,4
           CKD(I) = 0.
           CPD(I) = 0.
135       CONTINUE
C
C         CALCULATE THE APPROPRIATE COEFFICIENTS FOR INSERTION INTO
C         SOURCE VECTOR ACCORDING TO WHETHER TE OR TM MODE SOLVED.

          IF(IDX .EQ. 2)THEN
           CKD(IE) = CMPLX(0.,0.)
           CPD(IE) = CMPLX(1.,0.)
          ELSE
           DSIG = 1./CMPLX(SIG(M,IE,L),WE)
           CKD(IE)= DSIG*DSIG
           CPD(IE) = CMPLX(0.,0.)
          END IF
         END IF
         CKD12 = CKD(1) + CKD(2)
         CKD13 = CKD(1) + CKD(3)
         CKD14 = CKD(1) + CKD(4)
         CKD23 = CKD(2) + CKD(3)
         CKD24 = CKD(2) + CKD(4)
         CKD34 = CKD(3) + CKD(4)
         CPD12 = CPD(1) + CPD(2)
         CPD23 = CPD(2) + CPD(3)
         CPD34 = CPD(3) + CPD(4)
         CPD14 = CPD(1) + CPD(4)
         CPD1T4 = CPD12 + CPD34
C
C        ELEMENTS OF THE 5 X 5 DIFFERENTIATED MATRIX
C
         AD11 = A*CKD12 + 2.*C*CPD12
         AD12 = -B*CKD(2) + C*CPD(2)
         AD13 = B*CKD(1) + C*CPD(1)
         AD15 = DLYZ*CKD(1) + DLZY*CKD(2) + C*CPD12
         AD22 = A*CKD23 + 2.*C*CPD23
         AD24 = B*CKD(3) + C*CPD(3)
         AD25 = DLYZ*CKD(3) + DLZY*CKD(2) + C*CPD23
         AD33 = A*CKD14 + 2.*C*CPD14
         AD34 = -B*CKD(4) + C*CPD(4)
         AD35 = DLYZ*CKD(1) + DLZY*CKD(4) + C*CPD14
         AD44 = A*CKD34 + 2.*C*CPD34
         AD45 = DLYZ*CKD(3) + DLZY*CKD(4) + C*CPD34
         AD55 = 2.*(C*CPD1T4 - DLYZ*CKD13 - DLZY*CKD24)

C        ELEMENTS OF THE CONDENSED DIFFERENTIATED MATRIX

         DO 170 IA = 1,4
          AAD55(IA) = AD55*AA55(IA,M,L)
170      CONTINUE
         ED11 = AD11 - 2.*AD15*AA55(1,M,L) + AAD55(1)*AA55(1,M,L)
         GD11 = AD12 - AD15*AA55(2,M,L) - AD25*AA55(1,M,L)
     &               + AAD55(1)*AA55(2,M,L)
         CD11 = AD13 - AD15*AA55(3,M,L) - AD35*AA55(1,M,L)
     &               + AAD55(1)*AA55(3,M,L)
         HD11 =  - AD15*AA55(4,M,L) - AD45*AA55(1,M,L)
     &               + AAD55(1)*AA55(4,M,L)
         ED22 = AD22 - 2.*AD25*AA55(2,M,L) + AAD55(2)*AA55(2,M,L)
         HD22 =  - AD25*AA55(3,M,L) - AD35*AA55(2,M,L)
     &               + AAD55(2)*AA55(3,M,L)
         CD22 = AD24 - AD25*AA55(4,M,L) - AD45*AA55(2,M,L)
     &               + AAD55(2)*AA55(4,M,L)
         ED33 = AD33 - 2.*AD35*AA55(3,M,L) + AAD55(3)*AA55(3,M,L)
         GD33 = AD34 - AD35*AA55(4,M,L) - AD45*AA55(3,M,L)
     &               + AAD55(3)*AA55(4,M,L)
         ED44 = AD44 - 2.*AD45*AA55(4,M,L) + AAD55(4)*AA55(4,M,L)
C
C        MULTIPLYING THE CONDENSED DIFFERENTIATED MATRIX BY THE TOTAL
C        FIELDS ALONG STRIKE AT THE CORNERS OF THE RECTANGULAR
C        ELEMENT YIELDS THE CONDENSED SOURCE. EVALUATION OF SOURCE TERMS
C        UTILIZES FIELDS FROM FORWARD PROBLEM AND RECIPROCITY.
C 
         IF (NPTE .NE. 0) THEN
          FT1 = RM(L1) + FPA(M)  
          FT2 = RM(L2) + FPA(M+1)
          FT3 = RM(L3) + FPA(M)
          FT4 = RM(L4) + FPA(M+1)
          RJC1 = ED11*FT1 + GD11*FT2 + CD11*FT3 + HD11*FT4
          RJC2 = GD11*FT1 + ED22*FT2 + HD22*FT3 + CD22*FT4
          RJC3 = CD11*FT1 + HD22*FT2 + ED33*FT3 + GD33*FT4
          RJC4 = HD11*FT1 + CD22*FT2 + GD33*FT3 + ED44*FT4
C
          DO 107 KR = 1,NRC
C
           IF(IX .EQ. 0)THEN           !TE PARALLEL JACOBIAN
            FLDJ(KR,NPTE) = FLDJ(KR,NPTE) + RJC1*RJ(KR,L1)
     &       + RJC2*RJ(KR,L2) + RJC3*RJ(KR,L3) + RJC4*RJ(KR,L4)
           ELSE IF(IX .EQ. 1)THEN      !HORIZONTAL AUX. JACOBIAN
            FLDJH(KR,NPTE) = FLDJH(KR,NPTE) + RJC1*RJ(KR,L1)
     &       + RJC2*RJ(KR,L2) + RJC3*RJ(KR,L3) + RJC4*RJ(KR,L4)
           ELSE IF(IX .EQ. 2)THEN      !VERTICAL AUX. JACOBIAN
            FLDJV(KR,NPTE) = FLDJV(KR,NPTE) + RJC1*RJ(KR,L1)
     &       + RJC2*RJ(KR,L2) + RJC3*RJ(KR,L3) + RJC4*RJ(KR,L4)
           END IF
C
107       CONTINUE
         END IF
130     CONTINUE
131     CONTINUE
        L1 = L1 + 1
150    CONTINUE
140   CONTINUE
C
C     FOR TM MODE, ADD E-FIELD TO THE ALONG-SLOPE ELECTRIC JACOBIAN
C     WHEN RECEIVER POINT IS IN THE PARAMETER.
C
      IF(IDX .EQ. 3 .AND. IX .EQ. 1) THEN
       DO 177 KS = 1,NRC
        IF(NPARF(KS) .GT. 0)THEN
         FLDJH(KS,NPARF(KS)) = FLDJH(KS,NPARF(KS))
     &                         - AFT(KS)/CSIGU(KS)
        END IF
177    CONTINUE
      END IF
      RETURN
      END
C 
      SUBROUTINE GAUSSD(NNODE,NBAND,NSRC)
C
      include 'mt2Ddim.inc'
      COMPLEX S,RM,RJ,C,D
      COMMON /FORSOL/ S(IP3,IP2),RM(IP2),RJ(IPY,IP2)
      DO 50 N = 1,NNODE
       NBLIM = NBAND
       IF(N .GT. NNODE-NBAND+1)NBLIM = NNODE - N + 1
       IF(NBLIM .LT. 2)GO TO 70
        DO 60 L = 2,NBLIM
         C = S(L,N)
         I = N + L - 1
         DO 80 KP = 1,NSRC
          RJ(KP,I) = RJ(KP,I) - C*RJ(KP,N)
80       CONTINUE
60      CONTINUE
70     CONTINUE
       D = 1./S(1,N)
       DO 90 KP = 1,NSRC
        RJ(KP,N) = RJ(KP,N)*D
90     CONTINUE
50    CONTINUE
C
      DO 100 M = 2,NNODE
       N = NNODE + 1 - M
       NBLIM = NBAND
       IF(N .GT. NNODE-NBAND+1)NBLIM = NNODE - N + 1
       IF(NBLIM .LT. 2)GO TO 100
       DO 110 L = 2,NBLIM
        C = S(L,N)
        K = N + L - 1
        DO 120 KP = 1,NSRC
         RJ(KP,N) = RJ(KP,N) - C*RJ(KP,K)
120     CONTINUE
110    CONTINUE
100   CONTINUE
      RETURN
      END
C
      SUBROUTINE AUXADJ(I,ILD,IX)
C
C     LOADS THE AUXILIARY FIELD JACOBIAN SOURCE VECTORS ACCORDING TO
C     RECIPROCITY.  FOR EACH RECEIVER, SUBROUTINE DRVADJ COMPUTES
C     PARABOLA COEFFICIENTS A, B, AND C WHICH WILL BE LOADED INTO
C     SOURCE VECTOR RJ(,) TO BE SOLVED IN SUBROUTINE GAUSSD.  COEFF-
C     ICIENT VALUES DEPEND UPON TOPOGRAPHIC SLOPE AND ELEMENT PROPER-
C     TIES.  FOR TM MODE, PROJECTED VERTICAL AND HORIZONTAL DERIVA-
C     TIVES ARE ADDED TO GIVE E-FIELD ALONG SLOPE.
C
      include 'mt2D.inc'
      include 'blkd.inc'
      include 'blke.inc'
      include 'blkf.inc'
      include 'blki.inc'
      COMPLEX S,RM,UC,RJ
      COMMON /FORSOL/ S(IP3,IP2),RM(IP2),RJ(IPY,IP2)
      DIMENSION IR(8)
C
C     DEFINE NODAL ROW AND COLUMN LOCATIONS OF FIELDPOINT
C
      NT = NTA(I)
      NCL = I
      NCL1 = NCL - 1
      NRW = NTAO(I) + NTR
      NRW1 = NRW - 1
      UC = CSIGU(ILD)
C
C     COMPUTE SHIFT IN INDICES OF NODAL FIELDS ALONG STRIKE USED FOR
C     DETERMINING AUXILIARY FIELDS.  NODES USED ARE KEPT WITHIN THE
C     EARTH.  FOR BATHYMETRY, NODES ARE SHIFTED BY TWO INTO SEAWATER.
C
      IH = 0
      IV = -1
      AN = 0.
      NPL = 0
      NPR = 0
      IF(I .NE. 1)NPL = NPT(NRW,1,NCL1)
      IF(I .NE. NODEY)NPR = NPT(NRW,1,NCL)
      NPL1 = 0
      NPR1 = 0
      IF(NRW .NE. 1)THEN
       NPL1 = NPT(NRW1,3,NCL1)
       NPR1 = NPT(NRW1,3,NCL)
      END IF
      IF(I .EQ. 1 .OR. I .EQ. NODEY)GO TO 30
       CALL BNORM(NCL,NRW,NCL1,NRW1,AN,IR)       !SLOPE AT RECEIVER
       IF(NPL .EQ. 0)IH = 1                      !HORIZONTAL SHIFT TO
       IF(NPR .EQ. 0)IH = -1                     !AVOID AIR OR SEA
       IF(NPL .EQ. 0 .AND. NPR .EQ. 0)IH = 0
       IF(NRW .NE. 1)THEN                        !SEAFLOOR SHIFTS
        IF(NPL1 .EQ. ISEA)IH = -1
        IF(NPR1 .EQ. ISEA)IH = 1
        IF(NPL1 .EQ. ISEA .AND. NPR1 .EQ. ISEA)IH = 0
        IF(NPL1 .EQ. ISEA .OR. NPR1 .EQ. ISEA)IV = 1
       END IF
       GO TO 40
30    CONTINUE
      IF(I .EQ. 1 .OR. I .EQ. 2)THEN
       IH = 1
       IF(NRW .NE. 1 .AND. NPR1 .EQ. ISEA)IV = 1
      END IF
      IF(I .EQ. NODEY .OR. I .EQ. NODEY-1)THEN
       IH = -1
       IF(NRW .NE. 1 .AND. NPL1 .EQ. ISEA)IV = 1
      END IF
      AN = 0.
40    CONTINUE
C      IF(IV .EQ. 1)AN = 0.             !ASSUME SEAFLOOR LOCALLY FLAT
      IF(IDX .EQ. 2)AN = 0.                      !TE HORIZ. & VERT.
C
C     INDICES FOR FIELDPOINT LOCATIONS AND ELEMENT DIMENSIONS
C
      IH1 = NT + NDZ*(IH - 1)
      IH2 = NT + NDZ*IH
      IH3 = NT + NDZ*(IH + 1)
      IV1 = NT - IV - 1
      IV2 = NT - IV
      IV3 = NT - IV + 1
      IY1 = NCL1 + IH
      IY2 = NCL + IH
      IZ1 = NRW1 - IV
      IZ2 = NRW - IV
      ISB = IZ1 + (1 + IV)/2
C
      IF(IX .EQ. 1)THEN                !CROSS-STRIKE FIELD JACOBIAN
C
C      VERTICAL DERIVATIVE COEFFICIENTS A, B AND C FOR HORIZONTAL
C      AUXILIARY JACOBIAN.
C
       A = -1./DLZ(IZ1)                 !AVOID JUMPS IN SIG ONE NODE
       B = 1./DLZ(IZ1)                  !AWAY WITH 1ST-ORDER DIFF'S.
       C = 0.
       IF(IZ1 .LT. 1)GO TO 50
        IF(I .LT. NODEY) THEN
         IF(NPT(ISB,2,NCL) .NE. NPT(ISB-IV,2,NCL))GO TO 50
        END IF
        IF(I .GT. 1) THEN
         IF(NPT(ISB,4,NCL-1) .NE. NPT(ISB-IV,4,NCL-1))GO TO 50
        END IF
C
        CALL DRVADJ(DLZ(IZ1),DLZ(IZ2),IV,A,B,C)
C
50     CONTINUE
C
C      LOAD COEFFICIENTS A, B AND C INTO RHS VECTOR RJ, WEIGHTED BY
C      PROPERTY FACTOR FOR EITHER TE OR TM.  FOR TM MODE, COEFFICIENT
C      W.R.T. RECEIVER IS ALWAYS ZERO DUE TO NO SECONDARY H-FIELD AT
C      SURFACE.
C
       IF(IDX .EQ. 3 .AND. IV .EQ. -1)A = 0.
       RJ(ILD,IV1) = COS(AN)*CMPLX(A,0.)/UC
       RJ(ILD,IV2) = COS(AN)*CMPLX(B,0.)/UC
       RJ(ILD,IV3) = COS(AN)*CMPLX(C,0.)/UC
      END IF
C
C     HORIZONTAL DERIVATIVES FOR VERTICAL FIELD COMPONENTS.  THESE
C     ARE DONE FOR THE TE MODE GENERALLY AND THE TM FOR TOPOGRAPHY.
C
      NVGO = 0
      IF(IX .EQ. 1 .AND. IDX .EQ. 3)NVGO = 1
      IF(IX .EQ. 2 .AND. IDX .EQ. 2)NVGO = 1
      IF(NVGO .EQ. 1)THEN
       IF(IH .EQ. 1)THEN                    !FIRST-ORDER DEFAULTS
        A = -1./DLY(IY1)
        B = 1./DLY(IY1)
        C = 0.
       ELSE IF(IH .EQ. -1)THEN
        A = 0.
        B = -1./DLY(IY2)
        C = 1./DLY(IY2)
       END IF
       IF(I .EQ. 1 .OR. I .EQ. NODEY)GO TO 51
       IF(IH .NE. 0)THEN
        NHS = 1
        IF(IH .EQ. 1)NHS = 0
        IF(NPT(ISB,2+IV,NCL-NHS) .NE. 
     &     NPT(ISB,2+IV,NCL+1-3*NHS))GO TO 51
       END IF
C
       CALL DRVADJ(DLY(IY1),DLY(IY2),IH,A,B,C)
C
51     CONTINUE         
C
       ANM = AN
       IF(IDX .EQ. 2)ANM = AN + 1.570796         !ADD PI/2 FOR TE
       IF(IV .EQ. -1 .AND. IDX .EQ. 3)THEN
        IF(IH .EQ. 1)A = 0.
        IF(IH .EQ. 0)B = 0.
        IF(IH .EQ. -1)C = 0.
       END IF
C
       IF(IH .EQ. 0)THEN
        RJ(ILD,IH1) = -SIN(ANM)*CMPLX(A,0.)/UC
        RJ(ILD,IH2) = RJ(ILD,IH2) - SIN(ANM)*CMPLX(B,0.)/UC
        RJ(ILD,IH3) = -SIN(ANM)*CMPLX(C,0.)/UC
       ELSE IF(IH .EQ. 1)THEN
        RJ(ILD,IH1) = RJ(ILD,IH1) - SIN(ANM)*CMPLX(A,0.)/UC
        RJ(ILD,IH2) = -SIN(ANM)*CMPLX(B,0.)/UC
        RJ(ILD,IH3) = -SIN(ANM)*CMPLX(C,0.)/UC
       ELSE IF(IH .EQ. -1)THEN
        RJ(ILD,IH1) = -SIN(ANM)*CMPLX(A,0.)/UC
        RJ(ILD,IH2) = -SIN(ANM)*CMPLX(B,0.)/UC
        RJ(ILD,IH3) = RJ(ILD,IH3) - SIN(ANM)*CMPLX(C,0.)/UC
       END IF
      END IF
      RETURN
      END
C
      SUBROUTINE DRVADJ(D1,D2,IF,A,B,C)
C
C     CALCULATE PARABOLA FITTING COEFFICIENTS A,B AND C WHICH
C     WILL GO INTO RHS VECTOR OF RECIPROCITY SOLUTION FOR THE
C     AUXILIARY FIELD JACOBIANS.  PARABOLA COEFFICIENTS ARE
C     CALCULATED FROM SEPARATIONS OF THE THREE POINTS AT LEFT, 
C     MIDDLE OR RIGHT END.  FORM IS  R = A*R1 + B*R2 + C*R3.  
C
      REAL A,B,C,D1,D2,DEN
C
      DEN = D1*D2*D2 + D1*D1*D2
      IF(IF)10,20,30
C
C     DERIVATIVE AT LEFT POINT (IF=-1)
C
10    CONTINUE
       A = (-D2*D2 - 2.*D1*D2)/DEN
       B = (D2*D2 + D1*D1 + 2.*D1*D2)/DEN
       C = -D1*D1/DEN
      GO TO 40
C
C     DERIVATIVE AT MIDDLE POINT (IF=0)
C
20    CONTINUE
       A = -D2*D2/DEN
       B = (D2*D2 - D1*D1)/DEN
       C = D1*D1/DEN
      GO TO 40
C
C     DERIVATIVE AT RIGHT POINT (IF=1)
C
30    CONTINUE
       A = D2*D2/DEN
       B = -(D2*D2 + D1*D1 + 2.*D1*D2)/DEN
       C = (D1*D1 + 2.*D1*D2)/DEN
40    CONTINUE
      RETURN
      END
C
      SUBROUTINE APPRMTJ
C
C     CALCULATE JACOBIANS OF LOG APP. RES., IMPEDANCE PHASE AND TIPPER
C     FROM E AND H-FIELD JACOBIANS.  FINAL QUANTITIES ARE WITH RESPECT 
C     TO LOG RHO.
C
      include 'mt2D.inc'
      include 'blkd.inc'
      include 'blke.inc'
      include 'blkf.inc'
      include 'blki.inc'
      include 'blkj.inc'
      include 'blkjc.inc'
      COMPLEX S,RM,FT,ZIMP,RJ,FTD,TPD,ZIMPD,FLDJ,FLDJH,FLDJV
      CHARACTER*1 APT,AAPRM
      COMMON /RECIP/ FLDJ(IPY,ipi36),FLDJH(IPY,ipi36),FLDJV(IPY,ipi36)
      COMMON /FORSOL/ S(IP3,IP2),RM(IP2),RJ(IPY,IP2)
      COMMON /BKAP/ APT(IPZ,4,IPY),AAPRM(IPI)
C
      PI = 3.1415927
      RAD = 180./PI
      W = 2.*PI*FREK(LFRQ)
      WU = W*PI*4.E-07
      ELOG = 2.3025851
C
C     LOOP OVER RECEIVER NODES
C
      DO 10 IN = 1,NRC
       I = NRL(IN)
       NTO = NTAO(I) + NTR
       FT = RM(NTA(I)) + FPA(NTO)           ! TOTAL PARALLEL FIELD
       IF(IDX .EQ. 2)ZIMP = FT/AFT(IN)      ! TE -> E(par)/H(aux)
       IF(IDX .EQ. 3)ZIMP = AFT(IN)/FT      ! TM -> E(aux)/H(par)
C
C      LOOP OVER PARAMETERS
C
       DO 20 KP = 1,NPRM
        PFC = -1./y(LAPRM(KP)+1)
        FTD = FLDJ(IN,LAPRM(KP))            !PARALLEL FIELD JACOBIAN
        AFTD(KP) = FLDJH(IN,LAPRM(KP))      !HORIZONTAL AUX. JACOBIAN
        AFND(KP) = FLDJV(IN,LAPRM(KP))      !VERTICAL AUX. JACOBIAN
        IF(IDX .EQ. 2)THEN                            !TE MODE
         ZIMPD = (FTD - ZIMP*AFTD(KP))/AFT(IN)
         zimped(kp,in,lfrq) = ELOG*PFC*ZIMPD
         RTED(KP,IN,LFRQ) = PFC*(2./WU)*(REAL(ZIMP)*REAL(ZIMPD)
     &   + AIMAG(ZIMP)*AIMAG(ZIMPD))/RHTE(IN,LFRQ)
         AIOR = AIMAG(ZIMP)/REAL(ZIMP)
         PTED(KP,IN,LFRQ) = ELOG*PFC*RAD*(1./(1. + AIOR*AIOR))
     &   *(AIMAG(ZIMPD) - REAL(ZIMPD)*AIOR)/REAL(ZIMP)
         TPD = ELOG*PFC*(AFND(KP) - KZTE(IN,LFRQ)*AFTD(KP))/AFT(IN)
         KTED(KP,IN,LFRQ) = TPD
        ELSE                                          !TM MODE
         ZIMPD = (AFTD(KP) - ZIMP*FTD)/FT
         zimpmd(kp,in,lfrq) = ELOG*PFC*ZIMPD
         RTMD(KP,IN,LFRQ) = PFC*(2./WU)*(REAL(ZIMP)*REAL(ZIMPD)
     &   + AIMAG(ZIMP)*AIMAG(ZIMPD))/RHTM(IN,LFRQ)
         AIOR = AIMAG(ZIMP)/REAL(ZIMP)
         PTMD(KP,IN,LFRQ) = ELOG*PFC*RAD*(1./(1. + AIOR*AIOR))
     &   *(AIMAG(ZIMPD) - REAL(ZIMPD)*AIOR)/REAL(ZIMP)
         TPD = ELOG*PFC*(AFND(KP) - TZTM(IN,LFRQ)*AFTD(KP))/AFT(IN)
         TTMD(KP,IN,LFRQ) = TPD
        END IF
20     CONTINUE
10    CONTINUE
C
      RETURN
      END
C
